package hk.advanpro.android.sdk;


import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;


import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.ble.BLEAdvanproPiDevice;
import hk.advanpro.android.sdk.device.ble.BLEDevice;
import hk.advanpro.android.sdk.device.ble.BLEDeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanFilterCallback;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiCommandResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiDataEventResult;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;

/**
 * Instrumentation test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class ExampleInstrumentedTest {

    private Device device;

    @Test
    public void useAppContext() throws InterruptedException {

        final CountDownLatch latch = new CountDownLatch(1);
        // Context of the app under test.
        Context appContext = InstrumentationRegistry.getTargetContext();

        AdvanproAndroidSDK.init(appContext);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);

        manager.scan(30, new DeviceManagerScanFilterCallback<BLEDeviceScanResult>() {
            @Override
            public void onFilterScanning(BLEDeviceScanResult result) {
                device = result.create();
                manager.stop();
                latch.countDown();
            }

            @Override
            public boolean onFilter(BLEDeviceScanResult result) {
                return "93:27:70:04:00:0C".equals(result.getAddress());
            }

            @Override
            public void onStop() {

            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                manager.stop();
                latch.countDown();

            }
        });
        latch.await();
        final CountDownLatch latch2 = new CountDownLatch(1);

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

            }
        });

        latch2.await();

        final CountDownLatch latch3 = new CountDownLatch(1);

        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_RATE,
                new BLEAdvanproPiCommandParams("128"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch3.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                    }
                });

        latch3.await();

        final CountDownLatch latch4 = new CountDownLatch(1);

        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_RATE,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        assertEquals(result.getVal(), "128");
                        latch4.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                    }
                });


        latch4.await();

        final CountDownLatch latch5 = new CountDownLatch(1);

        final Map<String, String> params = new HashMap<String, String>() {{
            put("fsg", "2");
            put("spk", "2");
        }};

        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_RATEDIV,
                new BLEAdvanproPiCommandParams(params),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch5.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                    }
                });


        latch5.await();

        final CountDownLatch latch6 = new CountDownLatch(1);

        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_RATEDIV,
                new BLEAdvanproPiCommandParams("fsg1", "spk1"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        assertEquals(result.get("fsg"), "2");
                        assertEquals(result.get("spk"), "2");
                        latch6.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                    }
                });


        latch6.await();

        final CountDownLatch latch7 = new CountDownLatch(1);

        device.on(BLEDevice.EVENT_DEVICE_DATA, new DeviceEventCallback<BLEAdvanproPiDataEventResult>() {
            @Override
            public void onData(Device device, BLEAdvanproPiDataEventResult data) {
                assertNotNull(data.get("fsg1"));
                assertNotNull(data.get("fsg2"));
                assertNotNull(data.get("fsg3"));
                assertNotNull(data.get("spk1"));
                assertNotNull(data.get("spk2"));
                assertNotNull(data.get("acc1"));


                latch7.countDown();
            }
        });

        device.command(
                BLEAdvanproPiDevice.CMD_PI_START,
                new BLEAdvanproPiCommandParams("f=xxx.txt", "t=20170616140832"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                    }
                });


        latch7.await();


        final CountDownLatch latch8 = new CountDownLatch(1);

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch8.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

            }
        });

        latch8.await();
    }
}
