package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import com.alibaba.fastjson.JSON;

import org.apache.commons.lang3.StringUtils;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.DeviceSettingCallback;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiCommandParams;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiSettingParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.DeviceSettingResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiCommandResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiDataEventResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiGetSettingResult;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * Created by yejianfei on 2017/6/27.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEAdvanproPiDeviceTest {

    private static final String TAG = BLEAdvanproPiDeviceTest.class.getSimpleName();
    private static Context appContext;
    private static Device device;
    public static final String[] macAddressList = new String[]{"27:93:70:02:00:19", "27:93:70:04:00:0C"};

    @BeforeClass
    public static void begin() throws InterruptedException {
        // Context of the app under test.
        appContext = InstrumentationRegistry.getTargetContext();
        final CountDownLatch latch = new CountDownLatch(1);
        AConfig aConfig = new AConfig();
        aConfig.setDebugLog(true);
        AdvanproAndroidSDK.init(appContext, aConfig);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);

        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                if (macAddressList[1].equals(result.getRecord().getAddress())) {
                    device = result.create();
                    manager.stop();
                }
            }

            @Override
            public void onStop() {
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException error) {
                manager.stop();
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
        assertNotNull(device);
        final CountDownLatch latch2 = new CountDownLatch(1);

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                Log.d(TAG, "连接成功");
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();
    }

    private BLEAdvanproPiGetSettingResult settings;

    @Test
    public void test0000001_Setting() throws InterruptedException {

        final CountDownLatch latch3 = new CountDownLatch(1);

        device.readSetting(new DeviceSettingCallback<BLEAdvanproPiGetSettingResult>() {
            @Override
            public void onSuccess(Device device, BLEAdvanproPiGetSettingResult result) {
                Log.d(TAG, "获取设置成功");
                assertTrue(result != null);
                Log.d(TAG, JSON.toJSONString(result));
                settings = result;
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                Log.d(TAG, "获取设置失败");
                latch3.countDown();
                error.printStackTrace();
                assertTrue(false);

            }
        });
        latch3.await();

        assertTrue(settings != null);

        final CountDownLatch latch2 = new CountDownLatch(1);
        BLEAdvanproPiSettingParams params = new BLEAdvanproPiSettingParams();
        params.setRateDiv(settings.getRateDiv());
        params.setOutput(settings.getOutput());
        params.setSensorsActive(settings.getSensorsActive());
        params.setRange(settings.getRange());
        params.setRate(8);
        device.setting(params, new DeviceSettingCallback() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                latch2.countDown();
                Log.d(TAG, "设置成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch2.countDown();
                error.printStackTrace();
                assertTrue(false);
            }
        });
        latch2.await();

    }

    @Test
    public void test0001_EVENT_BATTERY_CHANGE() throws InterruptedException {
        final CountDownLatch countDownLatch = new CountDownLatch(3);
        device.on(BLEAdvanproPiDevice.EVENT_BATTERY_CHANGE, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                Log.d(TAG, "电量改变：" + device.getInfo(BLEAdvanproPiDevice.BLE_DEVICE_INFO_BATTERY));
                countDownLatch.countDown();
            }
        });
        countDownLatch.await();
    }


    /// 获取系统信息
    @Test
    public void test001_GET_INFO() throws InterruptedException {
        Log.d(TAG, "获取系统信息");
        final CountDownLatch latch00 = new CountDownLatch(1);
        device.command(BLEAdvanproPiDevice.CMD_PI_GET_INFO,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch00.countDown();


                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        latch00.await();

    }

    private static String[] sensorList;

    /// 获取传感器列表
    @Test
    public void test002_GET_SENSORLIST() throws InterruptedException {
        Log.d(TAG, "获取传感器列表");
        final CountDownLatch latch00 = new CountDownLatch(1);
        device.command(BLEAdvanproPiDevice.CMD_PI_GET_SENSORLIST,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        assertNotNull(result.getVals());
                        sensorList = result.getVals();
                        Log.d(TAG, result.toString());
                        latch00.countDown();

                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        latch00.await();
    }


    private String rate = "16";//主采样频率

    /// 设置主采样频率
    @Test
    public void test003_SET_RATE() throws InterruptedException {
        Log.d(TAG, "设置主采样频率");
        final CountDownLatch latch10 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_RATE,
                new BLEAdvanproPiCommandParams(rate),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch10.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        latch10.await();
    }

    /// 获取主采样频率
    @Test
    public void test004_GET_RATE() throws InterruptedException {
        Log.d(TAG, "获取主采样频率");
        final CountDownLatch latch11 = new CountDownLatch(1);

        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_RATE,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        assertEquals(result.getVal(), rate);
                        latch11.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });


        latch11.await();
    }

    public static String[] sensorActiveList = new String[]{"fss1", "spk1", "acc1"};

    /// 设置传感器为采集态
    @Test
    public void test005_SET_SENSORACTIVE() throws InterruptedException {
        Log.d(TAG, "设置传感器为采集态");
        final CountDownLatch latch30 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_SENSORACTIVE,
                new BLEAdvanproPiCommandParams("fss1", "spk2", "accX"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch30.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch30.await();
    }

    /// 获取标记为采集态的传感器列表
    @Test
    public void test006_GET_SENSORACTIVE() throws InterruptedException {
        Log.d(TAG, "获取标记为采集态的传感器列表");
        final CountDownLatch latch30 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_SENSORACTIVE,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch30.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch30.await();
    }


    /// 设置传感器的采样分频率
    @Test
    public void test007_SET_RATEDIV() throws InterruptedException {
        Log.d(TAG, "设置传感器的采样分频率");
        final CountDownLatch latch20 = new CountDownLatch(1);
        final Map<String, String> params = new HashMap<String, String>() {{
            put("fss", "2");
            put("spk", "2");
        }};
        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_RATEDIV,
                new BLEAdvanproPiCommandParams(params),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch20.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch20.await();
    }

    /// 获取传感器的采样分频率
    @Test
    public void test008_GET_RATEDIV() throws InterruptedException {
        Log.d(TAG, "获取传感器的采样分频率");
        final CountDownLatch latch21 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_RATEDIV,
                new BLEAdvanproPiCommandParams("fss", "spk"),
//                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
//                        assertEquals(result.get("fsg"), "2");
//                        assertEquals(result.get("spk"), "2");
                        latch21.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch21.await();
    }


    /// 设置部分传感器量程
    @Test
    public void test009_SET_RANGE() throws InterruptedException {
        Log.d(TAG, "设置部分传感器量程");
        final CountDownLatch latch50 = new CountDownLatch(1);
        final Map<String, String> params = new HashMap<String, String>() {{
            put("acc", "8");
            put("gyr", "500");
        }};
        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_RANGE,
                new BLEAdvanproPiCommandParams(params),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch50.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch50.await();
    }

    /// 获取传感器的采样分频数
    @Test
    public void test010_GET_RANGE() throws InterruptedException {
        Log.d(TAG, "获取传感器的采样分频数");
        final CountDownLatch latch51 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_RANGE,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch51.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch51.await();
    }

    /// 设置数据输出途径
    @Test
    public void test011_SET_OUTPUT() throws InterruptedException {
        Log.d(TAG, "设置数据输出途径");
        final CountDownLatch latch80 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_SET_OUTPUT,
                new BLEAdvanproPiCommandParams("ble"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch80.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch80.await();
    }

    /// 获取数据输出途径
    @Test
    public void test012_GET_OUTPUT() throws InterruptedException {
        Log.d(TAG, "获取数据输出途径");
        final CountDownLatch latch80 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_GET_OUTPUT,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch80.countDown();
                        Log.d(TAG, result.toString());
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {
                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch80.await();
    }


    /// 启动采样
    @Test
    public void test013_START() throws InterruptedException {
        final CountDownLatch latch90 = new CountDownLatch(1);
        device.on(BLEDevice.EVENT_DEVICE_DATA, new DeviceEventCallback<BLEAdvanproPiDataEventResult>() {
            @Override
            public void onData(Device device, BLEAdvanproPiDataEventResult data) {
                String[] names = data.getNames();
                for (int i = 0; i < names.length; i++) {
                    String value = data.get(names[i]);
                    Log.d(TAG, value);
                }

                if (latch90.getCount() == 10)
                    device.command(BLEAdvanproPiDevice.CMD_PI_GET_DATA_NAME,
                            new BLEAdvanproPiCommandParams(),
                            new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                                @Override
                                public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                                    assertEquals(result.getCode(), 0);
                                    assertNotNull(result.getVals());
                                    Log.d(TAG, StringUtils.join(result.getVals(), "\t"));
                                }

                                @Override
                                public void onError(Device device, DeviceCallbackException error) {

                                    error.printStackTrace();
                                    throw new RuntimeException(error);
                                }
                            });

                latch90.countDown();
            }
        });

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        device.command(
                BLEAdvanproPiDevice.CMD_PI_START,
                new BLEAdvanproPiCommandParams("f=12345"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, "启动采样");
                        latch90.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        latch90.await();
    }

    ///在设备处于采集状态时，获取 start 配置信息
    @Test
    public void test014_GET_START_CONFIG() {

        device.command(BLEAdvanproPiDevice.CMD_PI_GET_START_CONFIG,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        Log.d(TAG, "停止采样");
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());

                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });


    }

    /// 停止
    @Test
    public void test015_STOP() throws InterruptedException {

        final CountDownLatch latch7 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_STOP,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch7.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch7.await();
    }


    @Ignore
    public void test016_HELP() throws InterruptedException {
        final CountDownLatch latch16 = new CountDownLatch(1);
        device.command(
                BLEAdvanproPiDevice.CMD_PI_HELP,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        latch16.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
        latch16.await();

    }

    @AfterClass
    public static void end() throws InterruptedException {
        if (device == null)
            return;
        final CountDownLatch latch = new CountDownLatch(1);

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }
}
