package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.BatchDeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.enumeration.ConnectType;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEBatchDeviceConnectResult;
import hk.advanpro.android.sdk.device.result.ble.BLEConnectStateChangeResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_ADDRESS;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_BATTERY;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_CODE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_FIRMWARE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_HARDWARE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_ID;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_NAME;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_TYPE;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * 批量连接测试用例
 * Created by jalin on 2017/11/21.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEBatchDeviceConnectTest {

    private static final String TAG = BLEBatchDeviceConnectTest.class.getSimpleName();
    private static Context context;
    private static BLEBatchDeviceConnectResult result;


    @BeforeClass
    public static void begin() throws InterruptedException {
        context = InstrumentationRegistry.getTargetContext();
        AdvanproAndroidSDK.init(context);
        final AConfig config = AdvanproAndroidSDK.getConfig();
        //enable print debug log
        config.setDebugLog(true);
        AdvanproAndroidSDK.setConfig(config);
    }

    @Test
    public void testBatchConnect() throws InterruptedException {

        final CountDownLatch countDownLatch = new CountDownLatch(1);
        final BLEDeviceManager bleDeviceManager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        assertTrue(bleDeviceManager.isEnable(ConnectType.BLE));

        DeviceManager.DeviceConnection[] connection = new DeviceManager.DeviceConnection[]{
//                new DeviceManager.DeviceConnection("16:D1:A6:10:16:52"),
//                new DeviceManager.DeviceConnection("16:D2:B8:10:23:08")
                new DeviceManager.DeviceConnection("16:D2:B8:10:11:80"),
                new DeviceManager.DeviceConnection("16:D1:A6:10:09:98")
        };
        bleDeviceManager.connect(30, new BatchDeviceConnectCallback<BLEBatchDeviceConnectResult>() {
            @Override
            public void onCreated(DeviceManager.DeviceConnection deviceConnection, Device device) {
                Log.d(TAG, "创建设备成功：" + device.getInfo(BLEDevice.BLE_DEVICE_INFO_NAME));
                onCreateDevice(device);
            }

            @Override
            public void onConnectSuccess(DeviceManager.DeviceConnection deviceConnection, Device device) {
                Log.d(TAG, "连接设备成功：" + device.getInfo(BLEDevice.BLE_DEVICE_INFO_NAME));
            }

            @Override
            public void onFinish(BLEBatchDeviceConnectResult result) {
                Log.d(TAG, String.format("批量连接完成：总个数：%d,已创建个数：%d,已连接个数%d", result.getTotalCount(), result.getCreatedCount(),
                        result.getConnectCount()));
                assertTrue(result.getTotalCount() > 0);
                assertTrue(result.getCreatedCount() > 0);
                assertTrue(result.getConnectCount() > 0);
                BLEBatchDeviceConnectTest.this.result = result;
                countDownLatch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException e) {
                e.printStackTrace();
                countDownLatch.countDown();
            }
        }, connection);
        countDownLatch.await();

        final CountDownLatch countDownLatch2 = new CountDownLatch(1);
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Thread.sleep(30 * 1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                countDownLatch2.countDown();
            }
        }).start();
        countDownLatch2.await();
    }

    private void onCreateDevice(Device device) {
        device.on(BLEDevice.EVENT_READ_INFO, callback);
        device.on(BLEDevice.EVENT_READ_INFO_ERROR, errorCallback);
        device.on(BLEDevice.EVENT_CONNECT_STATE_CHANGE, connectChangeCallback);
    }

    DeviceEventCallback callback = new DeviceEventCallback() {
        @Override
        public void onData(Device device, DeviceEventResult data) {
            Log.d(TAG, String.format("设备信息：VV:%s\tID:%s\tTYPE:%s\tMAC:%s\tCODE:%s\tFIRMWARE:%s\tHARDWARE:%s\tBATTERY:%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                    device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                    device.getInfo(BLE_DEVICE_INFO_BATTERY)));
        }
    };
    DeviceEventCallback errorCallback = new DeviceEventCallback() {
        @Override
        public void onData(Device device, DeviceEventResult data) {
            Log.d(TAG, String.format("设备VV:%s，设备信息获取失败", device.getInfo(BLE_DEVICE_INFO_NAME)));

        }
    };
    DeviceEventCallback connectChangeCallback = new DeviceEventCallback<BLEConnectStateChangeResult>() {
        @Override
        public void onData(Device device, BLEConnectStateChangeResult data) {
            String name = device.getInfo(BLEDevice.BLE_DEVICE_INFO_NAME);
            switch (data.getConnectState()) {
                case BLEDevice.BLE_CONNECT_STATE_DISCONNECTED:
                    Log.d(TAG, String.format("设备%s断开成功  %d", name, data.getConnectState()));
                    break;
                case BLEDevice.BLE_CONNECT_STATE_DISCONNECTING:
                    Log.d(TAG, String.format("设备%s断开中  %d", name, data.getConnectState()));
                    break;
                case BLEDevice.BLE_CONNECT_STATE_CONNECTING:
                    Log.d(TAG, String.format("设备%s连接中  %d", name, data.getConnectState()));
                    break;
                case BLEDevice.BLE_CONNECT_STATE_CONNECTED:
                    Log.d(TAG, String.format("设备%s连接成功  %d", name, data.getConnectState()));
                    break;
            }
        }
    };

    @AfterClass
    public static void end() {
        assertNotNull(result);
        for (BLEBatchDeviceConnectResult.ConnectResult connectResult : result.getConnectResultList()) {
            if (connectResult.isCreated() && !connectResult.getDevice().isClose()) {
                connectResult.getDevice().disconnect(new DeviceDisconnectCallback() {
                    @Override
                    public void onSucceed(Device device) {

                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                    }
                });
            }

        }
    }
}
