package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import com.alibaba.fastjson.JSON;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.DeviceSettingCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltNotifyBreathCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltNotifySleepCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltNotifyStepCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltSectionStepCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltStepCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltStorageClearCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltStorageQueryCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltStorageStartCommandParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltStorageStopCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.DeviceSettingResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltHeartRateEventResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltNotifyBreathCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltNotifySleepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltNotifyStepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltNotifyStepEventResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltRealSensorEventResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltSectionStepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageClearCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageQueryCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageStartCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageStopCommandResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BleBeltNotifyBreathEventResult;
import hk.advanpro.android.sdk.device.result.ble.belt.BleBeltNotifySleepEventResult;

import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.*;
import static hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageStartCommandResult.STORAGE_STATUS_FAILURE;
import static hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageStartCommandResult.STORAGE_STATUS_HAVE_BEEN_STORAGE;
import static hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageStartCommandResult.STORAGE_STATUS_NOT_ENOUGH_STORAGE;
import static hk.advanpro.android.sdk.device.result.ble.belt.BLEBeltStorageStartCommandResult.STORAGE_STATUS_SUCCESS;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * Created by yejianfei on 2017/6/15.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEBeltDeviceTest {
    private static final String TAG = BLEBeltDeviceTest.class.getSimpleName();

    private static Context context;

    //C9:AE:9B:DE:02:A7
    private static final String address = "E4:93:57:1A:FA:09";//CE:F9:90:3A:4B:2E C6:4D:04:A0:8D:3F E4:83:B6:AA:48:3C EA:3D:8A:8A:AC:A8 E4:93:57:1A:FA:09

    private static Device device;

    @BeforeClass
    public static void begin() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);

        if (context == null) {
            context = InstrumentationRegistry.getTargetContext();
        }
        AdvanproAndroidSDK.init(context);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);

        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, String.format("%s [%s]", result.getRecord().getLocalName(), result.getBluetoothDevice().getAddress()));
                if (address.equals(result.getBluetoothDevice().getAddress())) {
                    device = result.create();
                    manager.stop();
                }
            }

            @Override
            public void onStop() {
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertNotNull(device);

        Log.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                Log.d("adv", "info");
                Log.d(TAG, String.format("设备信息：VV:%s\tID:%s\tTYPE:%s\tMAC:%s\tCODE:%s\tFIRMWARE:%s\tHARDWARE:%s\tBATTERY:%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                assertEquals(Enum.valueOf(DeviceType.class, device.getInfo(BLE_DEVICE_INFO_TYPE)), DeviceType.Belt);
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                latch2.countDown();
            }
        });

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                Log.d(TAG, "设备连接成功.");
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });


        latch2.await();
        assertEquals(device.isConnected(), true);
    }

    @Test
    public void test0000001_Setting() throws InterruptedException {

        final CountDownLatch latch3 = new CountDownLatch(1);

        device.readSetting(new DeviceSettingCallback<DeviceSettingResult>() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                Log.d(TAG, "获取设置成功");
                Log.d(TAG, JSON.toJSONString(result));
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                Log.d(TAG, "获取设置失败");
                latch3.countDown();
                error.printStackTrace();

            }
        });
        latch3.await();


        final CountDownLatch latch2 = new CountDownLatch(1);
        device.setting(null, new DeviceSettingCallback() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                latch2.countDown();

                Log.d(TAG, "设置成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch2.countDown();
                error.printStackTrace();
            }
        });
        latch2.await();

    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (device == null || !device.isConnected()) {
            return;
        }

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

    public static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private int heartEventCount = 0;//监听心跳事件个数
    private int sensorEventCount = 0;//监听传感器事件个数

    @Test
    public void testARealTimeEvent() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        DeviceEventCallback<BLEBeltHeartRateEventResult> callback = new DeviceEventCallback<BLEBeltHeartRateEventResult>() {
            @Override
            public void onData(Device device, BLEBeltHeartRateEventResult data) {
                heartEventCount++;
                Log.d(TAG, String.format("heart event %s", String.valueOf(data.getHeart())));
            }
        };
        device.on(BLEBeltDevice.EVENT_BELT_HEART_RATE, callback);
        DeviceEventCallback<BLEBeltRealSensorEventResult> callback2 = new DeviceEventCallback<BLEBeltRealSensorEventResult>() {
            @Override
            public void onData(Device device, BLEBeltRealSensorEventResult data) {
                sensorEventCount++;
                Log.d(TAG, String.format("real sensor event %s R0:%s,ACC_RANGE:%s,\n" +
                                "sensor:[%s],\n" +
                                "X     :[%s],\n" +
                                "Y     :[%s],\n" +
                                "Z     :[%s]",
                        sdf.format(data.getDataTime()),
                        String.valueOf(data.getRange()),
                        String.valueOf(data.getAccRange()),
                        doubleArrayToString(data.getSensorData()),
                        doubleArrayToString(data.getAccXData()),
                        doubleArrayToString(data.getAccYData()),
                        doubleArrayToString(data.getAccZData())));

                assertTrue(Math.abs(System.currentTimeMillis() - data.getDataTime().getTime()) <= 2 * 1000);//时间误差在2秒内

                assertTrue(data.getRange() > 0);
//                assertTrue(data.getAccRange() > 0);

            }
        };
        device.on(BLEBeltDevice.EVENT_BELT_REAL_SENSOR, callback2);
        Timer timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                latch.countDown();
            }
        }, 10 * 1000);
        latch.await();

//        assertTrue(heartEventCount > 0);
        assertTrue(sensorEventCount > 0);
        device.un(BLEBeltDevice.EVENT_BELT_HEART_RATE, callback);
        device.un(BLEBeltDevice.EVENT_BELT_REAL_SENSOR, callback2);
    }

    private int breathSN = 0;//呼吸个数

    @Test
    public void testZABreathAnalyzer() throws InterruptedException {

        final CountDownLatch latch = new CountDownLatch(11);
        device.on(BLEBeltDevice.EVENT_BELT_NOTIFY_BREATH, new DeviceEventCallback<BleBeltNotifyBreathEventResult>() {
            @Override
            public void onData(Device device, BleBeltNotifyBreathEventResult data) {
                Log.d(TAG, String.format("呼吸分析结果：%d [吸时长：%f,呼时长：%f，总时长：%f],[平均呼吸时长：%f,平均呼吸频率：%f]",
                        data.getBreathSN(), data.getInspiratoryTime(), data.getExspiratoryTime(), data.getBrthTime(),
                        data.getAvgBrthTime(), data.getAvgBrthFreq()));
                assertEquals(data.getBreathSN(), ++breathSN);
                assertTrue(data.getInspiratoryTime() > 0);
                assertTrue(data.getExspiratoryTime() > 0);
                assertTrue(data.getBrthTime() > 0);
                assertTrue(data.getAvgBrthTime() > 0);
                assertTrue(data.getAvgBrthFreq() > 0);
                latch.countDown();
            }
        });
        Log.d(TAG, "开启呼吸分析...");
        device.command(BLEBeltDevice.CMD_BELT_NOTIFY_BREATH, new BLEBeltNotifyBreathCommandParams(true), new DeviceCommandCallback<BLEBeltNotifyBreathCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltNotifyBreathCommandResult result) {
                Log.d(TAG, "开启呼吸分析" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch.countDown();
                Log.d(TAG, "请将呼吸带系在腰上，均匀呼吸10次...");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();

        final CountDownLatch latch2 = new CountDownLatch(1);
        Log.d(TAG, "关闭呼吸分析...");
        device.command(BLEBeltDevice.CMD_BELT_NOTIFY_BREATH, new BLEBeltNotifyBreathCommandParams(false), new DeviceCommandCallback<BLEBeltNotifyBreathCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltNotifyBreathCommandResult result) {
                Log.d(TAG, "关闭呼吸分析" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();

    }

    @Test
    public void testZBSleepAnalyzer() throws InterruptedException {
        //该测试需要花费较长时间
        final CountDownLatch latch = new CountDownLatch(2);
        device.on(BLEBeltDevice.EVENT_BELT_NOTIFY_SLEEP, new DeviceEventCallback<BleBeltNotifySleepEventResult>() {
            @Override
            public void onData(Device device, BleBeltNotifySleepEventResult data) {

                StringBuffer sb = new StringBuffer();
                for (int i = 0; i < data.getSleepGraph().length; i++) {
                    if (i > 0)
                        sb.append(",");
                    sb.append(String.valueOf(data.getSleepGraph()[i]));
                }
                Log.d(TAG, String.format("睡眠得分%d ，点间隔：%d ，条数：%d ，[%s]", data.getSleepQualityScore(), data.getIntervalOfPoints(), data.getSleepGraph().length, sb.toString()));
                latch.countDown();
            }
        });
        Log.d(TAG, "开启睡眠分析...");
        device.command(BLEBeltDevice.CMD_BELT_NOTIFY_SLEEP, new BLEBeltNotifySleepCommandParams(true, 10), new DeviceCommandCallback<BLEBeltNotifySleepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltNotifySleepCommandResult result) {
                Log.d(TAG, "开启睡眠分析" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch.countDown();

            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();

        final CountDownLatch latch2 = new CountDownLatch(1);
        Log.d(TAG, "关闭睡眠分析...");
        device.command(BLEBeltDevice.CMD_BELT_NOTIFY_SLEEP, new BLEBeltNotifySleepCommandParams(false), new DeviceCommandCallback<BLEBeltNotifySleepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltNotifySleepCommandResult result) {
                Log.d(TAG, "关闭睡眠分析" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();

    }

    private int realTimeStepCount = 0;

    /**
     * 测试实时推送的步数
     */
    @Test
    public void testBRealTimeStep() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        DeviceEventCallback<BLEBeltNotifyStepEventResult> callback = new DeviceEventCallback<BLEBeltNotifyStepEventResult>() {
            @Override
            public void onData(Device device, BLEBeltNotifyStepEventResult data) {
                realTimeStepCount++;
                Log.d(TAG, String.format("step event %s [walkStep:%d\trunSteps:%d\twalkSeconds:%d\trunSeconds:%d]", sdf.format(data.getDate()), data.getWalkSteps(), data.getRunSteps(), data.getWalkSeconds(), data.getRunSeconds()));

                assertTrue(Math.abs(System.currentTimeMillis() - data.getDate().getTime()) <= 10 * 1000);//时间误差在20秒内

                assertTrue(data.getWalkSteps() > 0 || data.getRunSteps() > 0);
                assertTrue(data.getWalkSeconds() > 0 || data.getRunSeconds() > 0);
            }
        };
        device.on(BLEBeltDevice.EVENT_BELT_NOTIFY_STEP, callback);

        //开启步数监听
        device.command(BLEBeltDevice.CMD_BELT_NOTIFY_STEP, new BLEBeltNotifyStepCommandParams(true), new DeviceCommandCallback<BLEBeltNotifyStepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltNotifyStepCommandResult result) {
                assertTrue(result.isSuccess());
                assertTrue(result.isEnable());
                Log.d(TAG, "开启实时步数推送成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        Timer timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                latch.countDown();
            }
        }, 30 * 1000);
        latch.await();
        assertTrue(realTimeStepCount > 0);

        final CountDownLatch latch2 = new CountDownLatch(1);
        //关闭步数监听
        device.command(BLEBeltDevice.CMD_BELT_NOTIFY_STEP, new BLEBeltNotifyStepCommandParams(false), new DeviceCommandCallback<BLEBeltNotifyStepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltNotifyStepCommandResult result) {
                assertTrue(result.isSuccess());
                assertTrue(!result.isEnable());
                Log.d(TAG, "关闭实时步数推送成功");
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
        device.un(BLEBeltDevice.EVENT_BELT_NOTIFY_STEP, callback);
    }

    private String doubleArrayToString(double[] data) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < data.length; i++) {
            if (i != 0)
                sb.append(",");
            sb.append(data[i]);
        }
        return sb.toString();
    }


    /**
     * 获取历史步数
     */
    @Test
    public void testCHisSteps() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        device.command(BLEBeltDevice.CMD_BELT_STEP, new BLEBeltStepCommandParams(new Date()), new DeviceCommandCallback<BLEBeltStepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltStepCommandResult result) {
                Log.d(TAG, "获取历史步数成功，size =" + result.getSteps().size());
                for (BLEBeltStepCommandResult.BeltStep step : result.getSteps()) {
                    Log.d(TAG, String.format("%d月%d日 走：%d , 跑：%d ,走时长：%d（秒） ,跑时长：%d（秒）", step.getMonth(), step.getDay(), step.getWalkSteps(), step.getRunSteps(), step.getWalkSeconds(), step.getRunSeconds()));
                }
                for (BLEBeltStepCommandResult.BeltStep step : result.getSteps()) {
                    assertTrue(step.getMonth() > 0 && step.getMonth() <= 12);
                    assertTrue(step.getDay() > 0 && step.getDay() <= 31);
                    assertTrue(step.getWalkSteps() >= 0 || step.getRunSteps() >= 0);
                    assertTrue(step.getWalkSeconds() >= 0 || step.getRunSeconds() >= 0);
                }

                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

    /**
     * 获取历史分段步数
     */
    @Test
    public void testDSectionSteps() throws InterruptedException, ParseException {
        final CountDownLatch latch = new CountDownLatch(1);
        Date date = new SimpleDateFormat("yyyy-MM-dd").parse("2018-01-01");
        device.command(BLEBeltDevice.CMD_BELT_SECTION_STEP, new BLEBeltSectionStepCommandParams(date, 1, 96), new DeviceCommandCallback<BLEBeltSectionStepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltSectionStepCommandResult result) {
                Log.d(TAG, "获取历史分段步数成功，size =" + result.getSteps().size());
                Collections.reverse(result.getSteps());
                for (BLEBeltSectionStepCommandResult.BeltStep step : result.getSteps()) {
                    Log.d(TAG, String.format("段号：%2d\t走：%d , 跑：%d ", step.getSection(), step.getWalkSteps(), step.getRunSteps()));
                }
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();
    }

    //=========数据存储============

    private int storageStage = 0;
    private long storageStartTime = 0;

    @Test
    public void testEStorageQuery() throws InterruptedException {
        Log.d(TAG, "发送查询数据存储命令...");
        final CountDownLatch latch = new CountDownLatch(1);
        device.command(BLEBeltDevice.CMD_BELT_STORAGE_QUERY, new BLEBeltStorageQueryCommandParams(), new DeviceCommandCallback<BLEBeltStorageQueryCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltStorageQueryCommandResult result) {
                switch (result.getStatus()) {
                    case STORAGE_STATUS_FAILURE:
                        Log.d(TAG, "非数据存储状态");
                        break;
                    case STORAGE_STATUS_HAVE_BEEN_STORAGE:
                        Log.d(TAG, "已经处于数据存储状态");
                        break;
                    default:
                        Log.d(TAG, "" + result.getStatus());

                }
                storageStage = result.getStatus();
                Log.d(TAG, String.format("存储开始时间：%s,已持续时长：%d（分钟），总时长：%d（分钟）", sdf.format(result.getStartDate()), (int) result.getDuration(), (int) result.getTotal()));

                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();
    }

    @Test
    public void testFStorageOpen() throws InterruptedException {
        assertTrue(storageStage == 0);
        Log.d(TAG, "发送开始数据存储命令...");
        final CountDownLatch latch1 = new CountDownLatch(1);
        device.command(BLEBeltDevice.CMD_BELT_STORAGE_START, new BLEBeltStorageStartCommandParams((short) 3), new DeviceCommandCallback<BLEBeltStorageStartCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltStorageStartCommandResult result) {
                switch (result.getStatus()) {
                    case STORAGE_STATUS_FAILURE:
                        Log.d(TAG, "开始数据存储失败");
                        break;
                    case STORAGE_STATUS_SUCCESS:
                        Log.d(TAG, "开始数据存储成功");
                        break;
                    case STORAGE_STATUS_HAVE_BEEN_STORAGE:
                        Log.d(TAG, "已经处于数据存储状态");
                        break;
                    case STORAGE_STATUS_NOT_ENOUGH_STORAGE:
                        Log.d(TAG, "数据存储空间不足");
                        break;
                    default:
                        Log.d(TAG, "" + result.getStatus());

                }
                Log.d(TAG, String.format("存储开始时间：%s,已持续时长：%d（分钟），总时长：%d（分钟）", sdf.format(result.getStartDate()), (int) result.getDurationMinute(), (int) result.getTotalMinute()));
                storageStartTime = result.getStartDate().getTime();
                assertTrue(result.isSuccess());
                assertTrue(Math.abs(System.currentTimeMillis() - result.getStartDate().getTime()) <= 5 * 1000);//时间误差在5秒内
                latch1.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch1.await();
    }

    @Test
    public void testGStorageStop() throws InterruptedException {
        //关闭数据存储
        Log.d(TAG, "发送关闭数据存储命令...");
        final CountDownLatch latch3 = new CountDownLatch(1);
        device.command(BLEBeltDevice.CMD_BELT_STORAGE_STOP, new BLEBeltStorageStopCommandParams(), new DeviceCommandCallback<BLEBeltStorageStopCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltStorageStopCommandResult result) {
                Log.d(TAG, "关闭数据存储" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch3.await();
    }

    @Test
    public void testHStorageClear() throws InterruptedException {
        //  清除数据存储
        Log.d(TAG, "发送清除数据命令...");
        final CountDownLatch latch3 = new CountDownLatch(1);
        device.command(BLEBeltDevice.CMD_BELT_STORAGE_CLEAR, new BLEBeltStorageClearCommandParams(), new DeviceCommandCallback<BLEBeltStorageClearCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEBeltStorageClearCommandResult result) {
                Log.d(TAG, "清除数据" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch3.await();
    }
}
