package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.os.Environment;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.DeviceUpgradeCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_ADDRESS;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_BATTERY;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_CODE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_FIRMWARE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_HARDWARE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_ID;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_MODE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_NAME;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_TYPE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.EVENT_READ_INFO;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * Created by jalin on 2017/8/22.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEBeltUpgradeTest {
    private static final String TAG = BLEBeltUpgradeTest.class.getSimpleName();
    private static Context context;
    private static final String deviceMac = "E4:83:B6:AA:48:3C";//E4:83:B6:AA:48:3C   C6:4D:04:A0:8D:3F
    private static Device device;
    private static File file;

    @BeforeClass
    public static void begin() throws InterruptedException {
        context = InstrumentationRegistry.getTargetContext();
        AdvanproAndroidSDK.init(context);
        String fileName = "BLTB.bin";
        file = new File(Environment.getExternalStorageDirectory().getAbsolutePath(), fileName);
        if (file == null || !file.exists())
            ALog.d(TAG, "请在手机SDK卡跟目录下放置升级固件包，名称 BLTB.bin");
        assertTrue(file != null && file.exists());


        final CountDownLatch latch = new CountDownLatch(1);
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        AdvanproAndroidSDK.init(context);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {

            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, String.format("%s [%s]", result.getRecord().getLocalName(), result.getBluetoothDevice().getAddress()));
                list.add(result);

                if (deviceMac.equals(result.getRecord().getAddress())) {
                    device = result.create();
                    manager.stop();
                }

            }

            @Override
            public void onStop() {
                ALog.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        //运行模式下升级
        if (DeviceMode.Application.name().equals(device.getInfo(BLE_DEVICE_INFO_MODE)))
            device.on(EVENT_READ_INFO, new DeviceEventCallback() {
                @Override
                public void onData(Device device, DeviceEventResult data) {
                    Log.d("adv", "info");
                    assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                    Log.d(TAG, String.format("设备信息：%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                            device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                            device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                    latch2.countDown();
                }
            });
        else
            latch2.countDown();
        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                ALog.d(TAG, "设备连接成功.");
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();

        assertEquals(device.isConnected(), true);
    }

    /**
     * 测试固件升级
     *
     * @throws InterruptedException
     */
    @Test(timeout = 30 * 60 * 1000)
    public void testBUpgrade() throws InterruptedException {
        ALog.d(TAG, "开始升级固件。。。");
        final CountDownLatch latch = new CountDownLatch(1);
        device.upgrade(file, new DeviceUpgradeCallback() {
            @Override
            public void onSuccess(Device device,DeviceCommandResult result)  {
                ALog.d(TAG, "升级固件成功");
                latch.countDown();
            }

            @Override
            public void onProgress(Device device,long total, long progress) {
                Log.d(TAG, String.format("Progress.. %d/%d", progress, total));
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                ALog.d(TAG, "升级固件失败");

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();
        assertTrue(DeviceMode.Application.name().equals(device.getInfo(BLE_DEVICE_INFO_MODE)));
    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (device == null || !device.isConnected()) {
            return;
        }

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device)  {
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }
}
