package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.enumeration.ConnectType;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.EVENT_READ_INFO;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_ADDRESS;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_BATTERY;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_CODE;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_FIRMWARE;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_HARDWARE;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_ID;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_NAME;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_TYPE;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * 测试用例
 * 设备搜索设备
 * 通过mac地址连接设备
 * 自动断开重连设备
 * Created by jalin on 2017/6/27.
 */
@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEDeviceConnect2Test {
    public static final int BLE_SCAN_TIME = 30 * 1000;//蓝牙扫描时间
    private static final String TAG = BLEDeviceConnect2Test.class.getSimpleName();
    private static Context context;
    private static String macAddress = "EA:3D:8A:8A:AC:A8";//16:D1:A6:03:30:5F 16:D1:A6:10:09:98  21:71:50:04:00:2F  E4:83:B6:AA:48:3C 27:93:70:02:00:19
    private static Device device;

    private static int count = 0;


    @BeforeClass
    public static void begin() throws InterruptedException {
        context = InstrumentationRegistry.getTargetContext();

        AdvanproAndroidSDK.init(context);
        final AConfig config = AdvanproAndroidSDK.getConfig();
        //enable print debug log
        config.setDebugLog(true);
        AdvanproAndroidSDK.setConfig(config);
    }

    @Test
    public void testConnect() throws InterruptedException {

        final BLEDeviceManager bleDeviceManager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        bleDeviceManager.isEnable(ConnectType.BLE);


        while (count++ < 10) {
            Log.d(TAG, String.format("===== 第%d次连接测试 ======", count));
            final CountDownLatch latch = new CountDownLatch(2);
            //扫描
            bleDeviceManager.instance(new DeviceManager.DeviceConnection(macAddress), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
                @Override
                public void onScanning(BLEDeviceScanResult result) {
                    assertTrue(result.getBluetoothDevice().getAddress().equals(macAddress));
                    device = result.create();
                    latch.countDown();
                }

                @Override
                public void onStop() {
                    latch.countDown();
                    bleDeviceManager.stop();
                }

                @Override
                public void onError(DeviceCallbackException error) {
                    error.printStackTrace();
                    throw new RuntimeException(error);
                }
            });
            latch.await();
            assertNotNull(device);

            final CountDownLatch latch2 = new CountDownLatch(2);
            device.on(EVENT_READ_INFO, new DeviceEventCallback() {
                @Override
                public void onData(Device device, DeviceEventResult data) {
                    latch2.countDown();
                    Log.d("adv", "info");
                    Log.d(TAG, String.format("设备信息：VV:%s\tID:%s\tTYPE:%s\tMAC:%s\tCODE:%s\tFIRMWARE:%s\tHARDWARE:%s\tBATTERY:%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                            device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                            device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                }
            });
            device.on(DefaultBLEDevice.EVENT_BATTERY_CHANGE, new DeviceEventCallback() {
                @Override
                public void onData(Device device, DeviceEventResult data) {
                    Log.d(TAG, "电量改变：" + device.getInfo(DefaultBLEDevice.BLE_DEVICE_INFO_BATTERY));
                }
            });

            //连接

            device.connect(1, new DeviceConnectCallback() {
                @Override
                public void onSucceed(Device device, DeviceConnectResult result) {
                    Log.d(TAG, String.format("第%d次连接成功", count));
                    latch2.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    error.printStackTrace();
                    throw new RuntimeException(error);
                }
            });
            latch2.await();

            //断开连接
            disconnect();
        }
    }

    public static void disconnect() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (device == null || !device.isConnected()) {
            return;
        }

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }


}
