package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.enumeration.ConnectType;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEConnectStateChangeResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.EVENT_READ_INFO;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_ADDRESS;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_BATTERY;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_CODE;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_FIRMWARE;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_HARDWARE;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_ID;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_NAME;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.BLE_DEVICE_INFO_TYPE;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * 测试用例
 * 设备搜索设备
 * 通过mac地址连接设备
 * 自动断开重连设备
 * Created by jalin on 2017/6/27.
 */
@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEDeviceConnectTest {
    public static final int BLE_SCAN_TIME = 30 * 1000;//蓝牙扫描时间
    private static final String TAG = BLEDeviceConnectTest.class.getSimpleName();
    private static Context context;
    private static String macAddress = "16:D2:B8:10:11:80";// 16:D2:B8:10:11:80 16:D1:A6:03:30:5F 16:D1:A6:10:09:98  21:71:50:04:00:2F  E4:83:B6:AA:48:3C 27:93:70:02:00:19
    // 05:71:50:04:00:1a
    private static Device device;

    @BeforeClass
    public static void begin() throws InterruptedException {

        context = InstrumentationRegistry.getTargetContext();

        final CountDownLatch latch = new CountDownLatch(2);
        AdvanproAndroidSDK.init(context);
        AConfig config = AdvanproAndroidSDK.getConfig();
        //enable print debug log
        config.setDebugLog(true);
        AdvanproAndroidSDK.setConfig(config);
        final BLEDeviceManager bleDeviceManager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        bleDeviceManager.isEnable(ConnectType.BLE);

        bleDeviceManager.instance(new DeviceManager.DeviceConnection(macAddress), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                assertTrue(result.getBluetoothDevice().getAddress().equals(macAddress));
                device = result.create();
                latch.countDown();
            }

            @Override
            public void onStop() {
                latch.countDown();
                bleDeviceManager.stop();
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();
        assertNotNull(device);

        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                latch2.countDown();
                Log.d("adv", "info");
                Log.d(TAG, String.format("设备信息：VV:%s\tID:%s\tTYPE:%s\tMAC:%s\tCODE:%s\tFIRMWARE:%s\tHARDWARE:%s\tBATTERY:%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
//                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
            }
        });
        device.on(DefaultBLEDevice.EVENT_BATTERY_CHANGE, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                Log.d(TAG, "电量改变：" + device.getInfo(DefaultBLEDevice.BLE_DEVICE_INFO_BATTERY));
            }
        });

        device.connect(30, new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                Log.d(TAG, "连接成功");
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (device == null || !device.isConnected()) {
            return;
        }

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

    /**
     * 测试重连设备是否成功
     *
     * @throws InterruptedException
     */
    @Test
    public void testBReConnect() throws InterruptedException {
        Log.d(TAG, "请将智能设备远离手机20米以外");
        final CountDownLatch latch = new CountDownLatch(1);
        device.on(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new DeviceEventCallback<BLEConnectStateChangeResult>() {
            @Override
            public void onData(Device device, BLEConnectStateChangeResult data) {
                Log.d(TAG, String.format("连接状态改变 connState=%d ", data.getConnectState()));
                if (data.getConnectState() == BLEDevice.BLE_CONNECT_STATE_DISCONNECTED) {
                    Log.d(TAG, "智能设备已断开连接");
                } else if (data.getConnectState() == BLEDevice.BLE_CONNECT_STATE_CONNECTED) {
                    Log.d(TAG, "智能设备重新连接成功");
                    latch.countDown();
                }
            }
        });

        latch.await();
        Thread.sleep(600 * 1000);
    }


}
