package hk.advanpro.android.sdk.device.ble;

import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.apache.commons.lang3.StringUtils;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.enumeration.ConnectType;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.ble.BLEConnectStateChangeResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiCommandResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiDataEventResult;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * Created by jalin on 2017/7/13.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEDeviceDoubleConnectTest {
    private static final String TAG = "BLEDeviceDoubleConnectTest";

    private static Device left;
    private static Device right;

    public static final String LeftMacAddress="16:D2:B8:10:11:80";
    public static final String RightMacAddress="16:D1:A6:10:09:98";

    @BeforeClass
    public static void begin() throws InterruptedException {
        AdvanproAndroidSDK.init(InstrumentationRegistry.getTargetContext());
        final CountDownLatch countDownLatch = new CountDownLatch(2);
        boolean enable = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class).isEnable(ConnectType.BLE);
        assertTrue(enable);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);

        manager.instance(new DeviceManager.DeviceConnection(LeftMacAddress), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "左脚搜索成功-" + result.getRecord().getLocalName() + " " + result.getRecord().getAddress());
                left = result.create();
                countDownLatch.countDown();
            }

            @Override
            public void onStop() {
                Log.d(TAG, "stop");
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                Log.d(TAG, "onError");
                countDownLatch.countDown();

            }
        });
        manager.instance(new DeviceManager.DeviceConnection(RightMacAddress), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "右脚搜索成功" + result.getRecord().getLocalName() + " " + result.getRecord().getAddress());
                right = result.create();
                countDownLatch.countDown();
            }

            @Override
            public void onStop() {
                Log.d(TAG, "stop");
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                Log.d(TAG, "onError");
                countDownLatch.countDown();

            }
        });
        countDownLatch.await();
        assertNotNull(left);
        assertNotNull(right);
        assertTrue(left != right);
    }

    @Test
    public void test00_connect() throws InterruptedException {

        left.on(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new DeviceEventCallback<BLEConnectStateChangeResult>() {
            @Override
            public void onData(Device device, BLEConnectStateChangeResult data) {
                switch (data.getConnectState()) {
                    case BLEDevice.BLE_CONNECT_STATE_DISCONNECTED:
                        Log.d(TAG, String.format("左脚-断开成功  %d", data.getConnectState()));
                        break;
                    case BLEDevice.BLE_CONNECT_STATE_DISCONNECTING:
                        Log.d(TAG, String.format("左脚-断开中  %d", data.getConnectState()));
                        break;
                    case BLEDevice.BLE_CONNECT_STATE_CONNECTING:
                        Log.d(TAG, String.format("左脚-连接中  %d", data.getConnectState()));
                        break;
                    case BLEDevice.BLE_CONNECT_STATE_CONNECTED:
                        Log.d(TAG, String.format("左脚-连接成功  %d", data.getConnectState()));
                        break;
                }
            }
        });
        right.on(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new DeviceEventCallback<BLEConnectStateChangeResult>() {
            @Override
            public void onData(Device device, BLEConnectStateChangeResult data) {
                switch (data.getConnectState()) {
                    case BLEDevice.BLE_CONNECT_STATE_DISCONNECTED:
                        Log.d(TAG, String.format("右脚-断开成功  %d", data.getConnectState()));
                        break;
                    case BLEDevice.BLE_CONNECT_STATE_DISCONNECTING:
                        Log.d(TAG, String.format("右脚-断开中  %d", data.getConnectState()));
                        break;
                    case BLEDevice.BLE_CONNECT_STATE_CONNECTING:
                        Log.d(TAG, String.format("右脚-连接中  %d", data.getConnectState()));
                        break;
                    case BLEDevice.BLE_CONNECT_STATE_CONNECTED:
                        Log.d(TAG, String.format("右脚-连接成功  %d", data.getConnectState()));
                        break;
                }
            }
        });
        final CountDownLatch count2 = new CountDownLatch(1);

        left.connect(30, new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                count2.countDown();
                Log.d(TAG, "左脚连接成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                count2.countDown();
            }
        });

        count2.await();
        final CountDownLatch count3 = new CountDownLatch(1);
        right.connect(30, new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                count3.countDown();
                Log.d(TAG, "右脚连接成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                count3.countDown();
            }
        });
        count3.await();

        assertTrue(left.isConnected());
        assertTrue(right.isConnected());
        Thread.sleep(60 * 1000);
    }

    @Test
    public void test01_disconnectAndConnect() throws InterruptedException {
        int count = 3;
        while (count-- > 0) {
            final CountDownLatch latch = new CountDownLatch(2);

            left.disconnect(new DeviceDisconnectCallback() {
                @Override
                public void onSucceed(Device device) {
                    Log.d(TAG, "左脚断开连接成功");
                    latch.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    latch.countDown();
                    error.printStackTrace();
                }
            });
            right.disconnect(new DeviceDisconnectCallback() {
                @Override
                public void onSucceed(Device device) {
                    Log.d(TAG, "右脚脚断开连接成功");
                    latch.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {

                    error.printStackTrace();
                    latch.countDown();
                }
            });
            latch.await();
            assertTrue(!left.isConnected());
            assertTrue(!right.isConnected());


            Log.d(TAG, "---搜索设备");

            final CountDownLatch countDownLatch = new CountDownLatch(2);
            boolean enable = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class).isEnable(ConnectType.BLE);
            BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);

            manager.instance(new DeviceManager.DeviceConnection("2C:93:70:04:00:0B"), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
                @Override
                public void onScanning(BLEDeviceScanResult result) {
                    Log.d(TAG, "右脚搜索成功" + result.getRecord().getLocalName() + " " + result.getRecord().getAddress());
                    left = result.create();
                    countDownLatch.countDown();

                }

                @Override
                public void onStop() {
                    Log.d(TAG, "stop");
                }

                @Override
                public void onError(DeviceCallbackException error) {
                    error.printStackTrace();
                    Log.d(TAG, "onError");

                }
            });
            manager.instance(new DeviceManager.DeviceConnection("1E:93:70:04:00:0A"), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
                @Override
                public void onScanning(BLEDeviceScanResult result) {
                    Log.d(TAG, "左脚搜索成功" + result.getRecord().getLocalName() + " " + result.getRecord().getAddress());
                    right = result.create();
                    countDownLatch.countDown();
                }

                @Override
                public void onStop() {
                    Log.d(TAG, "stop");
                }

                @Override
                public void onError(DeviceCallbackException error) {
                    error.printStackTrace();
                    Log.d(TAG, "onError");

                }
            });
            countDownLatch.await();
            assertNotNull(left);
            assertNotNull(right);
            assertTrue(left != right);

            left.on(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new DeviceEventCallback<BLEConnectStateChangeResult>() {
                @Override
                public void onData(Device device, BLEConnectStateChangeResult data) {
                    switch (data.getConnectState()) {
                        case BLEDevice.BLE_CONNECT_STATE_DISCONNECTED:
                            Log.d(TAG, String.format("左脚-断开成功  %d", data.getConnectState()));
                            break;
                        case BLEDevice.BLE_CONNECT_STATE_DISCONNECTING:
                            Log.d(TAG, String.format("左脚-断开中  %d", data.getConnectState()));
                            break;
                        case BLEDevice.BLE_CONNECT_STATE_CONNECTING:
                            Log.d(TAG, String.format("左脚-连接中  %d", data.getConnectState()));
                            break;
                        case BLEDevice.BLE_CONNECT_STATE_CONNECTED:
                            Log.d(TAG, String.format("左脚-连接成功  %d", data.getConnectState()));
                            break;
                    }
                }
            });
            right.on(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new DeviceEventCallback<BLEConnectStateChangeResult>() {
                @Override
                public void onData(Device device, BLEConnectStateChangeResult data) {
                    switch (data.getConnectState()) {
                        case BLEDevice.BLE_CONNECT_STATE_DISCONNECTED:
                            Log.d(TAG, String.format("右脚-断开成功  %d", data.getConnectState()));
                            break;
                        case BLEDevice.BLE_CONNECT_STATE_DISCONNECTING:
                            Log.d(TAG, String.format("右脚-断开中  %d", data.getConnectState()));
                            break;
                        case BLEDevice.BLE_CONNECT_STATE_CONNECTING:
                            Log.d(TAG, String.format("右脚-连接中  %d", data.getConnectState()));
                            break;
                        case BLEDevice.BLE_CONNECT_STATE_CONNECTED:
                            Log.d(TAG, String.format("右脚-连接成功  %d", data.getConnectState()));
                            break;
                    }
                }
            });


            Log.d(TAG, "---连接设备");
            final CountDownLatch count2 = new CountDownLatch(2);

            left.connect(30, new DeviceConnectCallback() {
                @Override
                public void onSucceed(Device device, DeviceConnectResult result) {
                    Log.d(TAG, "左脚连接成功 ");
                    count2.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    error.printStackTrace();
                    count2.countDown();
                }
            });

//            Thread.sleep(100);
            right.connect(30, new DeviceConnectCallback() {
                @Override
                public void onSucceed(Device device, DeviceConnectResult result) {
                    Log.d(TAG, "右脚连接成功");
                    count2.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    error.printStackTrace();
                    count2.countDown();
                }
            });
            count2.await();

            assertTrue(left.isConnected());
            assertTrue(right.isConnected());
            Thread.sleep(2000);
        }
    }

    @Test
    public void test02_getInfo() throws InterruptedException {
        Log.d(TAG, "获取系统信息");
        final CountDownLatch latch00 = new CountDownLatch(2);
        left.command(BLEAdvanproPiDevice.CMD_PI_GET_INFO,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch00.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {
                        latch00.countDown();
                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        right.command(BLEAdvanproPiDevice.CMD_PI_GET_INFO,
                new BLEAdvanproPiCommandParams(),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, result.toString());
                        latch00.countDown();


                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        latch00.countDown();
                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        latch00.await();

    }


    /// 启动采样
    @Ignore
    public void test03_START() throws InterruptedException {
        final CountDownLatch latch90 = new CountDownLatch(1000);
        left.on(BLEDevice.EVENT_DEVICE_DATA, new DeviceEventCallback<BLEAdvanproPiDataEventResult>() {
            @Override
            public void onData(Device device, BLEAdvanproPiDataEventResult data) {

                Log.d(TAG, String.format("left  data= %s ", StringUtils.join(data.getDatas(), ",")));
                latch90.countDown();
            }
        });

        left.command(
                BLEAdvanproPiDevice.CMD_PI_START,
                new BLEAdvanproPiCommandParams("f=12345"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, "左脚启动采样");
                        latch90.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        right.on(BLEDevice.EVENT_DEVICE_DATA, new DeviceEventCallback<BLEAdvanproPiDataEventResult>() {
            @Override
            public void onData(Device device, BLEAdvanproPiDataEventResult data) {

                Log.d(TAG, String.format("right data= %s ", StringUtils.join(data.getDatas(), ",")));
                latch90.countDown();
            }
        });

        right.command(
                BLEAdvanproPiDevice.CMD_PI_START,
                new BLEAdvanproPiCommandParams("f=12345"),
                new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {

                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        assertEquals(result.getCode(), 0);
                        Log.d(TAG, "右脚启动采样");
                        latch90.countDown();
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {

                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });

        latch90.await();
    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(2);

        left.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
                Log.d(TAG, "left disconnect success");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch.countDown();
                error.printStackTrace();
            }
        });
        right.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
                Log.d(TAG, "right disconnect success");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

                latch.countDown();
                error.printStackTrace();
            }
        });
        latch.await();
    }


}
