package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.net.ParseException;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.json.JSONException;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.params.ble.BLEEnableWritePermissionCommandParams;
import hk.advanpro.android.sdk.device.params.ble.BLESendPermissionCodeCommandParams;
import hk.advanpro.android.sdk.device.params.ble.BLEWriteDeviceCodeCommandParams;
import hk.advanpro.android.sdk.device.params.ble.BLEWriteDeviceIdCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleReadIdCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleWriteIdCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.BLEEnableWritePermissionCommandResult;
import hk.advanpro.android.sdk.device.result.ble.BLESendPermissionCodeCommandResult;
import hk.advanpro.android.sdk.device.result.ble.BLEWriteDeviceCodeCommandResult;
import hk.advanpro.android.sdk.device.result.ble.BLEWriteDeviceIdCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleReadIdCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleWriteIdCommandResult;

import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.*;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;


/**
 * 测试用例
 * 获取设备写入权限
 * 写入设备ID
 * 设备VV号
 * ID号
 * Created by jalin on 2017/6/23.
 */
@RunWith(AndroidJUnit4.class)
public class BLEDevicePermissionTest {
    private static final String TAG = BLEDevicePermissionTest.class.getSimpleName();
    private static final String deviceVV = "72U6PZ";//J1SBJD T9SI3L 72U6PZ
    private static Context context;
    private static Device device;


    @BeforeClass
    public static void begin() throws InterruptedException, ParseException, JSONException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (context == null) {
            context = InstrumentationRegistry.getTargetContext();
        }
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        AdvanproAndroidSDK.init(context);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result)  {
                Log.d(TAG, String.format("Name:%s ,Address:%s, Type:%s", result.getRecord().getLocalName(),
                        result.getRecord().getAddress(),
                        result.getRecord().getManufacturer().getType()));

                if (deviceVV.equals(result.getRecord().getLocalName())) {
                    device = result.create();
                    manager.stop();
                }
                list.add(result);
            }

            @Override
            public void onStop()  {
                ALog.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new DeviceEventCallback() {
            @Override
            public void onData(Device device,DeviceEventResult data)  {
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                Log.d(TAG, String.format("设备信息：%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                latch2.countDown();
            }
        });

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device,DeviceConnectResult result)  {
                latch2.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();
        assertEquals(device.isConnected(), true);


        /**
         * 获取设备写入权限
         * 第一步：获取权限码
         * 第二步：写入验证码
         * 写入权限会持续到本次蓝牙连接断开，即
         * 主机断开与设备的蓝牙连接后，权限即失效，如果想再次写入管
         * 控数据，则连接后需再次执行开启权限流程。
         */
        final CountDownLatch latch3 = new CountDownLatch(2);
        ALog.d(TAG, "正在获取写入权限...");
        device.command(CMD_ENABLE_WRITE_PERMISSION, new BLEEnableWritePermissionCommandParams(), new DeviceCommandCallback<BLEEnableWritePermissionCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEEnableWritePermissionCommandResult result)  {
                assertTrue(result.getTable() == 1 || result.getTable() == 2);
                assertTrue(result.getRow() >= 0);
                assertTrue(result.getA() >= 0);
                assertTrue(result.getB() >= 0);
                assertTrue(result.getC() >= 0);
                assertTrue(result.getD() >= 0);
                latch3.countDown();
                int permissionCore = PermissionCores.getPermissionCore(result.getTable(), result.getRow(), result.getA(), result.getB(), result.getC(), result.getD());

                device.command(CMD_SEND_PERMISSION_CODE, new BLESendPermissionCodeCommandParams(permissionCore), new DeviceCommandCallback<BLESendPermissionCodeCommandResult>() {

                    @Override
                    public void onSuccess(Device device,BLESendPermissionCodeCommandResult result)  {
                        ALog.d(TAG, "获取写入权限" + (result.isSuccess() ? "成功" : "失败"));
                        assertTrue(result.isSuccess());
                        latch3.countDown();
                    }

                    @Override
                    public void onError(Device device,DeviceCallbackException error) {
                        error.printStackTrace();
                        throw new RuntimeException(error);
                    }
                });
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch3.await();


    }

    /**
     * 测试计算校验码
     */
    @Test
    public void testPermissionCore() {
        int result = PermissionCores.getPermissionCore(1, 0, 1, 1, 1, 1);
        assertEquals(result, 556);
    }

    /**
     * 写入设备VV Code
     * 设备需要保持充电状态
     */
    @Test
    public void testWriteDeviceCode() throws InterruptedException {
        final String newVV = deviceVV;
        final CountDownLatch latch = new CountDownLatch(1);
        ALog.d(TAG, "正在写入VV号...");
        device.command(CMD_WRITE_DEVICE_CODE, new BLEWriteDeviceCodeCommandParams(newVV), new DeviceCommandCallback<BLEWriteDeviceCodeCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEWriteDeviceCodeCommandResult result)  {
                ALog.d(TAG, "写入VV号" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();
    }


    /**
     * 写入设备ID
     * 设备需要保持充电状态
     */
    @Test
    public void testWriteDeviceID() throws InterruptedException {
        final String newDevID = device.getInfo(BLE_DEVICE_INFO_ID);
        final CountDownLatch latch = new CountDownLatch(1);
        ALog.d(TAG, "正在设备ID...");
        device.command(CMD_WRITE_DEVICE_ID, new BLEWriteDeviceIdCommandParams(newDevID), new DeviceCommandCallback<BLEWriteDeviceIdCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEWriteDeviceIdCommandResult result)  {
                ALog.d(TAG, "写入设备ID" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();

    }

    /**
     * 测试读写ID命令
     *
     * @throws InterruptedException
     */
    @Test
    public void testWriteId() throws InterruptedException {
        final long writeId = Long.parseLong("70695011502527258");//需要写入的id
        final CountDownLatch latch = new CountDownLatch(1);
        ALog.d(TAG, "正在写入ID..." + writeId);
        device.command(BLEInsoleDevice.CMD_INSOLE_WRITE_ID, new BLEInsoleWriteIdCommandParams(writeId), new DeviceCommandCallback<BLEInsoleWriteIdCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEInsoleWriteIdCommandResult result)  {
                ALog.d(TAG, "写入设备ID" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
        final CountDownLatch latch2 = new CountDownLatch(1);

        ALog.d(TAG, "正在读取ID...");
        device.command(BLEInsoleDevice.CMD_INSOLE_READ_ID, new BLEInsoleReadIdCommandParams(), new DeviceCommandCallback<BLEInsoleReadIdCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEInsoleReadIdCommandResult result)  {
                ALog.d(TAG, "读取ID成功 :" + result.getId());
                assertEquals(result.getId(), writeId);
                latch2.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device)  {
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }


    /**
     * 获取权限验证码
     * Created by jalin on 2017/6/23.
     */
    public static class PermissionCores {

        /**
         * 获取权限验证码
         * Val=a×n0+b×n1+c×n2+d×n3
         *
         * @param table
         * @param row
         * @param a
         * @param b
         * @param c
         * @param d
         * @return
         */
        public static int getPermissionCore(int table, int row, int a, int b, int c, int d) {
            if (table <= 0 || table > TABLES.length) {
                return 0;
            }
            int[] tows = TABLES[table - 1][row];
            int n0 = tows[0];
            int n1 = tows[1];
            int n2 = tows[2];
            int n3 = tows[3];
            int result = a * n0 + b * n1 + c * n2 + d * n3;
            return result;
        }

        static final int[][][] TABLES = new int[][][]{{
                {103, 212, 12, 229},
                {0, 202, 184, 92},
                {112, 126, 155, 91},
                {223, 221, 147, 244},
                {28, 22, 171, 83},
                {211, 132, 246, 233},
                {71, 214, 9, 219},
                {234, 14, 244, 85},
                {135, 41, 167, 251},
                {41, 27, 50, 56},
                {231, 6, 55, 91},
                {202, 227, 249, 201},
                {65, 143, 10, 88},
                {41, 31, 119, 254},
                {244, 249, 77, 100},
                {49, 39, 174, 250},
                {61, 196, 46, 62},
                {45, 88, 26, 9},
                {143, 18, 178, 249},
                {67, 98, 147, 104},
                {16, 65, 246, 99},
                {66, 198, 60, 146},
                {44, 244, 35, 216},
                {58, 239, 6, 197},
                {94, 239, 124, 190},
                {223, 140, 70, 163},
                {89, 232, 214, 3},
                {221, 101, 91, 119},
                {104, 169, 12, 8},
                {204, 113, 43, 108},
                {189, 161, 123, 32},
                {180, 221, 118, 62},
                {194, 211, 179, 37},
                {170, 45, 241, 46},
                {93, 185, 9, 15},
                {128, 20, 171, 23},
                {9, 246, 115, 246},
                {175, 58, 155, 169},
                {161, 148, 215, 195},
                {107, 203, 69, 232},
                {50, 199, 6, 116},
                {37, 246, 101, 247},
                {122, 234, 88, 206},
                {125, 48, 20, 206},
                {166, 60, 236, 75},
                {178, 54, 228, 41},
                {86, 103, 110, 145},
                {206, 24, 59, 36},
                {51, 150, 242, 64},
                {166, 230, 154, 240},
                {23, 171, 0, 79},
                {174, 194, 227, 162},
                {96, 118, 16, 68},
                {234, 121, 11, 189},
                {141, 163, 235, 86},
                {164, 16, 242, 208},
                {104, 225, 81, 115},
                {68, 241, 52, 8},
                {113, 169, 178, 225},
                {151, 242, 157, 37},
                {238, 230, 234, 233},
                {158, 83, 156, 208},
                {166, 216, 43, 132},
                {130, 135, 14, 64},
                {129, 119, 102, 25},
                {213, 196, 65, 201},
                {250, 155, 182, 157},
                {81, 106, 157, 24},
                {59, 205, 209, 42},
                {145, 38, 152, 97},
                {99, 152, 80, 183},
                {150, 187, 146, 23},
                {176, 191, 125, 17},
                {236, 45, 105, 164},
                {218, 5, 159, 158},
                {8, 100, 28, 67},
                {78, 219, 33, 194},
                {235, 26, 191, 126},
                {28, 11, 158, 215},
                {206, 156, 61, 238},
                {93, 28, 96, 11},
                {36, 123, 166, 109},
                {149, 63, 45, 63},
                {4, 156, 73, 78},
                {11, 117, 109, 96},
                {133, 18, 70, 237},
                {93, 228, 105, 76},
                {215, 158, 201, 88},
                {105, 119, 205, 169},
                {40, 50, 201, 131},
                {14, 198, 151, 165},
                {199, 94, 25, 69},
                {56, 156, 130, 181},
                {40, 186, 251, 148},
                {232, 232, 33, 90},
                {31, 202, 7, 107},
                {42, 150, 139, 66},
                {47, 194, 82, 181},
                {119, 170, 56, 253},
                {18, 113, 141, 182},
                {119, 124, 202, 126},
                {83, 91, 165, 181},
                {133, 12, 0, 151},
                {77, 90, 74, 149},
                {118, 98, 213, 205},
                {203, 214, 55, 228},
                {49, 23, 153, 31},
                {218, 81, 144, 161},
                {70, 71, 67, 133},
                {68, 23, 12, 14},
                {188, 135, 180, 104},
                {238, 12, 114, 17},
                {156, 147, 124, 61},
                {133, 56, 79, 126},
                {105, 56, 133, 111},
                {248, 172, 64, 89},
                {134, 247, 108, 189},
                {21, 147, 143, 104},
                {183, 227, 82, 53},
                {167, 134, 54, 129},
                {118, 12, 227, 139},
                {160, 134, 162, 236},
                {3, 202, 120, 8},
                {213, 185, 252, 144},
                {66, 132, 126, 200},
                {108, 93, 196, 133},
                {127, 26, 176, 54},
                {137, 197, 44, 133},
                {49, 123, 97, 213},
                {133, 227, 87, 8},
                {47, 4, 241, 237},
                {51, 127, 199, 118},
                {125, 35, 119, 27},
                {111, 90, 181, 175},
                {248, 15, 7, 44},
                {80, 220, 229, 58},
                {29, 165, 36, 234},
                {5, 34, 97, 205},
                {41, 96, 88, 14},
                {2, 145, 17, 215},
                {144, 106, 27, 204},
                {252, 172, 96, 81},
                {181, 153, 99, 78},
                {220, 182, 82, 86},
                {140, 105, 205, 25},
                {62, 171, 153, 169},
                {99, 66, 39, 249},
                {108, 241, 8, 125},
                {75, 75, 85, 232},
                {12, 243, 80, 39},
                {76, 51, 184, 158},
                {49, 101, 232, 176},
                {78, 116, 203, 18},
                {173, 205, 193, 90},
                {246, 105, 195, 29},
                {174, 62, 246, 206},
                {5, 73, 124, 75},
                {163, 69, 166, 229},
                {70, 239, 176, 61},
                {65, 68, 114, 49},
                {129, 200, 173, 208},
                {109, 209, 95, 75},
                {217, 46, 160, 77},
                {43, 216, 224, 148},
                {199, 228, 147, 209},
                {82, 72, 200, 189},
                {63, 103, 216, 174},
                {250, 73, 108, 177},
                {185, 186, 87, 197},
                {115, 135, 216, 59},
                {54, 16, 6, 121},
                {19, 190, 32, 177},
                {234, 15, 98, 18},
                {11, 159, 246, 203},
                {179, 146, 151, 232},
                {13, 63, 153, 140},
                {156, 177, 159, 157},
                {53, 224, 145, 38},
                {114, 254, 207, 121},
                {46, 177, 108, 186},
                {171, 123, 101, 105},
                {41, 72, 178, 231},
                {22, 150, 44, 151},
                {213, 71, 141, 134},
                {221, 106, 251, 98},
                {210, 107, 119, 160},
                {54, 139, 175, 113},
                {24, 175, 212, 253},
                {92, 70, 47, 252},
                {75, 76, 14, 190},
                {17, 169, 49, 55},
                {78, 213, 171, 43},
                {27, 87, 76, 90},
                {139, 153, 95, 253},
                {128, 221, 84, 252},
                {12, 122, 79, 21},
                {134, 120, 92, 17},
                {26, 111, 30, 232},
                {68, 104, 110, 250},
                {10, 100, 75, 254},
                {245, 79, 76, 133},
                {97, 159, 231, 79},
                {94, 105, 223, 51},
                {40, 125, 31, 206},
                {178, 124, 147, 131},
                {80, 47, 39, 33},
                {250, 4, 216, 54},
                {37, 7, 95, 171},
                {217, 29, 54, 159},
                {14, 75, 44, 50},
                {166, 175, 180, 184},
                {139, 110, 26, 109},
                {86, 219, 19, 140},
                {188, 141, 59, 206},
                {239, 32, 196, 165},
                {222, 239, 41, 164},
                {124, 140, 128, 178},
                {253, 154, 168, 168},
                {8, 92, 134, 226},
                {59, 182, 190, 130},
                {36, 253, 178, 63},
                {119, 183, 112, 55},
                {66, 33, 158, 231},
                {12, 33, 30, 57},
                {10, 194, 127, 228},
                {219, 95, 123, 80},
                {158, 38, 52, 69},
                {80, 225, 63, 7},
                {190, 118, 128, 16},
                {43, 201, 206, 203},
                {199, 179, 85, 29},
                {151, 247, 139, 76},
                {245, 209, 75, 1},
                {188, 95, 84, 12},
                {238, 185, 244, 79},
                {221, 100, 122, 219},
                {160, 212, 217, 159},
                {230, 216, 22, 68},
                {102, 86, 18, 165},
                {168, 194, 166, 36},
                {40, 25, 28, 85},
                {124, 80, 85, 128},
                {200, 250, 198, 128},
                {61, 155, 43, 145},
                {42, 0, 39, 91},
                {220, 46, 121, 53},
                {146, 167, 112, 13},
                {14, 95, 77, 214},
                {170, 89, 117, 34},
                {9, 45, 5, 243},
                {16, 19, 97, 190},
                {189, 215, 9, 154},
                {29, 1, 175, 241},
                {110, 162, 181, 80},
                {248, 150, 223, 160},
                {49, 232, 59, 108}}, {

                //table 2
                {119, 137, 204, 170},
                {81, 33, 171, 228},
                {162, 193, 22, 97},
                {80, 135, 55, 71},
                {90, 86, 46, 83},
                {141, 237, 88, 164},
                {84, 196, 231, 62},
                {103, 162, 153, 224},
                {127, 174, 97, 173},
                {244, 110, 181, 253},
                {159, 185, 126, 95},
                {51, 98, 190, 196},
                {170, 146, 128, 111},
                {101, 108, 136, 62},
                {206, 116, 71, 138},
                {17, 221, 102, 12},
                {120, 168, 201, 11},
                {20, 192, 186, 199},
                {92, 224, 141, 13},
                {1, 41, 149, 224},
                {148, 104, 87, 138},
                {246, 198, 134, 215},
                {190, 136, 145, 28},
                {214, 241, 227, 251},
                {36, 254, 140, 198},
                {134, 16, 0, 37},
                {178, 136, 229, 184},
                {251, 112, 149, 109},
                {62, 222, 86, 73},
                {41, 79, 105, 169},
                {220, 90, 218, 170},
                {33, 82, 167, 188},
                {54, 237, 0, 194},
                {49, 41, 235, 1},
                {237, 155, 97, 176},
                {210, 53, 52, 242},
                {106, 110, 96, 43},
                {189, 30, 177, 7},
                {52, 91, 132, 103},
                {195, 26, 91, 62},
                {134, 82, 195, 78},
                {223, 175, 189, 100},
                {15, 12, 160, 243},
                {132, 91, 134, 153},
                {223, 90, 178, 11},
                {214, 191, 231, 42},
                {222, 193, 27, 253},
                {128, 220, 135, 166},
                {188, 229, 142, 125},
                {9, 115, 168, 8},
                {67, 169, 36, 106},
                {138, 84, 244, 103},
                {67, 164, 151, 94},
                {41, 125, 127, 122},
                {44, 151, 35, 173},
                {182, 27, 184, 29},
                {27, 231, 137, 118},
                {218, 173, 252, 56},
                {34, 35, 70, 138},
                {217, 147, 189, 40},
                {23, 222, 106, 232},
                {143, 194, 173, 130},
                {198, 178, 61, 127},
                {49, 63, 56, 28},
                {156, 171, 48, 56},
                {181, 203, 14, 9},
                {202, 40, 204, 113},
                {65, 236, 68, 85},
                {226, 218, 24, 190},
                {56, 125, 189, 52},
                {96, 141, 154, 179},
                {215, 47, 227, 232},
                {187, 94, 148, 212},
                {120, 138, 19, 113},
                {145, 248, 48, 235},
                {148, 46, 163, 73},
                {119, 36, 166, 63},
                {226, 247, 238, 88},
                {122, 15, 30, 208},
                {190, 83, 181, 84},
                {59, 247, 225, 42},
                {83, 212, 237, 24},
                {67, 82, 225, 107},
                {206, 120, 75, 179},
                {75, 54, 201, 140},
                {186, 142, 64, 26},
                {153, 212, 102, 184},
                {30, 31, 66, 146},
                {197, 58, 58, 113},
                {193, 253, 192, 87},
                {59, 154, 174, 16},
                {79, 47, 89, 220},
                {49, 62, 160, 170},
                {141, 84, 5, 88},
                {53, 97, 87, 88},
                {31, 132, 50, 86},
                {180, 168, 111, 245},
                {184, 170, 249, 192},
                {225, 160, 236, 34},
                {91, 56, 247, 65},
                {154, 139, 254, 4},
                {169, 45, 83, 31},
                {90, 89, 247, 210},
                {71, 206, 6, 246},
                {194, 26, 195, 3},
                {198, 115, 155, 98},
                {156, 63, 153, 213},
                {145, 211, 251, 214},
                {34, 57, 206, 111},
                {237, 248, 169, 140},
                {225, 51, 202, 220},
                {20, 14, 96, 196},
                {134, 32, 238, 175},
                {47, 209, 214, 198},
                {118, 119, 159, 10},
                {151, 110, 25, 56},
                {17, 200, 91, 62},
                {171, 251, 145, 46},
                {28, 185, 4, 114},
                {65, 191, 49, 217},
                {81, 28, 138, 151},
                {75, 167, 98, 75},
                {1, 129, 94, 180},
                {173, 204, 220, 71},
                {188, 203, 186, 102},
                {86, 121, 203, 148},
                {201, 91, 240, 218},
                {208, 233, 57, 3},
                {68, 231, 115, 20},
                {133, 199, 82, 38},
                {251, 45, 251, 238},
                {57, 211, 227, 140},
                {93, 55, 208, 15},
                {198, 70, 17, 133},
                {214, 33, 12, 139},
                {2, 54, 174, 65},
                {195, 165, 164, 76},
                {4, 212, 5, 4},
                {161, 71, 50, 238},
                {204, 220, 147, 128},
                {57, 221, 23, 37},
                {59, 7, 48, 218},
                {35, 239, 89, 189},
                {95, 199, 156, 3},
                {96, 110, 191, 58},
                {60, 58, 164, 182},
                {70, 67, 66, 229},
                {179, 101, 209, 248},
                {194, 161, 209, 60},
                {17, 107, 185, 110},
                {179, 248, 52, 148},
                {159, 206, 221, 248},
                {80, 7, 169, 15},
                {85, 38, 181, 95},
                {240, 147, 30, 18},
                {1, 81, 132, 231},
                {84, 52, 210, 23},
                {80, 35, 135, 252},
                {141, 110, 51, 227},
                {121, 176, 49, 243},
                {243, 139, 240, 35},
                {37, 74, 116, 233},
                {19, 103, 180, 13},
                {96, 177, 102, 100},
                {113, 158, 114, 160},
                {87, 64, 17, 199},
                {79, 243, 167, 52},
                {101, 235, 241, 219},
                {21, 205, 124, 240},
                {248, 45, 12, 73},
                {72, 146, 169, 186},
                {95, 170, 154, 14},
                {58, 32, 54, 222},
                {139, 150, 251, 55},
                {48, 229, 195, 20},
                {152, 245, 159, 131},
                {141, 253, 172, 106},
                {100, 204, 78, 143},
                {120, 93, 24, 88},
                {232, 177, 247, 31},
                {78, 241, 227, 53},
                {182, 31, 6, 47},
                {202, 213, 160, 118},
                {11, 74, 125, 193},
                {76, 153, 144, 16},
                {46, 167, 226, 213},
                {226, 49, 28, 105},
                {122, 36, 75, 47},
                {156, 75, 153, 241},
                {172, 42, 126, 37},
                {24, 8, 146, 157},
                {244, 111, 248, 37},
                {97, 39, 185, 147},
                {42, 92, 184, 63},
                {183, 137, 79, 14},
                {215, 11, 2, 137},
                {5, 23, 59, 93},
                {226, 182, 80, 54},
                {0, 116, 178, 187},
                {6, 103, 76, 199},
                {36, 76, 32, 201},
                {100, 176, 83, 150},
                {34, 250, 232, 200},
                {204, 133, 234, 35},
                {6, 119, 66, 229},
                {45, 255, 40, 83},
                {18, 112, 165, 141},
                {124, 191, 125, 118},
                {84, 150, 32, 15},
                {126, 3, 150, 91},
                {193, 109, 200, 3},
                {43, 144, 3, 76},
                {39, 179, 69, 232},
                {5, 243, 180, 199},
                {245, 19, 124, 81},
                {139, 108, 46, 18},
                {75, 23, 137, 126},
                {60, 0, 228, 51},
                {149, 233, 195, 186},
                {99, 30, 248, 130},
                {156, 205, 145, 2},
                {242, 34, 95, 70},
                {110, 194, 77, 178},
                {188, 120, 64, 248},
                {69, 212, 240, 142},
                {174, 208, 105, 199},
                {17, 196, 190, 119},
                {13, 83, 254, 96},
                {66, 175, 197, 156},
                {88, 183, 24, 141},
                {19, 75, 62, 70},
                {22, 33, 8, 145},
                {236, 242, 155, 185},
                {0, 215, 235, 71},
                {203, 71, 170, 98},
                {124, 144, 77, 229},
                {148, 27, 22, 246},
                {170, 241, 213, 57},
                {211, 16, 232, 2},
                {140, 255, 137, 31},
                {170, 91, 219, 93},
                {141, 59, 29, 126},
                {85, 4, 197, 217},
                {8, 67, 67, 62},
                {48, 14, 177, 233},
                {160, 200, 121, 5},
                {199, 34, 97, 89},
                {221, 102, 196, 10},
                {28, 180, 127, 147},
                {31, 14, 233, 18},
                {135, 231, 195, 206},
                {211, 165, 144, 214},
                {30, 75, 10, 143},
                {188, 130, 244, 205},
                {8, 116, 209, 214},
                {47, 237, 127, 192}}};

    }

}
