package hk.advanpro.android.sdk.device.ble;

import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;

import static junit.framework.Assert.assertNotNull;

/**
 * Created by jalin on 2017/7/12.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEDeviceScanTest {
    private static final String TAG = "BLEDeviceScanTest";
    private static BLEDeviceManager bleDeviceManager;

    @BeforeClass
    public static void begin() {
        AdvanproAndroidSDK.init(InstrumentationRegistry.getTargetContext());
        bleDeviceManager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
    }

    @Test
    public void scan() throws InterruptedException {

        final CountDownLatch countDownLatch = new CountDownLatch(4);

        //过滤设备
        bleDeviceManager.scan(60, DeviceType.Insole, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "0 -" + result.getRecord().getLocalName());
            }

            @Override
            public void onStop() {
                countDownLatch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException error) {

            }
        });

        bleDeviceManager.instance(new DeviceManager.DeviceConnection("16:D2:B8:10:11:80"), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "1-" + result.getRecord().getLocalName());
                countDownLatch.countDown();
            }

            @Override
            public void onStop() {
                Log.d(TAG, "stop");
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                Log.d(TAG, "onError");

            }
        });
        bleDeviceManager.instance(new DeviceManager.DeviceConnection("16:D1:A6:10:09:98"), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "2-" + result.getRecord().getLocalName());
                countDownLatch.countDown();
            }

            @Override
            public void onStop() {
                Log.d(TAG, "stop");
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                Log.d(TAG, "onError");

            }
        });

        bleDeviceManager.instance(new DeviceManager.DeviceConnection("16:D1:A6:10:09:98"), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "3-" + result.getRecord().getLocalName());
                countDownLatch.countDown();
            }

            @Override
            public void onStop() {
                Log.d(TAG, "stop");
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                Log.d(TAG, "onError");

            }
        });

        countDownLatch.await();


    }

    BLEDeviceScanResult scanResult;

    @Test
    public void instance() throws InterruptedException {
        final CountDownLatch countDownLatch = new CountDownLatch(2);

        bleDeviceManager.instance(new DeviceManager.DeviceConnection("16:D1:A6:10:09:98"), new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, "3-" + result.getRecord().getLocalName());
                countDownLatch.countDown();
                scanResult = result;
            }

            @Override
            public void onStop() {
                Log.d(TAG, "stop");
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                Log.d(TAG, "onError");

            }
        });
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Thread.sleep(30 * 1000);
                    //休眠30秒
                    countDownLatch.countDown();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }).start();

        countDownLatch.await();
        assertNotNull(scanResult);

    }


}
