package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import com.alibaba.fastjson.JSON;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.core.utils.DateUtils;
import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.DeviceSettingCallback;
import hk.advanpro.android.sdk.device.callback.MainThreadDeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.MainThreadDeviceEventCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.params.DeviceSettingParams;
import hk.advanpro.android.sdk.device.params.ble.belt.BLEBeltStartCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleStartCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleStopCommandParams;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiSettingParams;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiStartParams;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.DeviceSettingResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceInfoResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiGetSettingResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_TYPE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.EVENT_READ_INFO;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * 设备数据 采集 启动停止
 * 作者：jalin on 2018/1/2 19:06
 * 邮箱：zhouxiaolin@advanpro.com
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEDeviceStartStopSettingsTest {
    private static final String TAG = BLEDeviceStartStopSettingsTest.class.getSimpleName();
    private static final String[] namesOrAddress =
            new String[]{"72U6PZ", //鞋垫
                    "27:93:70:04:00:0C",// 开发版
                    "C9:AE:9B:DE:02:A7"};//呼吸带

    private static int deviceType = 0;
    private static Context context;
    private static Device device;
    private static DeviceSettingResult settingResult;

    @BeforeClass
    public static void begin() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);

        if (context == null) {
            context = InstrumentationRegistry.getTargetContext();
        }
        AdvanproAndroidSDK.init(context, new AConfig());
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {

            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, String.format("%s [%s]", result.getRecord().getLocalName(), result.getBluetoothDevice().getAddress()));
                list.add(result);

                if (namesOrAddress[deviceType].equals(result.getRecord().getLocalName()) || namesOrAddress[deviceType].equals(result.getRecord().getAddress())) {

                    device = result.create();
                    manager.stop();
                }

            }

            @Override
            public void onStop() {
                ALog.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        // 设置连接后获取状态
        device.setReadStateWhenConnected(true);

        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new MainThreadDeviceEventCallback() {
            @Override
            public void onMainThreadData(Device device, DeviceEventResult data) {
                Log.d("adv", "info");
                BLEDeviceInfoResult info = (BLEDeviceInfoResult) data;
                for (Map.Entry<String, Object> entries : info.entrySet()) {
                    Log.d(TAG, entries.getKey() + ":" + entries.getValue());
                }
                latch2.countDown();
            }
        });

        device.connect(new MainThreadDeviceConnectCallback() {
            @Override
            public void onMainThreadSucceed(Device device, DeviceConnectResult result) {
                latch2.countDown();
                ALog.d(TAG, "设备连接成功.");
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                latch2.countDown();
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });


        latch2.await();

        assertEquals(device.isConnected(), true);


        // 监听 EVENT_DEVICE_DATA  事件
        // 监听数据采集事件
        device.on(BLEDevice.EVENT_DEVICE_DATA, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult eventResult) {
                // 开启设备数据采集device.isStarted() == true , 数据回调

                // 智能鞋垫(DeviceType.Insole)数据类型 ：步数（BLEInsoleRealStepEventResult），步态（BLEInsoleRealGaitEventResult），三轴数据（BLEInsoleCollectDataEventResult）
                // 智能呼吸带(DeviceType.Belt)数据类型：心率（BLEBeltHeartRateEventResult），三轴及电阻（BLEBeltRealSensorEventResult），呼吸分析（BleBeltNotifyBreathEventResult）
                // 开发版（DeviceType.SDP）数据类型：BLEAdvanproPiDataEventResult

                assertNotNull(eventResult);
                assertTrue(device.isStarted());
                // 输出数据
                ALog.d(TAG, eventResult.getClass().getSimpleName() + "-->" + JSON.toJSONString(eventResult));
            }
        });
    }

    @Test
    public void test001_getSetting() throws InterruptedException {
        final CountDownLatch latch3 = new CountDownLatch(1);

        device.readSetting(new DeviceSettingCallback<DeviceSettingResult>() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                Log.d(TAG, "获取设置成功");
                Log.d(TAG, JSON.toJSONString(result));
                assertNotNull(device.getSetting());
                settingResult = result;
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                Log.d(TAG, "获取设置失败");
                latch3.countDown();
                error.printStackTrace();

            }
        });
        latch3.await();
        assertNotNull(settingResult);
    }

    @Test
    public void test002_setSetting() {
        DeviceSettingParams params = null;

        if (DeviceType.SDP.name().equals(device.getInfo(BLE_DEVICE_INFO_TYPE))) {

            // 开发版
            BLEAdvanproPiSettingParams params1 = new BLEAdvanproPiSettingParams();
            ArrayList<String> strings = new ArrayList<>();
            strings.add("ble");
            params1.setOutput(strings);
            params1.setRate(8);

            BLEAdvanproPiGetSettingResult oldSettings = (BLEAdvanproPiGetSettingResult) settingResult;
            if (oldSettings.getRange() != null)
                params1.setRange(oldSettings.getRange());
            if (oldSettings.getRateDiv() != null)
                params1.setRateDiv(oldSettings.getRateDiv());

            params = params1;
        } else if (DeviceType.Insole.name().equals(device.getInfo(BLE_DEVICE_INFO_TYPE))) {
            params = null;
        } else if (DeviceType.Belt.name().equals(device.getInfo(BLE_DEVICE_INFO_TYPE))) {
            params = null;
        }

        device.setting(params, new DeviceSettingCallback() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                Log.d(TAG, "设置成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                Log.d(TAG, "设置失败");
            }
        });

    }


    //开启
    @Test
    public void test003_Start() throws InterruptedException {
        assertTrue(!device.isStarted());
        DeviceCommandParams params = null;
        if (DeviceType.Insole.name().equals(device.getInfo(BLE_DEVICE_INFO_TYPE))) {
            // 鞋垫
            int rate = 8;
            params = new BLEInsoleStartCommandParams(rate);
        } else if (DeviceType.Belt.name().equals(device.getInfo(BLE_DEVICE_INFO_TYPE))) {
            // 1 秒钟
            params = new BLEBeltStartCommandParams(0, (short) 1);
        } else if (DeviceType.SDP.name().equals(device.getInfo(BLE_DEVICE_INFO_TYPE))) {
            // 配置参数
            String name = DateUtils.formatDate(new Date(), "yyyyMMddHHmmss");
            params = new BLEAdvanproPiStartParams(new Random().nextInt() + ".txt", name);
        }

        assertNotNull(params);

        final CountDownLatch latch = new CountDownLatch(1);
        device.start(params, new DeviceCommandCallback() {
            @Override
            public void onSuccess(Device device, DeviceCommandResult result) {
                Log.d(TAG, "开启数据采集" + (device.isStarted() ? "成功" : "失败"));

                assertTrue(device.isStarted());
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                Log.d(TAG, "开启数据采集" + (device.isStarted() ? "成功" : "失败"));
                assertTrue(device.isStarted());
                latch.countDown();

            }
        });
        latch.await();
        assertTrue(device.isStarted());
    }

    @Test
    public void test004_DataEvent() throws InterruptedException {
        assertTrue(device.isStarted());
        final CountDownLatch latch3 = new CountDownLatch(1);
        new Thread(new Runnable() {
            @Override
            public void run() {
                // 休息 60秒
                try {
                    Thread.sleep(60 * 1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                latch3.countDown();

            }
        }).start();
        latch3.await();
        assertTrue(device.isStarted());

    }

    @Test
    public void test005_Stop() throws InterruptedException {
        assertTrue(device.isStarted());
        final CountDownLatch latch3 = new CountDownLatch(1);
        Log.d(TAG, String.format("正在发送关闭数据采集命令"));
        device.stop(new BLEInsoleStopCommandParams(), new DeviceCommandCallback() {
            @Override
            public void onSuccess(Device device, DeviceCommandResult result) {
                Log.d(TAG, "关闭数据采集" + (!device.isStarted() ? "成功" : "失败"));
                assertTrue(!device.isStarted());
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch3.countDown();

            }
        });

        latch3.await();
        assertTrue(!device.isStarted());

        //等待5秒
        try {
            Thread.sleep(5 * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    // 断开设备
    @AfterClass
    public static void end() throws InterruptedException {
        if (device == null || !device.isConnected()) {
            return;
        }

        final CountDownLatch latch = new CountDownLatch(1);
        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }


}
