package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;


import com.alibaba.fastjson.JSON;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.commons.BatchExecutor;
import hk.advanpro.android.sdk.commons.DeviceCommandSyncTask;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceSettingCallback;
import hk.advanpro.android.sdk.device.callback.MainThreadDeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.MainThreadDeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.MainThreadDeviceEventCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleClearAllCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleDisableAccCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleEnableAccCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleHistoryStepCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleFirmwareUpgradeProtectCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleStartCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleStopCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleSyncTimeCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.DeviceSettingResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceInfoResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleClearAllCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectDataEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectStartOrStopCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleDisableAccCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleEnableAccCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleGetSettingResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleHistoryStepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealAccEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealGaitEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealStepEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleSyncTimeCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BleInsoleFirmwareUpgradeProtectCommandResult;

import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.*;
import static junit.framework.Assert.*;

/**
 * 鞋垫实时数据、时间同步、清除命令、三轴检查命令 、现场保护测试用例
 * Created by yejianfei on 2017/6/13.
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEInsoleDeviceTest {

    private static final String TAG = BLEInsoleDeviceTest.class.getSimpleName();

    private static final SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

    private static Context context;

    private static final String name = "72U6PZ";//72U6PZ  J1SBJD  T9SI3L J1SBJD Q36T3R CI4UNV FFPL7E

    private static Device device;

    @BeforeClass
    public static void begin() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);

        if (context == null) {
            context = InstrumentationRegistry.getTargetContext();
        }
        AdvanproAndroidSDK.init(context, new AConfig());
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {

            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, String.format("%s [%s]", result.getRecord().getLocalName(), result.getBluetoothDevice().getAddress()));
                list.add(result);

                if (name.equals(result.getRecord().getLocalName())) {
                    device = result.create();
                    manager.stop();
                }

            }

            @Override
            public void onStop() {
                ALog.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new MainThreadDeviceEventCallback() {
            @Override
            public void onMainThreadData(Device device, DeviceEventResult data) {
                Log.d("adv", "info");
                BLEDeviceInfoResult info = (BLEDeviceInfoResult) data;
                for (Map.Entry<String, Object> entries : info.entrySet()) {
                    Log.d(TAG, entries.getKey() + ":" + entries.getValue());
                }
                Object o = info.get(BLE_DEVICE_INFO_BATTERY);


                assertNotNull(data);
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                Log.d(TAG, String.format("设备信息：VV:%s\tID:%s\tTYPE:%s\tMAC:%s\tCODE:%s\tFIRMWARE:%s\tHARDWARE:%s\tBATTERY:%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                assertEquals(Enum.valueOf(DeviceType.class, device.getInfo(BLE_DEVICE_INFO_TYPE)), DeviceType.Insole);
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                latch2.countDown();
            }
        });

        device.connect(new MainThreadDeviceConnectCallback() {
            @Override
            public void onMainThreadSucceed(Device device, DeviceConnectResult result) {
                latch2.countDown();
                ALog.d(TAG, "设备连接成功.");
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });


        latch2.await();

        assertEquals(device.isConnected(), true);
    }

    @Test
    public void test0000001_Setting() throws InterruptedException {

        final CountDownLatch latch3 = new CountDownLatch(1);

        device.readSetting(new DeviceSettingCallback<DeviceSettingResult>() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                Log.d(TAG, "获取设置成功");
                Log.d(TAG, JSON.toJSONString(result));
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                Log.d(TAG, "获取设置失败");
                latch3.countDown();
                error.printStackTrace();

            }
        });
        latch3.await();


        final CountDownLatch latch2 = new CountDownLatch(1);
        device.setting(null, new DeviceSettingCallback() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                latch2.countDown();

                Log.d(TAG, "设置成功");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch2.countDown();
                error.printStackTrace();
            }
        });
        latch2.await();

    }

    @Ignore
    public void testADisconnectAndConnect() throws InterruptedException {
        ALog.d(TAG, "开始断开连接.");
        final CountDownLatch latch = new CountDownLatch(1);
        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                ALog.d(TAG, "断开连接设备.");
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();


        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                ALog.d("adv", "info");
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                ALog.d(TAG, String.format("设备信息：VV:%s\tID:%s\tTYPE:%s\tMAC:%s\tCODE:%s\tFIRMWARE:%s\tHARDWARE:%s\tBATTERY:%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                assertEquals(Enum.valueOf(DeviceType.class, device.getInfo(BLE_DEVICE_INFO_TYPE)), DeviceType.Insole);
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                latch2.countDown();
            }
        });

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                latch2.countDown();
                ALog.d(TAG, "设备连接成功.");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });


        latch2.await();

    }

    @AfterClass
    public static void end() throws InterruptedException {
        if (device == null || !device.isConnected()) {
            return;
        }

        final CountDownLatch latch = new CountDownLatch(1);
        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

    @Test
    public void testBEventRealGait() throws InterruptedException {
        ALog.d(TAG, "请按压鞋垫的脚后压力传感器.");
        final CountDownLatch latch = new CountDownLatch(1);
        final MainThreadDeviceEventCallback<BLEInsoleRealGaitEventResult> cb = new MainThreadDeviceEventCallback<BLEInsoleRealGaitEventResult>() {

            @Override
            public void onMainThreadData(Device device, BLEInsoleRealGaitEventResult data) {
                Calendar calendar = Calendar.getInstance();
                assertEquals(data.getYear(), calendar.get(Calendar.YEAR));
                assertEquals(data.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(data.getDay(), calendar.get(Calendar.DATE));
                assertTrue(data.getTouchA());
                latch.countDown();
            }
        };

        device.on(BLEInsoleDevice.EVENT_INSOLE_REAL_GAIT, cb);
        latch.await();
        device.un(BLEInsoleDevice.EVENT_INSOLE_REAL_GAIT, cb);

    }

    @Test
    public void testCEventRealStep() throws InterruptedException {
        final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        ALog.d(TAG, "请穿上鞋垫，走五步以上.");
        final CountDownLatch latch = new CountDownLatch(10);
        final MainThreadDeviceEventCallback<BLEInsoleRealStepEventResult> cb = new MainThreadDeviceEventCallback<BLEInsoleRealStepEventResult>() {

            @Override
            public void onMainThreadData(Device device, BLEInsoleRealStepEventResult data) {
                Log.d(TAG, String.format("实时步数 %s [走步数：%d ,跑步数：%d,走步时长：%d,跑步时长：%d,\n" +
                                "走跑状态：%d,步态：%d,着地方式:%d,足内旋：%d,冲击力等级：%d,时长：%d]",
                        sdf.format(data.getDate()), data.getWalkStep(), data.getRunStep(), data.getWalkDuration(), data.getRunDuration(),
                        data.getIsRun(), data.getGait(), data.getTouchdown(), data.getFootType(), data.getImpact(), data.getStepDur()
                ));
                Calendar calendar = Calendar.getInstance();
                assertEquals(data.getYear(), calendar.get(Calendar.YEAR));
                assertEquals(data.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(data.getDay(), calendar.get(Calendar.DATE));

                assertTrue(data.getWalkStep() > 0 || data.getRunStep() > 0);
                assertTrue(data.getWalkDuration() > 0 || data.getRunDuration() > 0);
                assertTrue(data.getGait() >= 0 && data.getGait() <= 15);
                assertTrue(data.getImpact() >= 0);
                assertTrue(data.getIsRun() == 0 || data.getIsRun() == 1);
                assertTrue(data.getFootType() >= 0 && data.getFootType() <= 3);
                assertTrue(data.getTouchdown() >= 0 && data.getTouchdown() <= 3);
                assertTrue(data.getStepDur() >= 0);

                latch.countDown();
            }
        };

        device.on(BLEInsoleDevice.EVENT_INSOLE_REAL_STEP, cb);
        latch.await();
        device.un(BLEInsoleDevice.EVENT_INSOLE_REAL_STEP, cb);
    }


    /**
     * 时间同步,返回存储时间，当日步数
     */
    @Test
    public void testDEventSetTime() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(2);
        ALog.d(TAG, "正在同步硬件时间...");
        //监听实时步数
        final MainThreadDeviceEventCallback<BLEInsoleRealStepEventResult> cb = new MainThreadDeviceEventCallback<BLEInsoleRealStepEventResult>() {
            @Override
            public void onMainThreadData(Device device, BLEInsoleRealStepEventResult data) {
                Calendar calendar = Calendar.getInstance();
                assertEquals(data.getYear(), calendar.get(Calendar.YEAR));
                assertEquals(data.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(data.getDay(), calendar.get(Calendar.DATE));
                Log.d(TAG, String.format("%d年%d月%d日，走步数：%d ,走步时间：%d,跑步数：%d，跑步时间:%d", data.getYear(), data.getMonth(), data.getDay(), data.getWalkStep(), data.getWalkDuration(), data.getRunStep(), data.getRunDuration()));
                latch.countDown();
            }
        };
        device.on(BLEInsoleDevice.EVENT_INSOLE_REAL_STEP, cb);
        Thread.sleep(20000);

        //发送时间同步命令
        device.command(BLEInsoleDevice.CMD_INSOLE_SYNC_TIME, new BLEInsoleSyncTimeCommandParams(new Date()), new MainThreadDeviceCommandCallback<BLEInsoleSyncTimeCommandResult>() {
            @Override
            public void onMainThreadSuccess(Device device, BLEInsoleSyncTimeCommandResult result) {
                assertNotNull(result);
                StringBuffer stringBuffer = new StringBuffer();
                for (BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog storageLog : result.getLogs()) {
                    stringBuffer.append(String.format("(%d-%02d-%02d)", storageLog.getYear(), storageLog.getMonth(), storageLog.getDay()));
                }
                ALog.d(TAG, "时间同步成功。存储时间：" + stringBuffer.toString());
                for (BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog storageLog : result.getLogs()) {
                    assertTrue(storageLog.getYear() >= 2017);
                    assertTrue(storageLog.getMonth() >= 0 && storageLog.getMonth() <= 12);
                    assertTrue(storageLog.getDay() > 0 && storageLog.getDay() <= 31);
                }

                latch.countDown();
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);

            }
        });
        latch.await();
        device.un(BLEInsoleDevice.EVENT_INSOLE_REAL_STEP, cb);
    }

    //同步线程回调
    @Test
    public void test004_testSyncCommand() throws DeviceCallbackException, InterruptedException {
        BLEInsoleSyncTimeCommandResult result = (BLEInsoleSyncTimeCommandResult)
                DeviceCommandSyncTask.create(device, BLEInsoleDevice.CMD_INSOLE_SYNC_TIME, new BLEInsoleSyncTimeCommandParams(new Date()))
                        .execute()
                        .getResult();
        assertNotNull(result);
        StringBuffer stringBuffer = new StringBuffer();
        for (BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog storageLog : result.getLogs()) {
            stringBuffer.append(String.format("(%d-%02d-%02d)", storageLog.getYear(), storageLog.getMonth(), storageLog.getDay()));
        }
        ALog.d(TAG, "时间同步成功。存储时间：" + stringBuffer.toString());
        for (BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog storageLog : result.getLogs()) {
            assertTrue(storageLog.getYear() >= 2017);
            assertTrue(storageLog.getMonth() >= 0 && storageLog.getMonth() <= 12);
            assertTrue(storageLog.getDay() > 0 && storageLog.getDay() <= 31);
        }
    }

    @Test
    public void test004_BatchExecutor() {
        final List<DeviceCommandSyncTask<BLEInsoleClearAllCommandResult>> tasks = new ArrayList<>();
        tasks.add(DeviceCommandSyncTask.create(device, BLEInsoleDevice.CMD_INSOLE_SYNC_TIME, new BLEInsoleSyncTimeCommandParams(new Date())));
        tasks.add(DeviceCommandSyncTask.create(device, BLEInsoleDevice.CMD_INSOLE_HISTORY_STEP, new BLEInsoleHistoryStepCommandParams(new Date())));
        BatchExecutor executor = new BatchExecutor(2);
        for (final DeviceCommandSyncTask task : tasks) {
            executor.execute(task);
        }
        try {
            executor.waitComplete();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        for (DeviceCommandSyncTask task : tasks) {
            assertTrue(task.isSuccess());
            switch (task.getCommandKey()) {
                case BLEInsoleDevice.CMD_INSOLE_SYNC_TIME:
                    StringBuffer stringBuffer = new StringBuffer();
                    if (task.isSuccess()) {
                        BLEInsoleSyncTimeCommandResult result = (BLEInsoleSyncTimeCommandResult) task.getResult();
                        for (BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog storageLog : result.getLogs()) {
                            stringBuffer.append(String.format("(%d-%02d-%02d)", storageLog.getYear(), storageLog.getMonth(), storageLog.getDay()));
                        }
                        ALog.d(TAG, "时间同步成功。存储时间：" + stringBuffer.toString());
                        for (BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog storageLog : result.getLogs()) {
                            assertTrue(storageLog.getYear() >= 2017);
                            assertTrue(storageLog.getMonth() >= 0 && storageLog.getMonth() <= 12);
                            assertTrue(storageLog.getDay() > 0 && storageLog.getDay() <= 31);
                        }
                    } else {
                        ALog.d(TAG, "时间同步失败");
                    }
                    break;
                case BLEInsoleDevice.CMD_INSOLE_HISTORY_STEP:
                    if (task.isSuccess()) {
                        BLEInsoleHistoryStepCommandResult result = (BLEInsoleHistoryStepCommandResult) task.getResult();

                        Calendar calendar = Calendar.getInstance();
                        assertEquals(result.getMonth(), calendar.get(Calendar.MONTH) + 1);
                        assertEquals(result.getDay(), calendar.get(Calendar.DAY_OF_MONTH));

                        Log.d(TAG, String.format("获取历史步数成功。%d月%d日 走步数：%d，跑步数：%d，走步时间：%d，跑步时间：%d", result.getMonth(), result.getDay(),
                                result.getWalkStep(), result.getRunStep(), result.getWalkDuration(), result.getRunDuration()));

                    } else {
                        ALog.d(TAG, "获取历史步数成功失败");
                    }
                    break;
            }
        }
    }

    /**
     * 清除全部存储数据
     */
    @Test
    public void testGClearAllData() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(2);
        ALog.d(TAG, "正在清除硬件数据...");
        device.command(BLEInsoleDevice.CMD_INSOLE_CLEAR_ALL, new BLEInsoleClearAllCommandParams(), new MainThreadDeviceCommandCallback<BLEInsoleClearAllCommandResult>() {
            @Override
            public void onMainThreadSuccess(Device device, BLEInsoleClearAllCommandResult result) {
                assertTrue(result.isSuccess());
                ALog.d(TAG, "清除数据成功");
                latch.countDown();
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        DeviceCommandCallback<BLEInsoleHistoryStepCommandResult> callback = new MainThreadDeviceCommandCallback<BLEInsoleHistoryStepCommandResult>() {
            @Override
            public void onMainThreadSuccess(Device device, BLEInsoleHistoryStepCommandResult result) {
                assertNotNull(result);

                Calendar calendar = Calendar.getInstance();
                assertEquals(result.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(result.getDay(), calendar.get(Calendar.DAY_OF_MONTH));

                assertEquals(result.getWalkStep(), 0);
                assertEquals(result.getWalkDuration(), 0);
                assertEquals(result.getRunStep(), 0);
                assertEquals(result.getRunDuration(), 0);
                Log.d(TAG, String.format("获取历史步数成功。%d月%d日 走步数：%d，跑步数：%d，走步时间：%d，跑步时间：%d", result.getMonth(), result.getDay(),
                        result.getWalkStep(), result.getRunStep(), result.getWalkDuration(), result.getRunDuration()));
                latch.countDown();
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        };
        device.command(BLEInsoleDevice.CMD_INSOLE_HISTORY_STEP, new BLEInsoleHistoryStepCommandParams(new Date()), callback);
        latch.await();
    }

    private long theFirstTimeGetAcc = 0;

    /**
     * 三轴出厂检测
     */
    @Test
    public void testEEventACC() throws InterruptedException {
        final int waitSecond = 60;//采集时间
        final CountDownLatch latch = new CountDownLatch(2);
        MainThreadDeviceEventCallback<BLEInsoleRealAccEventResult> callback = new MainThreadDeviceEventCallback<BLEInsoleRealAccEventResult>() {
            @Override
            public void onMainThreadData(Device device, BLEInsoleRealAccEventResult data) {
                assertNotNull(data);
                Log.d(TAG, String.format("acc real result a=[%d,%d,%d] b=[%d,%d,%d] c=[%d,%d,%d]",
                        data.getAx(), data.getAy(), data.getAz(),
                        data.getBx(), data.getBy(), data.getBz(),
                        data.getCx(), data.getCy(), data.getCz()));
                if (theFirstTimeGetAcc <= 0)
                    theFirstTimeGetAcc = System.currentTimeMillis();
            }
        };
        device.on(BLEInsoleDevice.EVENT_INSOLE_REAL_ACC, callback);
        ALog.d(TAG, "正在开启三轴出厂检测...");
        device.command(BLEInsoleDevice.CMD_INSOLE_ENABLE_ACC, new BLEInsoleEnableAccCommandParams(waitSecond), new MainThreadDeviceCommandCallback<BLEInsoleEnableAccCommandResult>() {
            @Override
            public void onMainThreadSuccess(Device device, BLEInsoleEnableAccCommandResult result) {
                assertTrue(result.isSuccess());
                ALog.d(TAG, "开启三轴出厂检测成功。采集 " + waitSecond + " 秒");
                latch.countDown();
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        //开启定时器
        new Timer().schedule(new TimerTask() {
            @Override
            public void run() {
                latch.countDown();
            }
        }, waitSecond * 1000);

        latch.await();
        assertTrue(theFirstTimeGetAcc > 0);

        final CountDownLatch latch2 = new CountDownLatch(1);

        ALog.d(TAG, "正在关闭三轴出厂检测...");
        device.command(BLEInsoleDevice.CMD_INSOLE_DISABLE_ACC, new BLEInsoleDisableAccCommandParams(), new MainThreadDeviceCommandCallback<BLEInsoleDisableAccCommandResult>() {
            @Override
            public void onMainThreadSuccess(Device device, BLEInsoleDisableAccCommandResult result) {
                assertTrue(result.isSuccess());
                ALog.d(TAG, "关闭三轴出厂检测成功");
                latch2.countDown();
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
        device.un(BLEInsoleDevice.EVENT_INSOLE_REAL_ACC, callback);
    }


    /**
     * 设备现场保护
     *
     * @throws InterruptedException
     */
    @Test
    public void testFStitaProtect() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);

        ALog.d(TAG, "正在发送现场保护命令");
        device.command(BLEInsoleDevice.CMD_INSOLE_FIRMWARE_UPGRADE_PROTECT, new BLEInsoleFirmwareUpgradeProtectCommandParams(), new MainThreadDeviceCommandCallback<BleInsoleFirmwareUpgradeProtectCommandResult>() {
            @Override
            public void onMainThreadSuccess(Device device, BleInsoleFirmwareUpgradeProtectCommandResult result) {
                ALog.d(TAG, "现场保护" + (result.isSuccess() ? "成功" : "失败"));
                assertTrue(result.isSuccess());
                latch.countDown();
            }

            @Override
            public void onMainThreadError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

    boolean isHadAccStart = false;

    /**
     * 测试开启三轴
     *
     * @throws InterruptedException
     */
    @Test
    public void testStart() throws InterruptedException {
        device.on(BLEInsoleDevice.EVENT_DEVICE_DATA, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult eventResult) {
                if (eventResult instanceof BLEInsoleCollectDataEventResult) {
                    BLEInsoleCollectDataEventResult data = (BLEInsoleCollectDataEventResult) eventResult;
                    for (BLEInsoleCollectDataEventResult.CollectData d : data.getDatas()) {
                        ALog.d(TAG, String.format("--> gait=%d%d%d%d\taccX=%s\taccY=%s\taccZ=%s",
                                d.isTouchA() ? 1 : 0,
                                d.isTouchB() ? 1 : 0,
                                d.isTouchC() ? 1 : 0,
                                d.isTouchD() ? 1 : 0,
                                String.valueOf(d.getAccX()),
                                String.valueOf(d.getAccY()),
                                String.valueOf(d.getAccZ())));
                    }
                } else {
                    ALog.d(TAG, String.format("-->" + JSON.toJSONString(eventResult)));
                }
            }
        });

        final CountDownLatch latch0 = new CountDownLatch(1);

        Log.d(TAG, "正在发送查询采集状态");
        device.readSetting(new DeviceSettingCallback<BLEInsoleGetSettingResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleGetSettingResult result) {

                isHadAccStart = device.isStarted();
                Log.d(TAG, String.format("是否正在采集：%s，采集频率 %d HZ", device.isStarted() ? "是" : "否", result.getRate()));
                latch0.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch0.countDown();

            }
        });
        latch0.await();

        if (device.isStarted()) {
            final CountDownLatch latch3 = new CountDownLatch(1);
            Log.d(TAG, String.format("正在发送关闭数据采集命令"));
            device.stop(new BLEInsoleStopCommandParams(), new DeviceCommandCallback<BLEInsoleCollectStartOrStopCommandResult>() {
                @Override
                public void onSuccess(Device device, BLEInsoleCollectStartOrStopCommandResult result) {
                    Log.d(TAG, "关闭数据采集" + (result.isSuccess() ? "成功" : "失败,错误码为" + result.getErrorCode()));
                    assertTrue(result.isSuccess());
                    latch3.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    latch3.countDown();

                }
            });

            latch3.await();
        }
        assertTrue(!device.isStarted());


        final CountDownLatch latch = new CountDownLatch(1);
        int rate = 8;
        Log.d(TAG, String.format("正在发送开启数据采集命令,采集频率 %d HZ", rate));
        device.start(new BLEInsoleStartCommandParams(rate), new DeviceCommandCallback<BLEInsoleCollectStartOrStopCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleCollectStartOrStopCommandResult result) {
                Log.d(TAG, "开启数据采集" + (result.isSuccess() ? "成功" : "失败,错误码为" + result.getErrorCode()));

                assertTrue(result.isSuccess());
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch.countDown();

            }
        });

        latch.await();
        assertTrue(device.isStarted());


        final CountDownLatch latch2 = new CountDownLatch(1);

        Log.d(TAG, "正在发送查询采集状态");
        device.readSetting(new DeviceSettingCallback<BLEInsoleGetSettingResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleGetSettingResult result) {

                isHadAccStart = device.isStarted();
                Log.d(TAG, String.format("是否正在采集：%s，采集频率 %d HZ", device.isStarted() ? "是" : "否", result.getRate()));
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch2.countDown();

            }
        });
        latch2.await();

        //采集数据2分钟，后停止
        Thread.sleep(2 * 60 * 1000);


        final CountDownLatch latch3 = new CountDownLatch(1);
        Log.d(TAG, String.format("正在发送关闭数据采集命令"));
        device.stop(new BLEInsoleStopCommandParams(), new DeviceCommandCallback<BLEInsoleCollectStartOrStopCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleCollectStartOrStopCommandResult result) {
                Log.d(TAG, "关闭数据采集" + (result.isSuccess() ? "成功" : "失败,错误码为" + result.getErrorCode()));
                assertTrue(result.isSuccess());
                latch3.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                latch3.countDown();

            }
        });

        latch3.await();
        assertTrue(!device.isStarted());
    }

}
