package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.os.Environment;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.callback.DeviceUpgradeCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleDisableAccCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleEnableAccCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleDisableAccCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleEnableAccCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealAccEventResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_MODE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.EVENT_READ_INFO;
import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.*;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * 设备固件升级
 * 升级完成重新连接固件
 * 检查后脚跟是否可用
 * 三轴出厂检查
 * Created by jalin on 2017/6/29.
 */
@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEInsoleUpgradeTest {
    private static final String TAG = BLEInsoleUpgradeTest.class.getSimpleName();
    private static Context context;
    private static final String[] deviceVV = new String[]{"72U6PZ", "J1SBJD", "T9SI3L", "FFPL7E"};
    private static final String deviceMac = "16:D2:B8:10:20:49";//16:D2:B8:03:30:26 16:D1:A6:10:09:98 16:D2:B8:10:20:49
    private static Device device;
    private static File file;

    @BeforeClass
    public static void begin() throws InterruptedException {

        context = InstrumentationRegistry.getTargetContext();

        AdvanproAndroidSDK.init(context);
        String fileName = "L.flash.xuv";
        file = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + "/DCIM/", fileName);
        if (file == null || !file.exists())
            ALog.d(TAG, "请在手机SDK卡跟目录下放置升级固件包，名称 R.flash.xuv");
        assertTrue(file != null && file.exists());


        final CountDownLatch latch = new CountDownLatch(1);
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        AdvanproAndroidSDK.init(context);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {

            @Override
            public void onScanning(BLEDeviceScanResult result)  {
                Log.d(TAG, String.format("%s [%s]", result.getRecord().getLocalName(), result.getBluetoothDevice().getAddress()));
                list.add(result);

                if ( deviceMac.equals(result.getRecord().getAddress())) {
                    device = result.create();
                    manager.stop();
                }

            }

            @Override
            public void onStop()  {
                ALog.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        //运行模式下升级
        if (DeviceMode.Application.name().equals(device.getInfo(BLE_DEVICE_INFO_MODE)))
            device.on(EVENT_READ_INFO, new DeviceEventCallback() {
                @Override
                public void onData(Device device,DeviceEventResult data)  {
                    Log.d(TAG, "EVENT_READ_INFO");
                    assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                    Log.d(TAG, String.format("设备信息：%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                            device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                            device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                    latch2.countDown();
                }
            });
        else
            latch2.countDown();
        device.on(EVENT_READ_INFO_ERROR, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                Log.d(TAG, "EVENT_READ_INFO_ERROR");

            }
        });
        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device,DeviceConnectResult result)  {
                latch2.countDown();
                ALog.d(TAG, "设备连接成功.");
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();

        assertEquals(device.isConnected(), true);
    }

    /**
     * 测试固件升级
     *
     * @throws InterruptedException
     */
    @Test(timeout = 30 * 60 * 1000)
    public void testBUpgrade() throws InterruptedException {
        ALog.d(TAG, "开始升级固件。。。");
        final CountDownLatch latch = new CountDownLatch(1);
        device.upgrade(file, new DeviceUpgradeCallback() {
            @Override
            public void onSuccess(Device device,DeviceCommandResult result)  {
                ALog.d(TAG, "升级固件成功");
                latch.countDown();
            }

            @Override
            public void onProgress(Device device,long total, long progress) {
                Log.d(TAG, String.format("Progress.. %d/%d", progress, total));
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                ALog.d(TAG, "升级固件失败");

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch.await();
        assertTrue(DeviceMode.Application.name().equals(device.getInfo(BLE_DEVICE_INFO_MODE)));
    }

    private long theFirstTimeGetAcc = 0;

    /**
     * 三轴出厂检测
     */
    @Test
    public void testDEventAcc() throws InterruptedException {
        final int waitSecond = 10;//采集时间
        final CountDownLatch latch = new CountDownLatch(2);
        device.on(BLEInsoleDevice.EVENT_INSOLE_REAL_ACC, new DeviceEventCallback<BLEInsoleRealAccEventResult>() {
            @Override
            public void onData(Device device,BLEInsoleRealAccEventResult data)  {
                assertNotNull(data);
                Log.d(TAG, String.format("acc real result a=[%d,%d,%d] b=[%d,%d,%d] c=[%d,%d,%d]",
                        data.getAx(), data.getAy(), data.getAz(),
                        data.getBx(), data.getBy(), data.getBz(),
                        data.getCx(), data.getCy(), data.getCz()));
                if (theFirstTimeGetAcc <= 0)
                    theFirstTimeGetAcc = System.currentTimeMillis();
            }
        });
        ALog.d(TAG, "正在开启三轴出厂检测...");
        device.command(BLEInsoleDevice.CMD_INSOLE_ENABLE_ACC, new BLEInsoleEnableAccCommandParams(waitSecond), new DeviceCommandCallback<BLEInsoleEnableAccCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEInsoleEnableAccCommandResult result)  {
                assertTrue(result.isSuccess());
                ALog.d(TAG, "开启三轴出厂检测成功。采集 " + waitSecond + " 秒");
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        //开启定时器
        new Timer().schedule(new TimerTask() {
            @Override
            public void run() {
                latch.countDown();
            }
        }, (waitSecond + 5) * 1000);

        latch.await();
        assertTrue(theFirstTimeGetAcc > 0);

        final CountDownLatch latch2 = new CountDownLatch(1);

        ALog.d(TAG, "正在关闭三轴出厂检测...");
        device.command(BLEInsoleDevice.CMD_INSOLE_DISABLE_ACC, new BLEInsoleDisableAccCommandParams(), new DeviceCommandCallback<BLEInsoleDisableAccCommandResult>() {
            @Override
            public void onSuccess(Device device,BLEInsoleDisableAccCommandResult result)  {
                assertTrue(result.isSuccess());
                ALog.d(TAG, "关闭三轴出厂检测成功");
                latch2.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (device == null || !device.isConnected()) {
            return;
        }

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device)  {
                latch.countDown();
            }

            @Override
            public void onError(Device device,DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

}
