package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.net.ParseException;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.json.JSONException;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleReadIdCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleReadIdCommandResult;

import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_ADDRESS;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_BATTERY;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_CODE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_FIRMWARE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_HARDWARE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_ID;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_NAME;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.BLE_DEVICE_INFO_TYPE;
import static hk.advanpro.android.sdk.device.ble.BLEDevice.EVENT_READ_INFO;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;


/**
 * Created by jalin on 2017/12/4.
 */

@RunWith(AndroidJUnit4.class)
public class BLEInsoleUserIdTest {
    private static final String TAG = BLEInsoleUserIdTest.class.getSimpleName();
    private static final String deviceVV = "5EZ5JM";//J1SBJD T9SI3L 72U6PZ
    private static Context context;
    private static Device device;

    @BeforeClass
    public static void begin() throws InterruptedException, ParseException, JSONException {
        final CountDownLatch latch = new CountDownLatch(1);
        if (context == null) {
            context = InstrumentationRegistry.getTargetContext();
        }
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        AdvanproAndroidSDK.init(context);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, String.format("Name:%s ,Address:%s, Type:%s", result.getRecord().getLocalName(),
                        result.getRecord().getAddress(),
                        result.getRecord().getManufacturer().getType()));

                if (deviceVV.equals(result.getRecord().getLocalName())) {
                    device = result.create();
                    manager.stop();
                }
                list.add(result);
            }

            @Override
            public void onStop() {
                ALog.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        ALog.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                Log.d(TAG, String.format("设备信息：%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                latch2.countDown();
            }
        });

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();
        assertEquals(device.isConnected(), true);


    }

    @Test
    public void testUserId() throws InterruptedException {
        final CountDownLatch latch2 = new CountDownLatch(1);

        ALog.d(TAG, "正在读取ID...");
        device.command(BLEInsoleDevice.CMD_INSOLE_READ_ID, new BLEInsoleReadIdCommandParams(), new DeviceCommandCallback<BLEInsoleReadIdCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleReadIdCommandResult result) {
                ALog.d(TAG, "读取ID成功 :" + result.getId());
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
    }
}
