package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import com.alibaba.fastjson.JSON;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;

import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.core.utils.UIUtils;
import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanFilterCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.shirt.BLEShirtHeartRateEventResult;
import hk.advanpro.android.sdk.device.result.ble.shirt.BLEShirtRealSensorEventResult;

import static junit.framework.Assert.assertNotNull;

/**
 * 作者：jalin on 2018/1/3 12:55
 * 邮箱：zhouxiaolin@advanpro.com
 */

@RunWith(AndroidJUnit4.class)
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BLEShirtDeviceTest {
    private static final String TAG = BLEAdvanproPiDeviceTest.class.getSimpleName();
    private static Context appContext;
    private static Device device;
    public static final String[] macAddressList = new String[]{"FD:BE:49:41:12:1D"};

    @BeforeClass
    public static void begin() throws InterruptedException {
        // Context of the app under test.
        appContext = InstrumentationRegistry.getTargetContext();
        final CountDownLatch latch = new CountDownLatch(1);
        AConfig aConfig = new AConfig();
        aConfig.setDebugLog(true);
        AdvanproAndroidSDK.init(appContext, aConfig);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);

        manager.scan(30, new DeviceManagerScanFilterCallback<BLEDeviceScanResult>() {
            @Override
            public boolean onFilter(BLEDeviceScanResult result) {
                return result.getRecord().getAddress().equalsIgnoreCase(macAddressList[0])
                        || result.getRecord().getManufacturer().getType() == DeviceType.BreathShirt ||
                        result.getRecord().getManufacturer().getType() == DeviceType.HeartShirt;
            }

            @Override
            public void onFilterScanning(BLEDeviceScanResult result) {
                manager.stop();
                device = result.create();
            }

            @Override
            public void onStop() {
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
        assertNotNull(device);
        final CountDownLatch latch2 = new CountDownLatch(1);

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                Log.d(TAG, "连接成功");
                Log.d(TAG, JSON.toJSONString(result));
                latch2.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });
        latch2.await();
    }

    // 监听事件
    @Test
    public void test0001_event() throws InterruptedException {
        // 心跳衣
        device.on(BLEShirtDevice.EVENT_SHIRT_HEART_RATE, new DeviceEventCallback<BLEShirtHeartRateEventResult>() {
            @Override
            public void onData(Device device, BLEShirtHeartRateEventResult data) {
                // 实时心跳数据
                assertNotNull(data);

                Log.d(TAG, "HEART:" + JSON.toJSONString(data));
            }
        });

        // 呼吸衣
        device.on(BLEShirtDevice.EVENT_SHIRT_REAL_SENSOR, new DeviceEventCallback<BLEShirtRealSensorEventResult>() {
            @Override
            public void onData(Device device, BLEShirtRealSensorEventResult data) {
                // 实时呼吸数据
                assertNotNull(data);
                Log.d(TAG, "SENSOR:" + JSON.toJSONString(data));

            }
        });

        final CountDownLatch latch = new CountDownLatch(1);
        UIUtils.getUIHandler().postDelayed(new Runnable() {
            @Override
            public void run() {
                latch.countDown();
            }
        }, 1 * 60 * 1000);
        latch.await();
    }

    //断开连接
    @AfterClass
    public static void end() throws InterruptedException {
        if (device == null)
            return;
        final CountDownLatch latch = new CountDownLatch(1);

        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }

}
