package hk.advanpro.android.sdk.device.ble;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.IOException;
import java.io.StringWriter;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import hk.advanpro.android.core.raw.RawDataFile;
import hk.advanpro.android.core.utils.DateUtils;
import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleHistoryGaitCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleHistoryStepCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleRawGaitCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleSectionStepCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleHistoryGaitCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleHistoryStepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRawGaitCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleStepSectionsCommandResult;

import static hk.advanpro.android.sdk.device.ble.DefaultBLEDevice.*;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

/**
 * 历史数据测试用例
 * Created by jalin on 2017/6/16.
 */
@RunWith(AndroidJUnit4.class)
public class InsoleDeviceHisDataTest {
    private static final String TAG = InsoleDeviceHisDataTest.class.getSimpleName();
    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    private static final String deviceVV = "72U6PZ";//J1SBJD  72U6PZ T9SI3L  N4L3LT 7X2MWO DXJBG2  YJL9MY 7X2MWO H1OX3J J1SBJD C44L2I  YJL9MY KR46SV NPBI0S FFPL7E
    private static final String time = "2017-12-06";//指定获取某天的日期，格式:yyyy-MM-dd,例如:2017-06-14
    private static Context context;
    private static Device device;
    private static Date date;//指定获取某天的日期

    @BeforeClass
    public static void begin() throws InterruptedException, ParseException {
        date = sdf.parse(time);

        final CountDownLatch latch = new CountDownLatch(1);
        if (context == null) {
            context = InstrumentationRegistry.getTargetContext();
        }
        final Set<BLEDeviceScanResult> list = new HashSet<>();
        AdvanproAndroidSDK.init(context);
        final BLEDeviceManager manager = AdvanproAndroidSDK.getDeviceManager(BLEDeviceManager.class);
        manager.scan(30, new DeviceManagerScanCallback<BLEDeviceScanResult>() {
            @Override
            public void onScanning(BLEDeviceScanResult result) {
                Log.d(TAG, result.getRecord().getLocalName());

                if (deviceVV.equals(result.getRecord().getLocalName())) {
                    device = result.create();
                    manager.stop();
                }
                list.add(result);
            }

            @Override
            public void onStop() {
                Log.d(TAG, "停止扫描");
                latch.countDown();
            }

            @Override
            public void onError(DeviceCallbackException err) {
                err.printStackTrace();
                throw new RuntimeException(err);
            }
        });

        latch.await();
        assertTrue(!list.isEmpty());
        assertNotNull(device);

        Log.d(TAG, "开始连接设备.");
        final CountDownLatch latch2 = new CountDownLatch(2);
        device.on(EVENT_READ_INFO, new DeviceEventCallback() {
            @Override
            public void onData(Device device, DeviceEventResult data) {
                assertTrue(Integer.parseInt(device.getInfo(BLE_DEVICE_INFO_BATTERY)) > 0);
                Log.d(TAG, String.format("设备信息：%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s", device.getInfo(BLE_DEVICE_INFO_NAME), device.getInfo(BLE_DEVICE_INFO_ID), device.getInfo(BLE_DEVICE_INFO_TYPE),
                        device.getInfo(BLE_DEVICE_INFO_ADDRESS), device.getInfo(BLE_DEVICE_INFO_CODE), device.getInfo(BLE_DEVICE_INFO_FIRMWARE), device.getInfo(BLE_DEVICE_INFO_HARDWARE),
                        device.getInfo(BLE_DEVICE_INFO_BATTERY)));
                latch2.countDown();
            }
        });

        device.connect(new DeviceConnectCallback() {
            @Override
            public void onSucceed(Device device, DeviceConnectResult result) {
                latch2.countDown();
                Log.d(TAG, "设备连接成功.");
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch2.await();
        assertEquals(device.isConnected(), true);
    }

    /**
     * 测试步态历史数据
     *
     * @throws ParseException
     * @throws InterruptedException
     */
    @Test
    public void testHistGait() throws ParseException, InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        DeviceCommandCallback<BLEInsoleHistoryGaitCommandResult> callback = new DeviceCommandCallback<BLEInsoleHistoryGaitCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleHistoryGaitCommandResult result) {
                assertNotNull(result);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                assertEquals(result.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(result.getDay(), calendar.get(Calendar.DAY_OF_MONTH));
                Log.d(TAG, String.format("获取步态历史数据成功。%d月%d日 \t内翻：%d\t正常：%d\t外翻：%d\t前掌：%d\t全掌：%d\t脚跟：%d",
                        (int) result.getMonth(), (int) result.getDay(),
                        (int) result.getVarus(),
                        result.getNormal(), (int) result.getEctropion(),
                        (int) result.getForefoot(), (int) result.getSole(), (int) result.getHeel()));


                assertTrue(result.getVarus() >= 0);//内翻
                assertTrue(result.getEctropion() >= 0);//外翻
                assertTrue(result.getForefoot() >= 0);//脚掌
                assertTrue(result.getHeel() >= 0);//脚跟
                assertTrue(result.getSole() >= 0);//全脚
                assertTrue(result.getNormal() >= 0);//正常
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        };
        BLEInsoleHistoryGaitCommandParams params = new BLEInsoleHistoryGaitCommandParams(date);
        device.command(BLEInsoleDevice.CMD_INSOLE_HISTORY_GAIT, params, callback);
        latch.await();
    }

    /**
     * 测试原始步态数据
     */
    @Test
    public void testRawGait() throws ParseException, InterruptedException {

        final CountDownLatch latch = new CountDownLatch(1);
        DeviceCommandCallback<BLEInsoleRawGaitCommandResult> callback = new DeviceCommandCallback<BLEInsoleRawGaitCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleRawGaitCommandResult result) {
                assertNotNull(result);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                assertEquals(result.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(result.getDay(), calendar.get(Calendar.DAY_OF_MONTH));
                Log.d(TAG, String.format("获取步态原始数据成功。%d月%d日 bytes = %d kb", result.getMonth(), result.getDay(), result.getLength()));
                Log.d(TAG, String.format("=> {[段号,时间偏移],[着地状态,冲击力等级],[走跑,着地方式,足内旋]}"));
                for (BLEInsoleRawGaitCommandResult.BLEInsoleRawGaitSection section : result.getSections()) {
                    Log.d(TAG, String.format("=> {[%d,%2d],[%2d,%d],[%s,%d,%d]}",
                            section.getSection(),
                            section.getMinute(),
                            section.getGait(),
                            section.getImpact(),
                            section.getType() == 0 ? "走" : "跑",
                            section.getFootType(),
                            section.getTouchdown()
                    ));
                }
                //写入原始文件
                String devId = device.getInfo(BLE_DEVICE_INFO_ID);
                RawDataFile rawDataFile = RawDataFile.getR1("HGait", "1", DateUtils.getStartOfDay(date).getTime());
                for (BLEInsoleRawGaitCommandResult.BLEInsoleRawGaitSection cell : result.getSections()) {
                    /**步态原始数据格式：时间 设备编号 步态原始数据 冲击等级 走跑标志 着地方式 内外翻**/
                    long time = result.getDate().getTime() + ((cell.getSection() - 1) * 15 + cell.getMinute()) * 60 * 1000;
                    rawDataFile.put(new Date(time),
                            devId, String.valueOf(cell.getGait()), String.valueOf(cell.getImpact()), String.valueOf(cell.getType()),
                            String.valueOf(cell.getTouchdown()), String.valueOf(cell.getFootType()));
                }
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        };
        BLEInsoleRawGaitCommandParams params = new BLEInsoleRawGaitCommandParams(date, 61, 12);
        device.command(BLEInsoleDevice.CMD_INSOLE_RAW_GAIT, params, callback);
        latch.await();
    }

    BLEInsoleRawGaitCommandResult gaitCommandResult;
    BLEInsoleHistoryStepCommandResult stepCommandResult;

    SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");

    /**
     * 测试原始步态数据传输数段及有效性
     */
    @Test
    public void testRawGaitSpeed() throws ParseException, InterruptedException, IOException {
        final CountDownLatch countDownLatch = new CountDownLatch(1);
        device.command(BLEInsoleDevice.CMD_INSOLE_HISTORY_STEP, new BLEInsoleHistoryStepCommandParams(date), new DeviceCommandCallback() {
            @Override
            public void onSuccess(Device device, DeviceCommandResult result) {
                stepCommandResult = (BLEInsoleHistoryStepCommandResult) result;
                countDownLatch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                countDownLatch.countDown();
                error.printStackTrace();

            }
        });
        countDownLatch.await();
        assertNotNull(stepCommandResult);
        int stepCount = stepCommandResult.getRunStep() + stepCommandResult.getWalkStep();
        assertTrue(stepCount > 0);

        int count = 20;
        while (count-- > 0) {
            final CountDownLatch latch = new CountDownLatch(1);
            StringWriter stringWriter = new StringWriter();
            ALog.begin(stringWriter);
            gaitCommandResult = null;
            DeviceCommandCallback<BLEInsoleRawGaitCommandResult> callback = new DeviceCommandCallback<BLEInsoleRawGaitCommandResult>() {
                @Override
                public void onSuccess(Device device, BLEInsoleRawGaitCommandResult result) {
                    assertNotNull(result);
                    gaitCommandResult = result;
                    latch.countDown();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    error.printStackTrace();
                    throw new RuntimeException(error);
                }
            };
            BLEInsoleRawGaitCommandParams params = new BLEInsoleRawGaitCommandParams(date, 1, 96);
            device.command(BLEInsoleDevice.CMD_INSOLE_RAW_GAIT, params, callback);
            latch.await();
            ALog.end(stringWriter);

            String[] logs = stringWriter.toString().split("\\n");

            boolean begin = false;
            int totalBufferByteSize = 0;
            int packageCount = 0;
            long startTime = 0;
            long endTime = 0;
            for (int i = 0; i < logs.length; i++) {
                String[] sss = logs[i].split("/");
                if (sss.length == 3 && sss[1].equals("BLEInsoleDevice")) {
                    if (!begin && sss[2].contains("begin buffer, length")) {//开始收包
                        startTime = simpleDateFormat.parse(sss[0]).getTime();
                        begin = true;
                        totalBufferByteSize = Integer.parseInt(sss[2].substring(sss[2].indexOf("length") + 7));
                    } else if (begin && sss[2].contains("history result 00001c00-d102-11e1-9b23-000efb00dbdb & 00001c0e-d102-11e1-9b23-000efb00dbdb")) {
                        packageCount++;
                    } else if (begin && sss[2].contains("end buffer")) {//结束收包
                        endTime = simpleDateFormat.parse(sss[0]).getTime();
                        break;
                    }
                }
            }
            int realPackageCount = ((totalBufferByteSize - 3) / 20) + ((totalBufferByteSize - 3) % 20 > 0 ? 1 : 0);

            int realStepCount = gaitCommandResult.getSections().size();

            assertTrue(realPackageCount == packageCount);
//            assertTrue(realStepCount == stepCount);

            Log.d("====", String.format("字节数：%d ,包数：%d ,是否丢包：%s,是否有效：%s,%d ms", totalBufferByteSize, packageCount, realPackageCount == packageCount, realStepCount == stepCount, endTime - startTime));

        }
    }

    /**
     * 测试历史步数
     *
     * @throws InterruptedException
     */
    @Test
    public void testHisSteps() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        DeviceCommandCallback<BLEInsoleHistoryStepCommandResult> callback = new DeviceCommandCallback<BLEInsoleHistoryStepCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleHistoryStepCommandResult result) {
                assertNotNull(result);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                assertEquals(result.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(result.getDay(), calendar.get(Calendar.DAY_OF_MONTH));

                Log.d(TAG, String.format("获取历史步数成功。%d月%d日 走步数：%d，跑步数：%d，走步时间：%d，跑步时间：%d", result.getMonth(), result.getDay(),
                        result.getWalkStep(), result.getRunStep(), result.getWalkDuration(), result.getRunDuration()));

                assertTrue(result.getWalkStep() >= 0);
                assertTrue(result.getRunStep() >= 0);
                assertTrue(result.getWalkDuration() >= 0);
                assertTrue(result.getRunDuration() >= 0);
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);
            }
        };
        BLEInsoleHistoryStepCommandParams params = new BLEInsoleHistoryStepCommandParams(date);
        device.command(BLEInsoleDevice.CMD_INSOLE_HISTORY_STEP, params, callback);
        latch.await();
    }

    //测试历史分段步数
    @Test
    public void testHisSectionSteps() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        DeviceCommandCallback<BLEInsoleStepSectionsCommandResult> callback = new DeviceCommandCallback<BLEInsoleStepSectionsCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleStepSectionsCommandResult result) {
                assertNotNull(result);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                assertEquals(result.getMonth(), calendar.get(Calendar.MONTH) + 1);
                assertEquals(result.getDay(), calendar.get(Calendar.DAY_OF_MONTH));
                Log.d(TAG, String.format("获取分段步数成功。%d月%d日,长度：%d", result.getMonth(), result.getDay(), result.getLength()));
                Log.d(TAG, "=>  [段号,走步数,跑步数]");
                for (BLEInsoleStepSectionsCommandResult.BLEInsoleStepSection section : result.getSections()) {
                    Log.d(TAG, String.format("=>  [%2d,%8d,%8d]", section.getSection(), section.getWalkStep(), section.getRunStep()));
                    assertTrue(section.getSection() > 0 && section.getSection() <= 96);
                    assertTrue(section.getWalkStep() >= 0);
                    assertTrue(section.getRunStep() >= 0);
                }

                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                error.printStackTrace();
                throw new RuntimeException(error);

            }
        };
        BLEInsoleSectionStepCommandParams params = new BLEInsoleSectionStepCommandParams(date, 1, 96);
        device.command(BLEInsoleDevice.CMD_INSOLE_SECTION_STEP, params, callback);
        latch.await();
    }

    @AfterClass
    public static void end() throws InterruptedException {
        final CountDownLatch latch = new CountDownLatch(1);
        device.disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                latch.countDown();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {

                error.printStackTrace();
                throw new RuntimeException(error);
            }
        });

        latch.await();
    }
}
