package hk.advanpro.android.sdk.algorithm.belt;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * 呼吸带jni算法库接入
 * Created by jalin on 2017/7/3.
 */
public class BLEBeltAlg {
    public static final int ErrorCode_Success = 0;
    public static final int ErrorCode_InitializationFailed = 1;  // 初始化失败
    public static final int ErrorCode_MemoryAllocationFailed = 2; // 内存分配失败
    public static final int ErrorCode_InvalidParameters = 3; // 参数无效
    public static final int ErrorCode_DataProcFailed = 4; // 数据处理失败
    public static final int ErrorCode_DataAnlysisFailed = 5; // 数据包分析失败
    public static final int ErrorCode_DifferentialFailed = 6; // 数据微分失败
    public static final int ErrorCode_WriteRwDataToBufFailed = 7; // 原始数据缓存失败
    public static final int ErrorCode_WriteDfDataToBufFailed = 8; // 微分数据缓存失败
    public static final int ErrorCode_BreathAnalysingFailed = 9; // 呼吸解析失败

    static {
        System.loadLibrary("ASWearModule");
    }

    public static class SmartBelt_Time {
        public int iYear;
        public int iMonth;  // 1~12
        public int iDay;    // 1~31
        public int iHour;   // 0~23
        public int iMinute; // 0~59
        public int iSecond; // 0~59
        public int iMilliSecond;

        public Date getDate() {
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.set(iYear, iMonth - 1, iDay, iHour, iMinute, iSecond);
            Date dt = new Date(calendar.getTimeInMillis() + iMilliSecond);
            return dt;
        }

        public void setDate(Date date) {
            GregorianCalendar calendar = new GregorianCalendar();
            calendar.setTime(date);
            iYear = calendar.get(Calendar.YEAR);
            iMonth = (byte) calendar.get(Calendar.MONTH) + 1;
            iDay = (byte) calendar.get(Calendar.DATE);
            iHour = (byte) calendar.get(Calendar.HOUR_OF_DAY);
            iMinute = (byte) calendar.get(Calendar.MINUTE);
            iSecond = (byte) calendar.get(Calendar.SECOND);
            iMilliSecond = calendar.get(Calendar.MILLISECOND);
        }

        // 总毫秒数
        public long getTotalMillSecond() {
            long lTotal = 0;
            lTotal += iYear * 31536000;
            lTotal += iMonth * 2592000;
            lTotal += iDay * 86400;
            lTotal += iHour * 3600;
            lTotal += iMinute * 60;
            lTotal += iSecond;
            lTotal *= 1000;
            lTotal += iMilliSecond;
            return lTotal;
        }
    }

    // 身体姿态
    public static class ESleepPosture {
        public static final int POSTURE_ILLEGAL = -1;
        public static final int POSTURE_UNKONWN = 0;
        public static final int POSTURE_UPSIDEDOWN = 0;  // 倒立，或者说，呼吸带戴反了
        public static final int POSTURE_FACEUP = 1;  // 仰躺
        public static final int POSTURE_FACERIGHT = 2;  // 左侧躺
        public static final int POSTURE_FACELEFT = 3;  // 右侧躺
        public static final int POSTURE_FACEDOWN = 4;  // 趴卧躺
        public static final int POSTURE_UPWARD = 5;  // 站立或坐立
    }

    //================================================================================================
    // 呼吸数据处理
    //================================================================================================

    public static native long InitDataProcMdl(SmartBelt_TMdlInitInfo pSInitInfo);

    public static native int SendBreathDataToDataProcMdl(long inst, SmartBelt_Data pSRawBData, SmartBelt_TSBreathWaveData pSWaveData);

    public static native int RcvBreathStatsFrmDataProcMdl(long inst, SmartBelt_TSBreathStats SBrthStats);

    public static native int ExitDataProcMdl(long inst);

    public static class SmartBelt_TMdlInitInfo {
        // 缓存大小，以数据点为单位，而非以字节单位，设置0表示默认值
        public int iBufferSize;
    }

    public static class SmartBelt_Data {
        // 数据包时间
        public SmartBelt_Time SPktTime = new SmartBelt_Time();
        // 数据包内容
        public double[] daFbrSnsrData;
        public double[] daAccXData;
        public double[] daAccYData;
        public double[] daAccZData;
        // 数据采集精度，硬件侧固定以16bit精度采集数据，算法库根据该参数做精度调整，设置范围[11,16]，默认值为14
        public int iDAQPrecision = 14;
        // 平滑阶数，对呼吸波形数据做平滑处理，设置范围[1,20]，1不做平滑，默认15
        public int iSmoothOrder = 15;
        // 三轴加速度的量程，单位为g,默认值为8g
        public int iAccRange = 8;
        // 睡眠身体姿态阈值，默认值为0.6
        public double dPostureThreshold = 0.6;
    }

    public static class SmartBelt_TSBreathWaveData {
        // 平滑之后的原始呼吸波形数据，最长10个数据点
        public double[] daBrthWave;
        // 平滑之后的三轴加速计波形数据
        public double[] daAccXWave;
        public double[] daAccYWave;
        public double[] daAccZWave;
        // 呼吸波形的斜率变化率数据. 最长10个数据点
        public double[] daBrthSlopeVaryRto;
    }

    public static class SmartBelt_TSBreathStats {
        // 呼吸记录序号，从1开始
        public long lBreathSN = 1;
        // 该呼吸在呼吸波形数据序列中的前波谷索引
        public long lPreTroughIndex;
        // 该呼吸在呼吸波形数据序列中的波峰索引
        public long lPeakIndex;
        // 该呼吸在呼吸波形数据序列中的后波谷索引
        public long lPostTroughIndex;

        /***********Current Breath Data***************/
        // 该呼吸的吸时长
        public double dInspiratoryTime;
        // 该呼吸的呼时长
        public double dExspiratoryTime;
        // 该呼吸的吸呼时长比.
        public double dRtoOfIns2Exs;
        // 该呼吸的持续时长(秒)
        public double dBrthTime;
        // 该呼吸的幅度，为容积变化百分比数值.
        public double dAmplitude;
        // 该呼吸的瞬时频率.
        public double dInstantFreq;

        // 该呼吸的起始时间
        public SmartBelt_Time SStartTime = new SmartBelt_Time();
        // 该呼吸的终止时间
        public SmartBelt_Time SEndTime = new SmartBelt_Time();
        // 该呼吸的持续时间
        public SmartBelt_Time SSpanTime = new SmartBelt_Time();

        // 该呼吸的校准波形数据
        public double[] daPerDataBuf;
        // 该呼吸对应的身体姿态序列, 类型参见ESleepPosture
        public int[] eaPostureBuf;
        // 该呼吸对应的三轴数据
        public double[] daAccX;
        public double[] daAccY;
        public double[] daAccZ;

        /*****************************************/
        // 所有呼吸的统计信息
        // 呼吸总计数
        public long lTotalBrthCnt;
        // 呼吸总时间
        public double dTotalBrthTime;
        // 平均呼吸时长
        public double dAvgBrthTime;
        // 总吸时长
        public double dTotalInsTime;
        // 平均吸时长
        public double dAvgInsTime;
        // 总呼时长
        public double dTotalExsTime;
        // 平均呼时长
        public double dAvgExsTime;
        // 总吸呼时长比.
        public double dTotalRtoOfIns2Exs;
        // 平均呼吸时长比
        public double dAvgRtoOfIns2Exs;
        // 平均呼吸频率
        public double dAvgBrthFreq;
        // 总幅度
        public double dTotalBrthAmp;
        // 平均呼吸幅度
        public double dAvgBrthAmp;
    }

    //================================================================================================
    // 睡眠数据处理
    //================================================================================================
    public static native long InitSleepAnalyzer();

    public static native int SendDataToSleepAnalyzer(long inst, double accx, double accy, double accz);

    public static native int RcvDataFrmSleepAnalyzer(long inst, int hourStart, SleepAnalyzerOut out);

    public static native int ExitSleepAnalyzer(long inst);

    public static class SleepAnalyzerOut {
        // 睡眠曲线，[0,3]数值越小睡眠越深
        public double[] dSleepGraph;
        // 睡眠曲线每个点的间隔，秒
        public long lIntervalOfPoints = 30;
        // 睡眠得分
        public long iSleepQualityScore;
    }

    //================================================================================================
    // 计步数据处理
    //================================================================================================

    public static final int MMAXSCDATAPACKETLEN = 20;
    public static final int MREALSCDATAPACKETLEN = 13;
    public static final double WALK_FACTOR = 0.45;  // 步行因子，默认值0.45
    public static final double RUN_FACTOR = 0.5; // 跑步因子，默认值0.50
    public static final double CALORIE_FACOTOR = 1.036;  // 卡路里消耗因子，默认值1.036

    // 计算距离（千米），dHeight身高，厘米
    public static double CalculateDistance(long lSteps, double dHeight, double dFactor) {
        if (0 >= lSteps || 0 >= dHeight || 0 >= dFactor)
            return 0;
        dHeight = dHeight / 100; // 米
        return (lSteps * dHeight * dFactor) / 2 / 1000;//km
    }

    // 计算卡路里（千卡），dWeight体重，公斤
    public static double CalculateCalories(double dDistance, double dWeight, double dFactor) {
        if (0 >= dDistance || 0 >= dWeight || 0 >= dFactor)
            return 0;
        return dDistance * dWeight * dFactor;
    }

    //================================================================================================
    // 内裤算法
    //================================================================================================

    public static native long InitUnderwearDataProcFunction();

    public static native int SendUnderwearDataToDataProcMdl(long inst, SmartBelt_Data pSRawData);

    public static native int RcvUnderwearAnalysedData(long inst, SmartUnderwear_TAnalysedDataRecord pSAnalysedData);

    public static native int ExitUnderwearDataProcFunction(long inst);

    public static class SmartUnderwear_TAnalysedDataRecord {
        // 序号，从1开始
        public long lFlightIndex = 1;
        // 飞行开始时间
        public SmartBelt_Time SFlightStartTime = new SmartBelt_Time();
        // 飞行结束时间
        public SmartBelt_Time SFlightEndTime = new SmartBelt_Time();
        // 起飞时长(秒)
        public double dTakeOffDuration = 0;
        // 飞行高度.%.
        public double dCruiseHeight = 0;
        // 飞行时长(秒)
        public double dCruiseDuration = 0;
        // 降落时长(秒)
        public double dDesndDuration = 0;
    }
}
