package hk.advanpro.android.sdk.algorithm.belt;

import java.util.Calendar;
import java.util.Date;

/**
 * 呼吸带睡眠分析
 * Created by jalin on 2017/7/3.
 */

public class BLEBeltSleepAnalyzer {

    private long inst;
    private boolean enable;
    private Date startTime;

    public BLEBeltSleepAnalyzer() {
    }

    /**
     * 初始化
     * @throws Exception 初始化异常
     */
    public void init() throws Exception {
        this.inst = BLEBeltAlg.InitSleepAnalyzer();
        enable = true;
        startTime = new Date();
    }

    /**
     * 输入三轴数据
     * @param accx X轴
     * @param accy Y轴
     * @param accz Z轴
     * @return 是否输入成功
     */
    public boolean push(double accx, double accy, double accz) {
        boolean ret = false;
        int ec = BLEBeltAlg.SendDataToSleepAnalyzer(inst, accx, accy, accz);
        if (ec == BLEBeltAlg.ErrorCode_Success) {
            ret = true;
        }
        return ret;
    }

    /**
     * 输出睡眠分析结果
     * @return 睡眠分析结果
     */
    public BLEBeltAlg.SleepAnalyzerOut poll() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(getStartTime());
        int hourStart = calendar.get(Calendar.HOUR_OF_DAY);//睡眠开始时间
        BLEBeltAlg.SleepAnalyzerOut out = new BLEBeltAlg.SleepAnalyzerOut();
        int ec = BLEBeltAlg.RcvDataFrmSleepAnalyzer(inst, hourStart, out);
        if (ec != BLEBeltAlg.ErrorCode_Success) {
            out = null;
        }
        return out;
    }

    /**
     * 退出并释放资源
     * @throws Exception 退出异常
     */
    public void exit() throws Exception {
        BLEBeltAlg.ExitSleepAnalyzer(inst);
        enable = false;
    }

    public boolean isEnable() {
        return enable;
    }

    public Date getStartTime() {
        return startTime;
    }

}
