package hk.advanpro.android.sdk.commons;

import android.util.Log;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;

/**
 * Created by jalin on 2017/7/12.
 */

public class ALog {
    private static final List<Writer> writerList = new ArrayList<>();
    private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");

    /**
     * 开始输出日志
     *
     * @param writer writer
     */
    public static void begin(Writer writer) {
        if (writer == null)
            return;
        writerList.add(writer);
    }

    /**
     * 结束输出日志
     *
     * @param writer writer
     */
    public static void end(Writer writer) {
        if (writer == null)
            return;
        writerList.remove(writer);
        try {
            writer.flush();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            try {
                writer.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

    private static void output(Date date, String tag, String message) {
        if (writerList.isEmpty())
            return;
        String line = String.format("%s/%s/%s\n", simpleDateFormat.format(date), tag, message);
        for (Writer writer : writerList) {
            try {
                writer.write(line);
            } catch (IOException e) {
                Log.e("ALog", "output", e);
            }
        }
    }

    private static void output(Date date, String tag, Exception exception) {
        if (writerList.isEmpty())
            return;
        String line = String.format("%s/%s/%s\n", simpleDateFormat.format(date), tag, getErrorInfoFromException(exception));
        for (Writer writer : writerList)
            try {
                writer.write(line);
            } catch (IOException e) {
                Log.e("ALog", "output", e);
            }
    }

    /**
     * 输出调试信息
     *
     * @param tag     TAG
     * @param message 输出内容
     */
    public static void d(String tag, String message) {
        if (AdvanproAndroidSDK.getConfig().isDebugLog()) {
            Log.d(tag, message);
            output(new Date(), tag, message);
        }
    }

    /**
     * 输出提示信息
     *
     * @param tag     TAG
     * @param message 输出内容
     */
    public static void i(String tag, String message) {
        Log.i(tag, message);
        if (AdvanproAndroidSDK.getConfig().isDebugLog()) {
            output(new Date(), tag, message);
        }
    }

    /**
     * 输出错误日志
     *
     * @param tag     Tag
     * @param message 输出内容
     */
    public static void e(String tag, String message) {
        Log.e(tag, message);
        if (AdvanproAndroidSDK.getConfig().isDebugLog()) {
            output(new Date(), tag, message);
        }
    }

    /**
     * 输出错误信息
     * @param tag tag
     * @param except except
     */
    public synchronized static void e(String tag, Exception except) {
        except.printStackTrace();
        if (AdvanproAndroidSDK.getConfig().isDebugLog()) {
            output(new Date(), tag, except);
        }
    }

    public static String getErrorInfoFromException(Exception e) {
        try {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            return "\r\n" + sw.toString() + "\r\n";
        } catch (Exception e2) {
            return "bad getErrorInfoFromException";
        }
    }

}
