package hk.advanpro.android.sdk.commons;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 异步执行多任务
 * execute(final Runnable runnable) 调用方法执行
 * <p>
 * waitComplete()调用该方法等待执行完成
 * <p>
 * Created by jalin on 2017/12/6.
 */

public class BatchExecutor {
    private ScheduledExecutorService es;
    private AtomicInteger count = new AtomicInteger();

    /**
     * 多任务执行器
     * @param threads 线程个数
     * @return 多任务执行器
     */
    public static BatchExecutor create(int threads) {
        return new BatchExecutor(threads);
    }

    public BatchExecutor(int threads) {
        es = Executors.newScheduledThreadPool(threads);
    }

    /**
     * 执行
     *
     * @param runnable 事务
     */
    public void execute(final Runnable runnable) {
        count.incrementAndGet();
        es.execute(new Runnable() {
            @Override
            public void run() {
                runnable.run();
                count.decrementAndGet();
            }
        });
    }

    /**
     * 等待事务完成
     *
     * @throws InterruptedException InterruptedException
     */
    public void waitComplete() throws InterruptedException {
        while (count.get() > 0) {
            synchronized (count) {
                count.wait(0, 200);
            }
        }
    }
}
