package hk.advanpro.android.sdk.commons;

import android.os.ConditionVariable;

import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;

/**
 * 同步获取设备命令结果任务
 * Created by jalin on 2017/12/4.
 */

public class DeviceCommandSyncTask<T extends DeviceCommandResult> implements Runnable {
    private Device device;
    private String commandKey;
    private DeviceCommandParams params;

    private int timeout;
    private T result;
    private DeviceCallbackException error;

    public static DeviceCommandSyncTask create(Device device, String key, DeviceCommandParams params) {
        DeviceCommandSyncTask deviceCommandTask = new DeviceCommandSyncTask();
        deviceCommandTask.device = device;
        deviceCommandTask.commandKey = key;
        deviceCommandTask.params = params;
        return deviceCommandTask;
    }

    public static DeviceCommandSyncTask create(Device device, String key, DeviceCommandParams params, int timeout) {
        DeviceCommandSyncTask deviceCommandTask = new DeviceCommandSyncTask();
        deviceCommandTask.device = device;
        deviceCommandTask.commandKey = key;
        deviceCommandTask.params = params;
        deviceCommandTask.timeout = timeout;
        return deviceCommandTask;
    }

    public boolean isSuccess() {
        return result != null;
    }

    public T getResult() {
        return result;
    }

    public DeviceCallbackException getError() {
        return error;
    }


    public DeviceCommandSyncTask execute() throws DeviceCallbackException {
        result = null;
        error = null;
        final ConditionVariable conditionVariable = new ConditionVariable();
        DeviceCommandCallback callback = new DeviceCommandCallback() {
            @Override
            public void onSuccess(Device device, DeviceCommandResult result) {
                DeviceCommandSyncTask.this.result = (T) result;
                open();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                DeviceCommandSyncTask.this.error = error;
                open();
            }

            private void open() {
                // 事情办完了，notify到另外线程
                conditionVariable.open();
            }
        };
        if (timeout > 0) {
            device.command(commandKey, params, timeout, callback);
        } else {
            device.command(commandKey, params, callback);
        }

        // 等着另外线程做的事完成，同时支持设置超时
        conditionVariable.block();

        if (result == null) {
            throw error;
        }
        return this;
    }

    @Override
    public void run() {
        try {
            execute();
        } catch (DeviceCallbackException e) {
            e.printStackTrace();
        }
    }

    public String getCommandKey() {
        return commandKey;
    }

    public Device getDevice() {
        return device;
    }

    public DeviceCommandParams getParams() {
        return params;
    }
}
