package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.support.annotation.NonNull;
import android.util.Log;

import com.alibaba.fastjson.JSON;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TimerTask;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import hk.advanpro.android.core.log.ALogger;
import hk.advanpro.android.core.utils.UIUtils;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.commons.DeviceCommandSyncTask;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceSettingCallback;
import hk.advanpro.android.sdk.device.callback.DeviceUpgradeCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.params.DeviceSettingParams;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiCommandParams;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiSettingParams;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceSettingResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceInfoResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiGetSettingResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiCommandResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiDataEventResult;
import hk.advanpro.android.sdk.utils.CommonUtils;

/**
 * Created by yejianfei on 2017/6/27.
 */

public class BLEAdvanproPiDevice extends BLEDevice {

    public static final String BLE_PI_INFO_SD_TOTAL_SIZE = "BLE_PI_INFO_SD_TOTAL_SIZE";
    public static final String BLE_PI_INFO_SD_FREE_SIZE = "BLE_PI_INFO_SD_FREE_SIZE";
    public static final String BLE_PI_INFO_STATE = "BLE_PI_INFO_STATE";
    public static final String EVENT_BATTERY_CHANGE = "EVENT_BATTERY_CHANGE";
    /**
     * 启动采样
     **/
    public static final String CMD_PI_START = "st";
    /**
     * 停止采样
     **/
    public static final String CMD_PI_STOP = "sp";
    /**
     * 在设备处于采集状态时，获取 start 配置信息
     **/
    public static final String CMD_PI_GET_START_CONFIG = "gsc";
    /**
     * 用于获取当前上发数据的列名
     **/
    public static final String CMD_PI_GET_DATA_NAME = "gdn";
    /**
     * 获取主采样频率
     **/
    public static final String CMD_PI_GET_RATE = "gr";
    /**
     * 设置主采样频率
     **/
    public static final String CMD_PI_SET_RATE = "sr";
    /**
     * 获取数据输出途径
     */
    public static final String CMD_PI_GET_OUTPUT = "go";
    /**
     * 设置数据输出途径
     **/
    public static final String CMD_PI_SET_OUTPUT = "so";
    /**
     * 获取传感器的采样分频率
     **/
    public static final String CMD_PI_GET_RATEDIV = "grd";
    /**
     * 设置传感器的采样分频率
     **/
    public static final String CMD_PI_SET_RATEDIV = "srd";
    /**
     * 获取传感器的采样分频数
     **/
    public static final String CMD_PI_GET_RANGE = "grg";
    /**
     * 设置部分传感器量程
     **/
    public static final String CMD_PI_SET_RANGE = "srg";
    /**
     * 获取系统信息
     **/
    public static final String CMD_PI_GET_INFO = "gi";
    /**
     * 获取传感器列表
     **/
    public static final String CMD_PI_GET_SENSORLIST = "gsl";
    /**
     * 设置传感器为采集态
     **/
    public static final String CMD_PI_SET_SENSORACTIVE = "ssa";
    /**
     * 获取标记为采集态的传感器列表
     **/
    public static final String CMD_PI_GET_SENSORACTIVE = "gsa";
    /**
     * 获取命令说明
     **/
    public static final String CMD_PI_HELP = "help";
    private static final String TAG = BLEAdvanproPiDevice.class.getSimpleName();
    private static final UUID PRIMARY_SERVICE_UUID = UUID.fromString("00008001-0000-1000-8000-00805F9B34FB");

    private static final UUID PRIMARY_CHARACTERISTIC_UUID = UUID.fromString("00008086-0000-1000-8000-00805F9B34FB");

    private static short num = 1;

    final private StringBuffer buffer = new StringBuffer();
    private final Map<String, BLEAdvanproPiCommandResult> cache = new HashMap<>();
    protected boolean isInfoRead;
    private int readInfoTimeout = 30;//获取设备数据超时数据，单位秒
    private boolean ignore = false;
    private String[] names;//data表头

    private boolean isStarted;//是否启动状态
    private BLEAdvanproPiGetSettingResult setting = new BLEAdvanproPiGetSettingResult();

    final private BLEDeviceBluetoothGattCallback callback = new BLEDeviceBluetoothGattCallback() {
        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            super.onServicesDiscovered(gatt, status);
            if (DeviceMode.BootLoader.name().equals(getInfo(BLE_DEVICE_INFO_MODE))) {
                fireConnected(null);
            } else if (!isFindService(PRIMARY_SERVICE_UUID)) {
                //服务不存在
                fireConnected(new DeviceCallbackException(ERROR_SERVICE_NOT_EXISTS));
            } else if (!isFindCharacteristic(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID)) {
                //特征不存在
                fireConnected(new DeviceCallbackException(ERROR_CHARACTERISTIC_NOT_EXISTS));
            } else {
                BLEAdvanproPiDevice.this.notify(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID, true);

                fireConnected(null);
                UIUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        readDeviceInfo();
                    }
                });
            }
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);
            ALog.d(TAG, String.format("[%s] notify %s & %s [%s]", bluetoothDevice.getAddress(), characteristic.getService().getUuid().toString(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(characteristic.getValue())));

            if (!PRIMARY_CHARACTERISTIC_UUID.equals(characteristic.getUuid())) {
                return;
            }

            line(characteristic.getValue());
        }
    };

    @Override
    public void connect(int timeOut, DeviceConnectCallback cb) {
        isInfoRead = false;
        super.connect(timeOut, cb);
        //开启5分钟获取剩余电量定时器
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                if (isUpgrading() || !isConnected()) {
                    return;
                }
                command(CMD_PI_GET_INFO, new BLEAdvanproPiCommandParams("bat"), new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        String bat = result.getVal();
                        if (StringUtils.isNotEmpty(bat)) {
                            int battery = Integer.parseInt(bat);
                            info.put(BLE_DEVICE_INFO_BATTERY, String.valueOf(battery));

                            //日志埋点，电量改变
                            ALogger.i("battery", "id", getInfo(BLEDevice.BLE_DEVICE_INFO_ID), "bat", getInfo(BLE_DEVICE_INFO_BATTERY));

                            fireEvent(EVENT_BATTERY_CHANGE, null);
                        }
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {
                        error.printStackTrace();
                    }
                });
            }

        }, 300 * 1000, 300 * 1000);
    }

    /**
     * 刷新设备信息
     */
    public void readDeviceInfo() {
        command(CMD_PI_GET_INFO, new BLEAdvanproPiCommandParams(), readInfoTimeout, new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                String dID = result.get("dID");
                String sdt = result.get("sdt(MB)");
                String sdf = result.get("sdf(MB)");
                String hw = result.get("hw");
                String fw = result.get("fw");
                String state = result.get("state");//run/stop
                String bat = result.get("bat(%)");//bat
                String pID = result.get("pID");//pID C10B41
                info.put(BLE_DEVICE_INFO_ID, dID);
                info.put(BLE_DEVICE_INFO_HARDWARE, hw);
                info.put(BLE_DEVICE_INFO_FIRMWARE, fw);
                info.put(BLE_PI_INFO_SD_TOTAL_SIZE, sdt);
                info.put(BLE_PI_INFO_SD_FREE_SIZE, sdf);
                info.put(BLE_PI_INFO_STATE, state);
                info.put(BLE_DEVICE_INFO_CODE, "");
                info.put(BLE_DEVICE_INFO_BATTERY, bat);
                isInfoRead = true;


                //日志埋点，读取设备信息
                ALogger.i("dev_info_read", "addr", getInfo(BLEDevice.BLE_DEVICE_INFO_ADDRESS),
                        "id", getInfo(BLEDevice.BLE_DEVICE_INFO_ID),
                        "code", getInfo(BLE_DEVICE_INFO_CODE),
                        "fw", getInfo(BLE_DEVICE_INFO_FIRMWARE),
                        "hw", getInfo(BLE_DEVICE_INFO_HARDWARE));
                fireEvent(EVENT_READ_INFO, new BLEDeviceInfoResult(info));

                //connectCallback != null，回调连接成功
                if (isNeedConnectedCallback()) {
                    onConnectedCallback(null);
                }
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                //日志埋点，读取设备信息,失败
                ALogger.e("dev_info_read", "addr", getInfo(BLEDevice.BLE_DEVICE_INFO_ADDRESS),
                        "id", getInfo(BLEDevice.BLE_DEVICE_INFO_ID),
                        "code", getInfo(BLE_DEVICE_INFO_CODE),
                        "fw", getInfo(BLE_DEVICE_INFO_FIRMWARE),
                        "hw", getInfo(BLE_DEVICE_INFO_HARDWARE),
                        "err", String.valueOf(error.getCode()));
                fireEvent(EVENT_READ_INFO_ERROR, null);

                //connectCallback != null，回调连接成功
                if (isNeedConnectedCallback()) {
                    onConnectedCallback(null);
                }
            }
        });
    }

    @Override
    public boolean isStarted() {
        // 运行状态
        return isStarted;
    }

    @Override
    public boolean isApplicationMode() {
        return true;
    }

    public void setReadInfoTimeout(int readInfoTimeout) {
        this.readInfoTimeout = readInfoTimeout;
    }

    private void line(byte[] data) {
        if (data == null || data.length == 0) {
            return;
        }

        synchronized (buffer) {
            int index = ArrayUtils.indexOf(data, (byte) 0x0a);
            if (index >= 0) {
                parse(buffer.toString() + new String(Arrays.copyOfRange(data, 0, index)));
                buffer.delete(0, buffer.length());

                line(Arrays.copyOfRange(data, index + 1, data.length));

            } else {
                buffer.append(new String(data));
            }
        }

    }

    private void parse(String str) {
        ALog.d(TAG, String.format("[%s] parse string: %s", bluetoothDevice.getAddress(), str));
        Pattern pattern = Pattern.compile("^^\\[(\\d+)] (input|head|name|data|val|result|end)((\\t(.+))*)(\\r*)$");
        Matcher matcher = pattern.matcher(str);
        if (!matcher.find()) {
            return;
        }

        synchronized (cache) {
            String number = matcher.group(1);
            String key = matcher.group(2);

            if ("name".equals(key)) {
                names = matcher.group(3) != null
                        ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                return;
            }

            if ("data".equals(key)) {
                String[] datas = matcher.group(3) != null ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                if (isStarted()) {
                    // 开启才发送数据
                    fireEvent(EVENT_DEVICE_DATA, new BLEAdvanproPiDataEventResult(Integer.parseInt(number), names, datas));
                }
                return;
            }

            BLEAdvanproPiCommandResult result = cache.get(number);
            if (result == null) {
                return;
            }

            switch (key) {
                case "input":
                    break;
                case "head":
                    String[] heads = matcher.group(3) != null
                            ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                    result.setHeads(heads);
                    break;
                case "val":
                    String[] vals = matcher.group(3) != null
                            ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                    if (vals.length == 1) {
                        result.setVal(vals[0]);
                    } else {
                        result.setVals(vals);
                    }
                    break;
                case "name":
                    break;
                case "data":
                    break;
                case "result":
                    result.setCode(Integer.parseInt(matcher.group(3).replace("[", "").replace("]", "").trim()));
                    break;
                case "end":
                    synchronized (cache) {
                        cache.remove(number);
                    }
                    fireCommand(number, result);
                    break;
                default:
            }

        }

    }

    @Override
    protected void beforeFireCommand(String number, DeviceCommandResult result1) {
        BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) result1;
        if (!result.isSuccess()) {
            // 命令不成功，直接 return
            return;
        }
        switch (result.getKey()) {
            case CMD_PI_GET_DATA_NAME:
                names = result.getVals();
                break;
            case CMD_PI_START:
                info.put(BLE_PI_INFO_STATE, "run");
                // 修改设置方法
                isStarted = true;
                break;
            case CMD_PI_STOP:
                info.put(BLE_PI_INFO_STATE, "stop");
                // 修改设置方法
                isStarted = false;
                break;
            case CMD_PI_GET_INFO:
                String state = result.get("state");//run/stop
                if (StringUtils.isNotEmpty(state)) {
                    info.put(BLE_PI_INFO_STATE, state);
                    // 修改设置方法
                    isStarted = "run".equalsIgnoreCase(state);
                }
                break;
            default:
                break;
        }
    }

    @Override
    public synchronized void command(String key, DeviceCommandParams params, int timeOut, DeviceCommandCallback cb) {
        if (num == Short.MAX_VALUE) {
            num = 0;
        }
        short n = num++;

        BLEAdvanproPiCommandParams cp = (BLEAdvanproPiCommandParams) params;

        cp.setKey(key);
        cp.setCmdNumber(n);
        cache.put(String.valueOf(n), new BLEAdvanproPiCommandResult(key, n));

        BLEDevice.CommandChannelInfo info = new BLEDevice.CommandChannelInfo(String.valueOf(n), PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID);

        if (registerCommand(info, timeOut, cb)) {
            write(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID, params.toBytes());
            afterRegisterCommand(info.getKey(), cb);
        }
    }

    // 开发版（DeviceType.SDP）数据类型：BLEAdvanproPiDataEventResult
    @Override
    public void start(final DeviceCommandParams params, final @NonNull DeviceCommandCallback cb) {
        command(CMD_PI_START, params, new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
            @Override
            public void onSuccess(Device device, final BLEAdvanproPiCommandResult startResult) {
                if (startResult.isSuccess()) {
                    // 命令成功
                    isStarted = true;

                    // 获取启动配置
                    command(CMD_PI_GET_START_CONFIG, new BLEAdvanproPiCommandParams(), new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                        @Override
                        public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                            if (result.getHeads() != null) {
                                if (setting.getStartCfg() == null) {
                                    setting.setStartCfg(new HashMap<String, String>());
                                }
                                for (String key : result.getHeads()) {
                                    String val = result.get(key);
                                    if (val != null) setting.getStartCfg().put(key, val);
                                }
                                cb.onSuccess(device, startResult);
                            }
                        }

                        @Override
                        public void onError(Device device, DeviceCallbackException error) {
                            cb.onSuccess(device, startResult);
                        }
                    });
                } else {
                    cb.onError(device, new DeviceCallbackException(0, "start failure , errorCode = " + startResult.getCode()));
                }
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                cb.onError(device, error);
            }
        });
    }

    @Override
    public void stop(DeviceCommandParams params, final @NonNull DeviceCommandCallback cb) {
        if (params == null) {
            params = new BLEAdvanproPiCommandParams();
        }
        command(CMD_PI_STOP, params, new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                if (result.isSuccess()) {
                    // 命令成功
                    isStarted = false;
                    if (setting.getStartCfg() != null) {
                        //设置启动配置
                        setting.setStartCfg(null);
                    }
                    cb.onSuccess(device, result);
                } else {
                    cb.onError(device, new DeviceCallbackException(0, "stop failure , errorCode = " + result.getCode()));
                }
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                cb.onError(device, error);
            }
        });
    }


    @Override
    public void setting(final DeviceSettingParams params, final DeviceSettingCallback cb) {
        if (!DeviceType.SDP.name().equals(getInfo(BLE_DEVICE_INFO_TYPE))) {
            super.setting(params, cb);
            return;
        }
        //开发版 settings 设置
        BLEAdvanproPiSettingsTask task = new BLEAdvanproPiSettingsTask(this, 20, (BLEAdvanproPiSettingParams) params, new DeviceSettingCallback() {
            @Override
            public void onSuccess(Device device, DeviceSettingResult result) {
                BLEAdvanproPiSettingParams settingParams = (BLEAdvanproPiSettingParams) params;

                // 设置成功修改设置参数
                setting.setAddress(getAddress());
                if (settingParams.getRate() > 0)
                    setting.setRate(settingParams.getRate());
                if (settingParams.getSensorsActive() != null)
                    setting.setSensorsActive(settingParams.getSensorsActive());
                if (settingParams.getRange() != null)
                    setting.setRange(settingParams.getRange());
                if (settingParams.getRateDiv() != null)
                    setting.setRateDiv(settingParams.getRateDiv());
                if (settingParams.getOutput() != null)
                    setting.setOutput(settingParams.getOutput());
                cb.onSuccess(device, result);

            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                cb.onError(device, error);

            }
        });
        task.start();
    }

    @Override
    public void readSetting(final DeviceSettingCallback cb) {
        if (DeviceType.SDP.name().equals(getInfo(BLE_DEVICE_INFO_TYPE))) {
            //开发板 setting
            BLEAdvanproPiGetSettingsTask task = new BLEAdvanproPiGetSettingsTask(this, 20, new DeviceSettingCallback() {
                @Override
                public void onSuccess(Device device, DeviceSettingResult result) {
                    BLEAdvanproPiGetSettingResult settingResult = ((BLEAdvanproPiGetSettingResult) result);
                    // 修改设置信息
                    setting.setAddress(getAddress());
                    setting.setDataName(settingResult.getDataName());
                    setting.setRate(settingResult.getRate());
                    setting.setSensorsActive(settingResult.getSensorsActive());
                    setting.setSensors(settingResult.getSensors());
                    setting.setRange(settingResult.getRange());
                    setting.setRateDiv(settingResult.getRateDiv());
                    setting.setOutput(settingResult.getOutput());
                    // 回调
                    cb.onSuccess(device, setting);
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    cb.onError(device, error);

                }
            });
            task.start();
        } else {
            //医疗鞋垫 setting 直接返回空 map
            cb.onSuccess(this, null);
        }
    }

    @Override
    public BLEAdvanproPiGetSettingResult getSetting() {
        return setting;
    }

    private class BLEAdvanproPiSettingsTask {
        private Device device;
        private int timeOut;
        private DeviceSettingCallback callback;
        private BLEAdvanproPiSettingParams params = null;

        public BLEAdvanproPiSettingsTask(Device device, int timeOut, BLEAdvanproPiSettingParams params, DeviceSettingCallback callback) {
            this.device = device;
            this.timeOut = timeOut;
            this.callback = callback;
            this.params = params;
        }

        List<DeviceCommandSyncTask> tasks;

        public void start() {
            //初始化 settings task

            tasks = new ArrayList<>();
            if (params.getRate() > 0) {
                tasks.add(DeviceCommandSyncTask.create(device, CMD_PI_SET_RATE, new BLEAdvanproPiCommandParams(String.valueOf(params.getRate())), timeOut));
            }
            if (params.getOutput() != null) {
                tasks.add(DeviceCommandSyncTask.create(device, CMD_PI_SET_OUTPUT, new BLEAdvanproPiCommandParams(params.getOutput().toArray(new String[params.getOutput().size()])), timeOut));
            }

            if (params.getRateDiv() != null) {
                tasks.add(DeviceCommandSyncTask.create(device, CMD_PI_SET_RATEDIV, new BLEAdvanproPiCommandParams(params.getRateDiv()), timeOut));
            }

            if (params.getRange() != null) {
                tasks.add(DeviceCommandSyncTask.create(device, CMD_PI_SET_RANGE, new BLEAdvanproPiCommandParams(params.getRange()), timeOut));
            }
            if (params.getSensorsActive() != null) {
                tasks.add(DeviceCommandSyncTask.create(device, CMD_PI_SET_SENSORACTIVE, new BLEAdvanproPiCommandParams(params.getSensorsActive().toArray(new String[params.getSensorsActive().size()])), timeOut));
            }
            if (tasks.size() == 0) {
                callback.onSuccess(device, null);
                return;
            }
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        for (DeviceCommandSyncTask task : tasks) {
                            // 执行
                            task.execute();
                            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.getResult();

                            if (!result.isSuccess()) {
                                // 修改不成功，直接抛异常
                                throw new DeviceCallbackException(0, "settings failure ,errorCode=" + result.getCode());
                            }
                        }
                        // 设置成功
                        callback.onSuccess(device, null);
                    } catch (DeviceCallbackException e) {
                        // 设置失败
                        e.printStackTrace();
                        callback.onError(device, e);
                    }
                }
            }).start();


        }

    }


    private class BLEAdvanproPiGetSettingsTask {
        private Device device;
        private int timeOut;
        private DeviceSettingCallback callback;

        public BLEAdvanproPiGetSettingsTask(Device device, int timeOut, DeviceSettingCallback callback) {
            this.device = device;
            this.timeOut = timeOut;
            this.callback = callback;
        }

        BLEAdvanproPiGetSettingResult settingsResult = new BLEAdvanproPiGetSettingResult();

        private void start() {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        getInfo();
                        getDataName();
                        getRate();
                        getOutput();
                        getRateDiv();
                        getSensorList();
                        getRange();
                        getSensorActive();
                        getStartConfig();
                        ALog.d(TAG, "settingsMap:" + JSON.toJSONString(settingsResult));
                        callback.onSuccess(device, settingsResult);
                    } catch (DeviceCallbackException e) {
                        e.printStackTrace();
                        callback.onError(device, e);
                    }
                }
            }).start();
        }

        private void getInfo() throws DeviceCallbackException {
            //获取设备信息
            DeviceCommandSyncTask task = new DeviceCommandSyncTask().create(device, CMD_PI_GET_INFO, new BLEAdvanproPiCommandParams(), timeOut);//获取 info
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            String state = result.get("state");//启动状态 run/stop

            // 启动状态 1启动 ,0停止
            isStarted = "run".equals(state);
        }

        private void getDataName() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_DATA_NAME, new BLEAdvanproPiCommandParams(), timeOut);// 获取Data Name
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            settingsResult.setDataName(Arrays.asList(result.getVals()));
        }

        private void getRate() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_RATE, new BLEAdvanproPiCommandParams(), timeOut);// 获取采样率
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            String val = result.getVal();
            settingsResult.setRate(Integer.parseInt(val));

        }

        private void getRange() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_RANGE, new BLEAdvanproPiCommandParams(), timeOut); // 获取量程
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            Map<String, String> map = new HashMap<>();
            for (int i = 0; i < result.getHeads().length; ++i) {
                String heads = result.getHeads()[i];
                String val = result.getVals()[i];
                map.put(heads, val);
            }
            settingsResult.setRange(map);
        }


        private void getRateDiv() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_RATEDIV, new BLEAdvanproPiCommandParams(), timeOut); // 获取分频数
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            Map<String, String> map = new HashMap<>();
            for (int i = 0; i < result.getHeads().length; ++i) {
                String heads = result.getHeads()[i];
                String val = result.getVals()[i];
                map.put(heads, val);
            }
            settingsResult.setRateDiv(map);
        }


        private void getSensorList() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_SENSORLIST, new BLEAdvanproPiCommandParams(), timeOut);// 获取传感器列表
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            settingsResult.setSensors(Arrays.asList(result.getVals()));
        }

        private void getOutput() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_OUTPUT, new BLEAdvanproPiCommandParams(), timeOut);// 获取输出路径
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            settingsResult.setOutput(Arrays.asList(result.getVals()));

        }

        private void getSensorActive() throws DeviceCallbackException {
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_SENSORACTIVE, new BLEAdvanproPiCommandParams(), timeOut);// 获取当前激活的传感器列表
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            settingsResult.setSensorsActive(Arrays.asList(result.getVals()));
        }

        private void getStartConfig() throws DeviceCallbackException {
            if (!isStarted()) {
                //没有启动直接返回
                return;
            }
            // 启动中,获取启动配置
            DeviceCommandSyncTask task = DeviceCommandSyncTask.create(device, CMD_PI_GET_START_CONFIG, new BLEAdvanproPiCommandParams(), timeOut);// 获取当前激活的传感器列表
            BLEAdvanproPiCommandResult result = (BLEAdvanproPiCommandResult) task.execute().getResult();
            Map<String, String> map = new HashMap<>();
            if (result.getHeads() != null) {
                for (int i = 0; i < result.getHeads().length; ++i) {
                    String heads = result.getHeads()[i];
                    String val = result.getVals()[i];
                    map.put(heads, val);
                }
            }
            settingsResult.setStartCfg(map);

        }


    }

    @Override
    public boolean upgrade(File file, DeviceUpgradeCallback cb) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isUpgrading() {
        return false;
    }

    @Override
    public boolean isInfoRead() {
        return isInfoRead;
    }

    @Override
    protected BluetoothGattCallback getBluetoothGattCallback() {
        return callback;
    }

    @Override
    protected CommandChannelInfo getCommandChannelInfo(String key) {
        return new CommandChannelInfo(key, PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID);
    }

    @Override
    protected void afterRegisterCommand(String key, DeviceCommandCallback cb) {

    }

    @Override
    protected void onCommandTimeOut(String key) {
        synchronized (cache) {
            cache.remove(key);
        }
    }

}
