package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.support.annotation.NonNull;

import java.io.File;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.Map;
import java.util.TimerTask;
import java.util.UUID;

import hk.advanpro.android.core.utils.UIUtils;
import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceSettingCallback;
import hk.advanpro.android.sdk.device.callback.DeviceUpgradeCallback;
import hk.advanpro.android.sdk.device.ota.DeviceFirmwareUpgradeCallback;
import hk.advanpro.android.sdk.device.ota.csr.BLEInsoleFirmwareUpgrade;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleCollectStatusCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleStartCommandParams;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleStopCommandParams;
import hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectStartOrStopCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectStatusCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleEnableAccCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleGetSettingResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealGaitEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealStepEventResult;
import hk.advanpro.android.sdk.utils.CommonUtils;

import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buidSectionStepResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildACCEventResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildClearAllResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildClearDataResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildCollectEventResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildDisableAccResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildGaitEventResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildHeartbeatCommand;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildHisoryGaitResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildHistoryStepResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildRawGaitResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildReadIdResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildStepEventResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildStitaProtectResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildStopSendResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildSyncTimeResult;
import static hk.advanpro.android.sdk.device.protocol.BLEInsoleProtocol.buildWriteIdResult;

/**
 * 智能鞋垫设备实现类
 * Created by yejianfei on 2017/6/7.
 */

@SuppressWarnings("unchecked")
public class BLEInsoleDevice extends DefaultBLEDevice {

    /**
     * 鞋垫左右脚标示
     * 0左脚 1右脚
     */
    public static final String BLE_INSOLE_INFO_FOOT = "BLE_INSOLE_INFO_FOOT";

    /**
     * 实时步态数据
     */
    public static final String EVENT_INSOLE_REAL_STEP = "EVENT_INSOLE_REAL_STEP";

    /**
     * 实时步态数据
     */
    public static final String EVENT_INSOLE_REAL_GAIT = "EVENT_INSOLE_REAL_GAIT";

    /**
     * 实时三轴数据
     */
    public static final String EVENT_INSOLE_REAL_ACC = "EVENT_INSOLE_REAL_ACC";
    /**
     * 历史步态数据
     */
    public static final String CMD_INSOLE_HISTORY_GAIT = "CMD_INSOLE_HISTORY_GAIT";
    /**
     * 历史步数数据
     */
    public static final String CMD_INSOLE_HISTORY_STEP = "CMD_INSOLE_HISTORY_STEP";
    /**
     * 历史分段步数
     */
    public static final String CMD_INSOLE_SECTION_STEP = "CMD_INSOLE_SECTION_STEP";
    /**
     * 原始步态数据
     */
    public static final String CMD_INSOLE_RAW_GAIT = "CMD_INSOLE_RAW_GAIT";
    /**
     * 时间设置
     */
    public static final String CMD_INSOLE_SYNC_TIME = "CMD_INSOLE_SYNC_TIME";
    /**
     * 写入ID
     */
    public static final String CMD_INSOLE_WRITE_ID = "CMD_INSOLE_WRITE_ID";
    /**
     * 读取ID
     */
    public static final String CMD_INSOLE_READ_ID = "CMD_INSOLE_READ_ID";

    /**
     * 停止发包命令
     */
    public static final String CMD_INSOLE_STOP_SEND = "CMD_INSOLE_STOP_SEND";

    /**
     * 清除某天的设备数据
     */
    public static final String CMD_INSOLE_CLEAR_DATA = "CMD_INSOLE_CLEAR_DATA";

    /**
     * 清除全部数据
     */
    public static final String CMD_INSOLE_CLEAR_ALL = "CMD_INSOLE_CLEAR_ALL";

    /**
     * 开启发送三轴数据
     */
    public static final String CMD_INSOLE_ENABLE_ACC = "CMD_INSOLE_ENABLE_ACC";

    /**
     * 关闭发送三轴数据
     */
    public static final String CMD_INSOLE_DISABLE_ACC = "CMD_INSOLE_DISABLE_ACC";

    /**
     * 获取原始数据采集状态
     */
    public static final String CMD_INSOLE_COLLECT_STATUS = "CMD_INSOLE_COLLECT_STATUS";

    /**
     * 开启/关闭原始数据采集
     */
    public static final String CMD_INSOLE_COLLECT_START_OR_STOP = "CMD_INSOLE_COLLECT_START_OR_STOP";

    /**
     * 升级固件前需要保护固件现场
     */
    public static final String CMD_INSOLE_FIRMWARE_UPGRADE_PROTECT = "CMD_INSOLE_FIRMWARE_UPGRADE_PROTECT";

    private static final UUID PRIMARY_SERVICE_UUID = UUID.fromString("00001C00-D102-11E1-9B23-000EFB00DBDB");

    private static final UUID PRIMARY_CHARACTERISTIC_UUID = UUID.fromString("00001C01-D102-11E1-9B23-000EFB00DBDB");

    private static final UUID PRIMARY_NOTIFY_CHARACTERISTIC_UUID = UUID.fromString("00001C0F-D102-11E1-9B23-000EFB00DBDB");

    private static final UUID SECONDARY_CHARACTERISTIC_UUID = UUID.fromString("00001C02-D102-11E1-9B23-000EFB00DBDB");

    private static final UUID SECONDARY_NOTIFY_CHARACTERISTIC_UUID = UUID.fromString("00001C0E-D102-11E1-9B23-000EFB00DBDB");

    private static final String TAG = BLEInsoleDevice.class.getSimpleName();
    protected boolean isUpgrading;//是否正在升级
    private BluetoothGattCallback callback;
    private Map<UUID, SectionByteBuffer> sectionByteBufferMap = new HashMap<>();

    private boolean isStarted;
    private BLEInsoleGetSettingResult setting = new BLEInsoleGetSettingResult();

    @Override
    public synchronized void open(DeviceScanResult result) {
        super.open(result);
        HashMap<String, String> description = this.result.getRecord().getManufacturer().getDescription();
        if (description != null)
            this.info.put(BLE_INSOLE_INFO_FOOT, description.get("foot"));
    }

    @Override
    protected CommandChannelInfo getCommandChannelInfo(String key) {
        CommandChannelInfo info = super.getCommandChannelInfo(key);
        if (info == null) {
            UUID uuid;
            switch (key) {
                case CMD_INSOLE_SYNC_TIME:
                case CMD_INSOLE_STOP_SEND:
                case CMD_INSOLE_CLEAR_DATA:
                case CMD_INSOLE_DISABLE_ACC:
                case CMD_INSOLE_ENABLE_ACC:
                case CMD_INSOLE_CLEAR_ALL:
                case CMD_INSOLE_WRITE_ID:
                case CMD_INSOLE_READ_ID:
                case CMD_INSOLE_FIRMWARE_UPGRADE_PROTECT:
                case CMD_INSOLE_COLLECT_STATUS:
                case CMD_INSOLE_COLLECT_START_OR_STOP:
                    uuid = PRIMARY_CHARACTERISTIC_UUID;
                    break;
                default:
                    uuid = SECONDARY_CHARACTERISTIC_UUID;
            }

            info = new CommandChannelInfo(key, PRIMARY_SERVICE_UUID, uuid);
        }

        return info;
    }

    @Override
    protected void afterRegisterCommand(String key, DeviceCommandCallback cb) {
        if (key.equals(CMD_INSOLE_ENABLE_ACC)) {
            // 开启三轴命令没有回复，模拟命令回复
            fireCommand(CMD_INSOLE_ENABLE_ACC, new BLEInsoleEnableAccCommandResult());
        }
    }

    @Override
    public void connect(int timeOut, DeviceConnectCallback cb) {
        super.connect(timeOut, cb);
        //发送心跳包定时器
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                if (!isUpgrading() && isConnected()) {
                    ALog.d(TAG, String.format("[%s] send to heart beat.", bluetoothDevice.getAddress()));
                    write(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID, buildHeartbeatCommand());
                }
            }
        }, 5000, 10000);
    }

    @Override
    protected void onCommandTimeOut(String key) {

    }

    // 智能鞋垫(DeviceType.Insole)数据类型 :步数（BLEInsoleRealStepEventResult），步态（BLEInsoleRealGaitEventResult），三轴数据（BLEInsoleCollectDataEventResult）
    @Override
    public void start(final DeviceCommandParams params, @NonNull final DeviceCommandCallback cb) {
        command(CMD_INSOLE_COLLECT_START_OR_STOP, params, new DeviceCommandCallback<BLEInsoleCollectStartOrStopCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleCollectStartOrStopCommandResult result) {
                if (result.isSuccess()) {
                    // 设置 setting
                    BLEInsoleStartCommandParams insoleStartParams = (BLEInsoleStartCommandParams) params;
                    setting.setRate(insoleStartParams.getRate());
                    // 修改启动状态
                    isStarted = true;
                    cb.onSuccess(device, result);
                } else {
                    cb.onError(device, new DeviceCallbackException(0, "start errorCode = " + result.getErrorCode()));
                }
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                cb.onError(device, error);
            }
        });
    }

    @Override
    public void stop(DeviceCommandParams params, final DeviceCommandCallback cb) {
        if (params == null) {
            params = new BLEInsoleStopCommandParams();
        }
        command(CMD_INSOLE_COLLECT_START_OR_STOP, params, new DeviceCommandCallback<BLEInsoleCollectStartOrStopCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleCollectStartOrStopCommandResult result) {
                if (result.isSuccess()) {
                    // 修改启动状态
                    isStarted = false;
                    cb.onSuccess(device, result);
                } else {
                    cb.onError(device, new DeviceCallbackException(0, "start errorCode = " + result.getErrorCode()));
                }
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                cb.onError(device, error);
            }
        });
    }


    @Override
    public void readSetting(final DeviceSettingCallback cb) {
        //鞋垫设置
        command(BLEInsoleDevice.CMD_INSOLE_COLLECT_STATUS, new BLEInsoleCollectStatusCommandParams(), new DeviceCommandCallback<BLEInsoleCollectStatusCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEInsoleCollectStatusCommandResult result) {
                cb.onSuccess(device, setting);
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                cb.onError(device, error);
            }
        });
    }

    @Override
    protected void beforeFireCommand(String key, DeviceCommandResult result) {
        switch (key) {
            case BLEInsoleDevice.CMD_INSOLE_COLLECT_STATUS: {
                BLEInsoleCollectStatusCommandResult settingResult = (BLEInsoleCollectStatusCommandResult) result;
                // 缓存设备设置信息
                setting.setAddress(getAddress());
                setting.setRate(settingResult.getRate());
                // 修改启动状态
                isStarted = settingResult.getStatus() == 1;
                break;
            }
        }
        super.beforeFireCommand(key, result);
    }

    @Override
    public boolean isStarted() {
        //启动状态
        return isStarted;
    }

    @Override
    public BLEInsoleGetSettingResult getSetting() {
        return setting;
    }


    @Override
    protected BluetoothGattCallback getBluetoothGattCallback() {
        if (callback == null) {
            callback = new DefaultBluetoothGattCallback() {
                @Override
                public void onServicesDiscovered(BluetoothGatt gatt, int status) {
                    super.onServicesDiscovered(gatt, status);
                    BLEInsoleDevice.this.notify(PRIMARY_SERVICE_UUID, PRIMARY_NOTIFY_CHARACTERISTIC_UUID, true);
                    BLEInsoleDevice.this.notify(PRIMARY_SERVICE_UUID, SECONDARY_NOTIFY_CHARACTERISTIC_UUID, true);
                }

                @Override
                public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
                    super.onCharacteristicChanged(gatt, characteristic);

                    byte[] data = characteristic.getValue();
                    if (data.length < 2) {
                        return;
                    }

                    // 处理命令及消息头
                    byte head = data[0];
                    byte flag = data[1];
                    if (head == (byte) 0x26) {
                        flag = head;
                        head = (byte) 0x06;
                    }
                    if (head == (byte) 0xcc) {
                        flag = head;
                    }
                    short commandKey = (short) (((head << 8) & 0xffff) | (flag & 0xff));

                    ALog.d(TAG, String.format("notify command key by %s and bytes is %s",
                            Integer.toHexString(commandKey & 0xffff), CommonUtils.bytesToHexString(new byte[]{head, flag})));


                    // 记录分段数据
                    SectionByteBuffer sectionByteBuffer = sectionByteBufferMap.get(characteristic.getUuid());
                    if (sectionByteBuffer != null && sectionByteBuffer.append(data) && sectionByteBuffer.isEnd(data)) {
                        // 分段数据结束
                        sectionByteBufferMap.remove(characteristic.getUuid());
                        ALog.d(TAG, String.format("[%s] end buffer", bluetoothDevice.getAddress()));
                        short key = sectionByteBuffer.getKey();
                        ByteBuffer bb = sectionByteBuffer.getData();
                        switch (key) {
                            case (short) 0x0604:
                                fireCommand(CMD_INSOLE_SECTION_STEP, buidSectionStepResult(bb.array()));
                                break;
                            case (short) 0x0607:
                                fireCommand(CMD_INSOLE_RAW_GAIT, buildRawGaitResult(bb.array()));
                                break;
                            case (short) 0xaabb:
                                fireCommand(CMD_INSOLE_SYNC_TIME, buildSyncTimeResult(bb.array()));
                                break;
                            default:
                                break;
                        }
                    }


                    // 消息及控制推送协议处理。
                    if (characteristic.getUuid().equals(PRIMARY_NOTIFY_CHARACTERISTIC_UUID)) {
                        ALog.d(TAG, String.format("[%s] notify real data %s & %s [%s]", bluetoothDevice.getAddress(), characteristic.getService().getUuid().toString(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(data)));

                        switch (commandKey) {
                            case (short) 0x0603:
                                BLEInsoleRealGaitEventResult gait = buildGaitEventResult(data);
                                fireEvent(EVENT_INSOLE_REAL_GAIT, gait);


                                if (isStarted()) {
                                    // 启动过程中发送 device data , 鞋垫 ：步数，步态，三轴
                                    fireEvent(EVENT_DEVICE_DATA, gait);
                                }
                                break;
                            case (short) 0x0626:
                                BLEInsoleRealStepEventResult step = buildStepEventResult(data);
                                fireEvent(EVENT_INSOLE_REAL_STEP, step);
                                if (isStarted()) {
                                    // 启动过程中发送 device data , 鞋垫 ：步数，步态，三轴
                                    fireEvent(EVENT_DEVICE_DATA, step);
                                }
                                break;
                            case (short) 0xaafd:
                                fireEvent(EVENT_INSOLE_REAL_ACC, buildACCEventResult(data));
                                break;
                            case (short) 0xbbaa:
                                fireCommand(data.length == 4 ? CMD_INSOLE_WRITE_ID : CMD_INSOLE_READ_ID, data.length == 4 ? buildWriteIdResult(data) : buildReadIdResult(data));
                                break;
                            case (short) 0xaabb:
                                ByteBuffer header = ByteBuffer.wrap(data);
                                int length = header.get(2);
                                header.position(0);
                                sectionByteBufferMap.put(PRIMARY_NOTIFY_CHARACTERISTIC_UUID, new SectionByteBuffer(PRIMARY_NOTIFY_CHARACTERISTIC_UUID, header, length));
                                break;
                            case (short) 0xaacc:
                                fireCommand(CMD_INSOLE_STOP_SEND, buildStopSendResult(data));
                                break;
                            case (short) 0xaadd:
                                fireCommand(CMD_INSOLE_CLEAR_DATA, buildClearDataResult(data));
                                break;
                            case (short) 0xaaed:
                                fireCommand(CMD_INSOLE_DISABLE_ACC, buildDisableAccResult(data));
                                break;
                            case (short) 0xaaff:
                                fireCommand(CMD_INSOLE_CLEAR_ALL, buildClearAllResult(data));
                                break;
                            case (short) 0xaaee:
                                if ((data[2] & 0xFF) == 0) {// 0 结束现场保护 1开启现场保护
                                    fireCommand(CMD_INSOLE_FIRMWARE_UPGRADE_PROTECT, buildStitaProtectResult(data));
                                }
                                break;
                            case (short) 0xcccc:
                                //开启/关闭 原始数据采集功能
                                if (data.length == 2) {
                                    fireCommand(CMD_INSOLE_COLLECT_START_OR_STOP, BLEInsoleProtocol.buildCollectStartOrStopResult(data));
                                }
                                //获取原始数据采集状态
                                if (data.length == 3) {
                                    fireCommand(CMD_INSOLE_COLLECT_STATUS, BLEInsoleProtocol.buildCollectStatusResult(data));
                                }
                                break;
                            default:
                                //原始数据采集
                                if (((data[0] >> 4) & 0xF) == 0xF) {
                                    if (isStarted()) {
                                        // 启动过程中发送 device data , 鞋垫 ：步数，步态，三轴
                                        fireEvent(EVENT_DEVICE_DATA, buildCollectEventResult(data));
                                    }
                                }
                                break;
                        }
                    }

                    // 历史命令回复处理。
                    if (characteristic.getUuid().equals(SECONDARY_NOTIFY_CHARACTERISTIC_UUID)) {
                        ALog.d(TAG, String.format("[%s]  history result %s & %s  [%s]", bluetoothDevice.getAddress(), characteristic.getService().getUuid().toString(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(data)));

                        switch (commandKey) {
                            case (short) 0x0601:
                                fireCommand(CMD_INSOLE_HISTORY_GAIT, buildHisoryGaitResult(data));
                                break;
                            case (short) 0x0602:
                                fireCommand(CMD_INSOLE_HISTORY_STEP, buildHistoryStepResult(data));
                                break;
                            case (short) 0x0603:
                            case (short) 0x0604:
                            case (short) 0x0607:
                                ByteBuffer header = ByteBuffer.wrap(data);
                                byte[] bytes = new byte[]{0x0, 0x00, 0x00, 0x00};
                                header.position(4);
                                header.get(bytes, 0, 3);
                                int byteBufferLength = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt();
                                header.position(0);
                                sectionByteBufferMap.put(SECONDARY_NOTIFY_CHARACTERISTIC_UUID, new SectionByteBuffer(SECONDARY_NOTIFY_CHARACTERISTIC_UUID, header, byteBufferLength));
                                ALog.d(TAG, String.format("[%s] begin buffer, length %d", bluetoothDevice.getAddress(), byteBufferLength));
                                break;
                            default:
                        }
                    }
                }
            };
        }

        return callback;
    }

    @Override
    public boolean upgrade(@NonNull final File file, @NonNull final DeviceUpgradeCallback cb) {
        if (this.isUpgrading()) {
            //不能重复升级
            cb.onError(this, new DeviceCallbackException(Device.ERROR_ALREADY_UPGRADE));
            return false;
        }
        if (!file.exists()) {
            //文件不存在
            cb.onError(this, new DeviceCallbackException(Device.ERROR_UPGRADE_FILE_NOT_EXISTS));
            return false;
        }
        if (!this.isConnected()) {
            //升级之前先连接设备
            cb.onError(this, new DeviceCallbackException(Device.ERROR_IS_DISCONNECTED));
            return false;
        }

        DeviceFirmwareUpgradeCallback callback = new DeviceFirmwareUpgradeCallback() {
            @Override
            public void start(Device device) {
            }

            @Override
            public void onStatusChange(Device device, int status) {

            }

            @Override
            public void onProgress(final Device device, final long total, final long progress) {
                UIUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        cb.onProgress(device, total, progress);
                    }
                });
            }

            @Override
            public void onSuccess(final Device device) {
                isUpgrading = false;
                UIUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        cb.onSuccess(device, null);
                    }
                });
            }

            @Override
            public void onError(final Device device, final DeviceCallbackException error) {
                isUpgrading = false;
                UIUtils.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        cb.onError(device, error);
                    }
                });
            }
        };
        BLEInsoleFirmwareUpgrade bleInsoleFirmwareUpgrade = new BLEInsoleFirmwareUpgrade();
        isUpgrading = true;
        bleInsoleFirmwareUpgrade.start(AdvanproAndroidSDK.getAppContext(), this, file, callback);

        return true;
    }

    @Override
    public boolean isUpgrading() {
        return isUpgrading;
    }

    static class SectionByteBuffer {
        ByteBuffer header;//开始接收分段第一个包
        ByteBuffer buffer;//分段分段中间接收包
        UUID uuid;//通道UUID
        int length;//分段包总字节数

        public SectionByteBuffer(UUID uuid, ByteBuffer header, int length) {
            this.uuid = uuid;
            this.header = header;
            this.length = length;
            this.buffer = ByteBuffer.allocate(length);
        }

        boolean isEnd(byte[] data) {
            if (data.length == 3) {
                ByteBuffer end = ByteBuffer.wrap(data);
                byte[] bytes = new byte[]{0x00, 0x00, 0x00, 0x00};
                end.get(bytes, 0, 3);
                if ((ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt() & length) == 0) {
                    return true;
                }
            }
            return false;
        }

        public short getKey() {
            return (short) (((header.get(0) << 8) & 0xffff) | (header.get(1) & 0xff));
        }

        ByteBuffer getData() {
            ByteBuffer bb = ByteBuffer.allocate(header.capacity() + buffer.capacity());
            bb.put(header.array()).put(buffer.array());
            return bb;
        }

        boolean append(byte[] data) {
            if (buffer.capacity() > 0 && buffer.capacity() - buffer.position() >= data.length) {
                buffer.put(data);
                return true;
            }
            return false;
        }

    }
}
