package hk.advanpro.android.sdk.device.callback;

import hk.advanpro.android.core.utils.UIUtils;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.DeviceManager;
import hk.advanpro.android.sdk.device.result.BatchDeviceConnectResult;

/**
 * 批量设备连接主线程回调
 * Created by jalin on 2017/11/20.
 */

public abstract class MainThreadBatchDeviceConnectCallback<T extends BatchDeviceConnectResult> implements BatchDeviceConnectCallback<T> {


    //当扫描设备成功，自动初始化设备回调
    public abstract void onMainThreadCreated(DeviceManager.DeviceConnection deviceConnection, Device device);

    //连接设备成功设备回调
    public abstract void onMainThreadConnectSuccess(DeviceManager.DeviceConnection deviceConnection, Device device);

    //结束并返回结果
    public abstract void onMainThreadFinish(T result);

    //报错
    public abstract void onMainThreadError(DeviceCallbackException e);

    //当扫描设备成功，自动初始化设备回调
    final public void onCreated(final DeviceManager.DeviceConnection deviceConnection, final Device device) {
        UIUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                onMainThreadCreated(deviceConnection, device);
            }
        });
    }

    //连接设备成功设备回调
    final public void onConnectSuccess(final DeviceManager.DeviceConnection deviceConnection, final Device device) {
        UIUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                onMainThreadConnectSuccess(deviceConnection, device);
            }
        });
    }

    //结束并返回结果
    final public void onFinish(final T result) {
        UIUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                onMainThreadFinish(result);
            }
        });
    }

    //报错
    final public void onError(final DeviceCallbackException e) {
        UIUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                onMainThreadError(e);
            }
        });
    }
}
