package hk.advanpro.android.sdk.device.callback;


import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;

/**
 * 于设备数据发送命令响应的主线程回调接口
 * 可以直接在回调中更新UI
 * Created by jalin on 2017/7/7.
 */

public abstract class MainThreadDeviceCommandCallback<T extends DeviceCommandResult> implements DeviceCommandCallback<T> {
    private final static Handler handler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
        }
    };

    @Override
    public void onSuccess(final Device device, final T result) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                onMainThreadSuccess(device, result);
            }
        });
    }

    @Override
    public void onError(final Device device, final DeviceCallbackException error) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                onMainThreadError(device, error);
            }
        });
    }

    public abstract void onMainThreadSuccess(Device device, T result);

    public abstract void onMainThreadError(Device device, DeviceCallbackException error);
}
