package hk.advanpro.android.sdk.device.callback;
import hk.advanpro.android.core.utils.UIUtils;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.result.DeviceConnectResult;

/**
 * 用于设备连接后的主线程回调接口。
 * 可以直接在回调中更新UI
 * Created by jalin on 2017/7/10.
 */

public abstract class MainThreadDeviceConnectCallback<T extends DeviceConnectResult> implements DeviceConnectCallback<T> {

    public void onSucceed(final Device device, final T result) {
        UIUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                onMainThreadSucceed(device, result);
            }
        });
    }

    public void onError(final Device device, final DeviceCallbackException error) {
        UIUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                onMainThreadError(device, error);
            }
        });
    }

    public abstract void onMainThreadSucceed(Device device, T result);

    public abstract void onMainThreadError(Device device, DeviceCallbackException error);

}
