package hk.advanpro.android.sdk.device.ota;

import android.content.Context;

import java.io.File;

import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;

/**
 * Created by jalin on 2017/12/15.
 */

public interface DeviceFirmwareUpgrade {
    int STATUS_PREPARE_UPGRADE = 1;//准备升级
    int STATUS_INTO_BOOTLOADER = 2;//进入升级模式
    int STATUS_OUT_BOOTLOADER = 3;//退出升级模式，切换回应用模式
    int STATUS_DISCONNECTED = 4;//断开连接
    int STATUS_RECONNECTED = 5;//重连
    int STATUS_BLE_SCAN = 6;//设备搜索
    int STATUS_TRANSMIT_START = 7;//开始传输文件
    int STATUS_TRANSMIT_COMPLETE = 8;//传输文件完成

    void start(Context context, Device device, File file, DeviceFirmwareUpgradeCallback callback);

    /**
     * 准备升级
     */
    void prepareUpgrade();

    /**
     * 升级过程状态改变
     *
     * @param status status
     */
    void onStatusChange(int status);

    /**
     * 传输进度
     *
     * @param total    总数
     * @param progress 已完数
     */
    void onTransmitProgress(long total, long progress);

    /**
     * 传输完成
     */
    void onTransmitComplete();

    /**
     * 升级成功
     */
    void onUpgradeSuccess();

    /**
     * 升级出错
     *
     * @param error DeviceCallbackException 错误
     */
    void onUpgradeError(DeviceCallbackException error);

    /**
     * 是否传输成功
     *
     * @return true 传输成功
     */
    boolean isTransmitSuccess();

    /**
     * 是否正在传输
     *
     * @return true 正在传输
     */
    boolean isTransmiting();
}
