package hk.advanpro.android.sdk.device.ota.advanpro;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;

/**
 * 蓝牙服务
 */
public class BluetoothBLE {

    public static final String TAG = "BluetoothBLE";
    public static final int MESSAGE_GATT_CONNECTED = 1001;    // 设备已连接消息
    public static final int MESSAGE_GATT_DISCONNECTED = 1002; // 设备连接已断开
    public static final int MESSAGE_DATA_AVAILABLE = 1003;    // 收到设备数据
    public static final int MESSAGE_DEVICE_DISCOVER = 1004;   // 蓝牙设备发现
    public static final int MESSAGE_DATA_REQUEST = 1005;      // 蓝牙连接请求数据发送
    public static final int MESSAGE_RSSI = 1006;

    public static BluetoothBLE getInstance() {
        return Instance.ble;
    }

    private static class Instance {  // 唯一实例
        private static BluetoothBLE ble = new BluetoothBLE(AdvanproAndroidSDK.getAppContext());
    }

    /**
     * 初始化蓝牙实例
     */
    private BluetoothBLE(Context activity) {
        try {
            // For API level 18 and above, get a reference to BluetoothAdapter through BluetoothManager.
            if (mBluetoothManager == null) {
                mBluetoothManager = (BluetoothManager) activity.getSystemService(Context.BLUETOOTH_SERVICE);
                if (mBluetoothManager == null) {
                    ALog.e(TAG, "Unable to initialize BluetoothManager.");
                    return;
                }
            }

            mBluetoothAdapter = mBluetoothManager.getAdapter();
            if (mBluetoothAdapter == null) {
                ALog.e(TAG, "Unable to obtain a BluetoothAdapter.");
                return;
            }

        } catch (Exception Ex) {
            ALog.e(TAG, "打开蓝牙适配器异常.");
        }
    }


    /**
     * 连接蓝牙设备
     * @param context context
     * @param address mac address
     * @param handler handler
     * @return BLEConnect
     */
    public BLEConnect connect(Context context, String address, EventHandler handler) {
        if (mBluetoothAdapter == null || address == null) {
            ALog.e(TAG, "BluetoothAdapter not initialized or unspecified address.");
            return null;
        }

        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        if (device == null) {
            ALog.e(TAG, " BluetoothDevice not found.  Unable to connect.");
            return null;
        }

        return BLEConnect.createConnect(context, device, handler);
    }

    /**
     * 禁用蓝牙
     */
    public void disable() {
        mBluetoothAdapter.disable();
    }

    /**
     * 启用蓝牙
     */
    public void enable() {
        mBluetoothAdapter.enable();
    }

    /**
     * 蓝牙是否启用
     * @return boolean
     */
    public boolean isEnabled() {
        return mBluetoothAdapter.isEnabled();
    }


    private BluetoothManager mBluetoothManager;
    private BluetoothAdapter mBluetoothAdapter;
}
