package hk.advanpro.android.sdk.device.ota.advanpro;

import android.os.Looper;

public abstract class EventHandler {

    public EventHandler() {
        handler = new MyHandler(Looper.getMainLooper(), this);  // 主线程Handler
    }

    public EventHandler(Looper looper) {
        handler = new MyHandler(looper, this);
    }

    /**
     * 启用事件处理
     */
    public void open() {
        handler.setReference(this);
    }

    /**
     * 停止事件处理
     */
    public void close() {
        handler.setReference(null);
    }

    public android.os.Handler getHandler() { return handler; }

    public abstract void handleMessage(android.os.Message msg);  // 处理事件

    public boolean sendMessage(android.os.Message msg) {
        return handler.sendMessage(msg);
    }

    public boolean sendMessage(int what, Object obj) {
        android.os.Message msg = new android.os.Message();
        msg.what = what;
        msg.obj = obj;
        return handler.sendMessage(msg);
    }

    public boolean sendEmptyMessage(int what) {
        return handler.sendEmptyMessage(what);
    }

    public void removeMessages(int what) {
        handler.removeMessages(what);
    }

    public boolean sendMessageDelayed(android.os.Message msg, long delayMillis) {
        return handler.sendMessageDelayed(msg, delayMillis);
    }

    public boolean sendEmptyMessageDelayed(int what, long delayMillis) {
        return handler.sendEmptyMessageDelayed(what, delayMillis);
    }

    private static class MyHandler extends android.os.Handler {
        //WeakReference<EventHandler> reference;
        EventHandler reference;
        public MyHandler(Looper looper, EventHandler h) {
            //reference = new WeakReference<>(h);
            super(looper);
            this.reference = h;
        }

        public synchronized EventHandler getReference() {
            return reference;
        }

        public synchronized void setReference(EventHandler reference) {
            this.reference = reference;
        }

        @Override
        public void handleMessage(android.os.Message msg) {
            //EventHandler h = reference.get();
            EventHandler h = getReference();
            if (h != null) {
                h.handleMessage(msg);
            }
        }
    }

    private MyHandler handler;
}
