package hk.advanpro.android.sdk.device.ota.csr;

import android.bluetooth.BluetoothDevice;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.Looper;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;

import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.ble.BLEDevice;
import hk.advanpro.android.sdk.device.ble.BLEInsoleDevice;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.ota.BLEFirmwareUpgradeBase;
import hk.advanpro.android.sdk.device.params.ble.insole.BLEInsoleFirmwareUpgradeProtectCommandParams;
import hk.advanpro.android.sdk.device.result.ble.insole.BleInsoleFirmwareUpgradeProtectCommandResult;

/**
 * 鞋垫固件升级
 * Created by jalin on 2017/12/15.
 */

public class BLEInsoleFirmwareUpgrade extends BLEFirmwareUpgradeBase {

    public static final String TAG = "BLEInsoleFirmwareUpgrade";
    private static final int MAX_CS_KEY_NUMBER = 100;

    private String mBtAddress;
    private int mCrystalTrim = -1;
    private String mIdentityRoot;
    private String mEncryptionRoot;
    private int mCSVersion = 0;
    private String filePath;
    private Handler handler;
    private boolean refreshing;
    public boolean initResult;
    private int otaTimeCount;
    private boolean isComplete = false;
    private int tryCount = 0;//尝试

    private BluetoothDevice bluetoothDevice;

    private final static char[] hexArray = "0123456789ABCDEF".toCharArray();

    @Override
    public void prepareUpgrade() {
        filePath = getFile().getPath();
        bluetoothDevice = ((BLEDevice) getDevice()).getScanResult().getBluetoothDevice();

        if (getDevice().isApplicationMode()) {
            //发送状态改变事件，进入ota
            onStatusChange(STATUS_INTO_BOOTLOADER);
            //应用模式，先进行现场保护
            getDevice().command(BLEInsoleDevice.CMD_INSOLE_FIRMWARE_UPGRADE_PROTECT, new BLEInsoleFirmwareUpgradeProtectCommandParams(), 30, new DeviceCommandCallback<BleInsoleFirmwareUpgradeProtectCommandResult>() {
                @Override
                public void onSuccess(Device device, final BleInsoleFirmwareUpgradeProtectCommandResult result) {
                    //现场保护成功，断开连接
                    disconnect();
                }

                @Override
                public void onError(Device device, DeviceCallbackException error) {
                    onUpgradeError(error);
                }
            });
        } else {
            //直接断开连接
            disconnect();
        }
    }

    public void disconnect() {
        onStatusChange(STATUS_DISCONNECTED);
        //断开连接
        getDevice().disconnect(new DeviceDisconnectCallback() {
            @Override
            public void onSucceed(Device device) {
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    ALog.e(TAG, e);
                }
                //断开连接成功后，进入开始固件升级
                startUpgrade();
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                onUpgradeError(error);
            }
        });
    }

    public void startUpgrade() {
        this.mBtAddress = getAddress();
        this.handler = new Handler(Looper.getMainLooper());
        this.initResult = OtauManager.initialize(getContext(), bluetoothDevice);
        tryCount = 0;
        if (this.initResult) {
            OtauManager.getInstance(getAddress()).setIsMeshDevice(false);
            loadUserKeyConfiguration();
            startOta();
        }
    }

    /**
     * Handle start OTA button pressed event
     */
    private void startOta() {
        if (filePath == null || bluetoothDevice == null) {
            onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
            onOtaLog(getAddress() + ": Please choose updated image or set target!");
            return;
        }

        OtauManager.getInstance(getAddress()).connect(iOtauMessageListener);
        isComplete = false;
        this.handler.postDelayed(runnable, 1000);
    }


    /**
     * Set confirmation method and upgrade behaviour according by user keys configuration
     */
    private void loadUserKeyConfiguration() {
        if (Config.getConfirmationMethod() == Config.ConfirmationMethod2 || Config.getConfirmationMethod() == Config.ConfirmationMethod4) {
            OtauManager.getInstance(getAddress()).setHostValidation(true);
        } else {
            OtauManager.getInstance(getAddress()).setHostValidation(false);
        }
    }

    private void onOtaSuccess() {
        destroy();
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                isComplete = true;
                refreshing = false;
                onOtaLog(getAddress() + ": " + "Device has been updated successfully!");

                //调用完成传输
                onTransmitComplete();
            }
        }, 4000);
    }


    private void destroy() {
        OtauManager manager = OtauManager.getInstance(getAddress());
        if (manager != null) {
            manager.destroy();
        }
    }

    /**
     * Handle cancel OTA button pressed event
     */
    public void cancel() {
        if (initResult) {
            OtauManager manager = OtauManager.getInstance(getAddress());
            if (manager != null) {
                manager.cancelDownload();
            }
        }
    }

    Runnable runnable = new Runnable() {
        @Override
        public void run() {
            if (isComplete)
                return;
            otaTimeCount++;
            //15秒检查一次超时
            if (otaTimeCount >= 15) {
                //进入超时
                otaTimeCount = 0;
                onOtaLog(getAddress() + ": time out!");
                OtauManager.getInstance(getAddress()).disconnectAndClose();
                if (tryCount++ <= 3) {
                    onOtaLog(getAddress() + ": tryCount=" + tryCount);
                    startOta();
                } else {
                    isComplete = true;
                    ALog.e(TAG, "Upgrade timeout!");
                    onUpgradeError(new DeviceCallbackException(Device.ERROR_IS_TIME_OUT));
                }
            } else
                handler.postDelayed(runnable, 1000);
        }
    };


    /**
     * Set which application area will be updated in remote device
     */
    private void setApplicaitonId() {
        if (Config.getUpgradeBehaviour() == Config.UpgradeMethod1 || Config.getUpgradeBehaviour() == Config.UpgradeMethod4) {
            OtauManager.getInstance(getAddress()).setApplicationOnChip(1);
        }
    }

    /**
     * Start downloading image
     */
    private void startDownloading() {
        if (filePath == null) {
            onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
            onOtaLog(getAddress() + ": " + "mFilePath is null");
            return;
        }

        if (mBtAddress == null || mCrystalTrim == -1) {
            onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
            onOtaLog(getAddress() + ": " + "mBtAddress or mCrystalTrim is empty");
            return;
        }
        // csr added start
        // clearClientRequest GATT operation queue before downloading
        OtauManager.getInstance(getAddress()).clearClientRequest();
        // csr added end
        new MergeCsKeysToImage().execute(filePath);
    }


    private void onOtaError(final int code) {
        if (code == OtauManager.REQUEST_SET_TRANSFER_CONTROL_COMPLETE || code == OtauManager.ERROR_MESSAGE_ID_OTHERS) {
            isComplete = true;
            onUpgradeError(new DeviceCallbackException(Device.ERROR_UPGRADE_OTHER_ERROR));
        }
    }

    private void onOtaLog(final String log) {
        ALog.d(this.getClass().getName(), log);
    }

    /**
     * Set specific CS key item into array list which will be used to merge
     *
     * @param cskeyId int specific cskey id
     * @param data    byte[] the value of CS keys
     */
    private void setMergedCskeyValue(int cskeyId, byte[] data) {
        OtauManager manager = OtauManager.getInstance(getAddress());
        ArrayList<CskeyItem> cskeyList = manager.getCskeyList();
        if (cskeyList == null) return;

        try {
            for (CskeyItem entry : cskeyList) {
                if (entry.id == cskeyId) {
                    CskeyItem cskeyItem = new CskeyItem(entry.name, entry.id, entry.offset, entry.length);
                    System.arraycopy(data, 0, cskeyItem.value, 0, data.length);
                    onOtaLog(getAddress() + ": " + "setMergedCskeyValue: name = " + entry.name + ", value = " + bytesToHex(cskeyItem.value));
                    manager.addMergedCskeyItem(cskeyItem);
                    break;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
            onOtaLog(getAddress() + ": " + "Reading CS Block throw exception!");
        }
    }

    private String bytesToHex(byte[] bytes) {
        char[] hexChars = new char[bytes.length * 2];
        for (int j = 0; j < bytes.length; j++) {
            int v = bytes[j] & 0xFF;
            hexChars[j * 2] = hexArray[v >>> 4];
            hexChars[j * 2 + 1] = hexArray[v & 0x0F];
        }
        return new String(hexChars);
    }


    /**
     * It performs background operations and publish results on the UI thread without having to manipulate threads and/or handlers.
     * Merging CS keys will take up UI thread's time slot and memory, so move to background thread.
     */
    private class MergeCsKeysToImage extends AsyncTask<String, Integer, Boolean> {

        @Override
        protected Boolean doInBackground(String... filePath) {
            File imgFile = new File(filePath[0]);
            int size = (int) imgFile.length();
            final byte[] data = new byte[size];
            boolean rtn;
            try {
                BufferedInputStream buf = new BufferedInputStream(new FileInputStream(imgFile));
                buf.read(data, 0, data.length);
                rtn = OtauManager.getInstance(getAddress()).setImageData(data, mBtAddress, mCrystalTrim, mIdentityRoot, mEncryptionRoot);
                buf.close();
            } catch (Exception e) {
                e.printStackTrace();
                rtn = false;
            }
            return rtn;
        }

        @Override
        protected void onPostExecute(Boolean result) {
            if (result) {
                OtauManager.getInstance(getAddress()).sendNextImageChunk();
            } else {
                onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
                onOtaLog(getAddress() + ": " + "Merging image CS Key exception!");
            }
        }
    }

    /**
     * Loading cs keys xml file into memory, it run on background thread
     */
    private class LoadCskeysFromXmlFileTask extends AsyncTask<Integer, Integer, Boolean> {
        @Override
        protected Boolean doInBackground(Integer... buildId) {
            onOtaLog(getAddress() + ": " + "Cskey build id is:" + buildId[0]);
            if (buildId[0] <= 0 || buildId[0] >= MAX_CS_KEY_NUMBER) {
                onOtaLog(getAddress() + ": " + "Cskey build id is not right," + buildId[0]);
                return false;
            }
            onOtaLog(getAddress() + ": " + "LoadCskeysFromXmlFileTask setCskeyList");
            OtauManager.getInstance(getAddress()).setCskeyList(Config.loadCskeyList());
            return true;
        }

        @Override
        protected void onPostExecute(Boolean result) {
            if (result) {
                OtauManager.getInstance(getAddress()).setNextReadCsBlockState();
                OtauManager.getInstance(getAddress()).readNextCsBlock();
            } else {
                onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
                onOtaLog(getAddress() + ": " + "Read CSKey error!");
            }
        }
    }


    IOtauMessageListener iOtauMessageListener = new IOtauMessageListener() {

        @Override
        public void onBtAddressUpdate(String btAddress) {
            mBtAddress = btAddress;
            onOtaLog(getAddress() + ": " + "Read Bluetooth address succeed");
        }

        @Override
        public void onCrystalTrimUpdate(int value) {
            mCrystalTrim = value;
            onOtaLog(getAddress() + ": " + "Read crystal trim value succeed");
        }

        @Override
        public void onIdentityRootUpdate(String value) {
            mIdentityRoot = value;
            onOtaLog(getAddress() + ": " + "[Identity Root]:" + value);
        }

        @Override
        public void onEncryptionRootUpdate(String value) {
            mEncryptionRoot = value;
            onOtaLog(getAddress() + ": " + "[Encryption Root]:" + value);
        }

        @Override
        public void onBootloaderVersionUpdate(String version) {
            onOtaLog(getAddress() + ": " + "onBootloaderVersionUpdate");
        }

        @Override
        public void onCsrOtaVersionUpdate(String version) {
            onOtaLog(getAddress() + ": " + "Read CSR software version succeed");
            try {
                int ver = Integer.parseInt(version);
                if (ver > 4) {
                    handler.post(new Runnable() {
                        @Override
                        public void run() {
                            OtauManager.getInstance(getAddress()).readNextCsBlock();
                        }
                    });
                }
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onSoftwareVersionUpdate(String version) {
            onOtaLog(getAddress() + ": " + "OTAU Software Version:" + version);
        }

        @Override
        public void onApplicationVersionUpdate(String version) {
            onOtaLog(getAddress() + ": " + "Read CSR software version succeed");
        }

        @Override
        public void onOtaMessageUpdate(int errorMessageId) {
            String message;
            switch (errorMessageId) {
                case OtauManager.ERROR_MESSAGE_ID_READCSKEY_BTADDRESS:
                    message = "Read Bluetooth address error!";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_READCSKEY_XLST:
                    message = "Read cyrstal trim value error!";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_READCSKEY_IDROOT:
                    message = "Read identity root error!";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_READCSKEY_ENCROOT:
                    message = "Read encryption root error!";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_CHALLENGE_RESPONSE:
                    message = "Read challenge response failed, please check the configuration";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_ENABLE_OTA:
                    message = "Set OTAU mode error";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_WRITE_TO_TARGET:
                    message = "Remote target rejected the package sent";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_BL_TRANSFERCONTROL_NOTIFICATION:
                    message = "Request boot loader transfer control notification failed!";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_NULL_CHARACTERISTIC:
                case OtauManager.ERROR_MESSAGE_ID_NULL_SERVICE:
                    message = "Access remote GATT service or characteristic error.";
                    break;
                case OtauManager.ERROR_MESSAGE_ID_SET_APPLICATION_ID:
                    message = "The target might not have to set application position.";
                    break;
                case OtauManager.ERROR_MESSAGE_GATT_CLIENT_DESTROYED:
                    message = "the BluetoothGatt instance is null";
                    break;
                default:
                    message = "request failed. requestId: " + errorMessageId;
                    break;
            }
            onOtaError(errorMessageId);
            onOtaLog(getAddress() + ": " + message);
        }

        @Override
        public void onConnectionStateUpdate(int state) {
            if (refreshing) {
                return;
            }
            mCSVersion = 0;
            switch (state) {
                case OtauClient.MESSAGE_CONNECTED:
                    onStatusChange(STATUS_RECONNECTED);
                    onOtaLog(getAddress() + ": " + "Gatt server connected and service discovery is completed.");
                    break;
                case OtauClient.MESSAGE_DISCONNECTED:
                    onStatusChange(STATUS_DISCONNECTED);
                    onOtaLog(getAddress() + ": " + "gatt disconnected");
                    break;
                case OtauClient.MESSAGE_UNKNOWN:
                    onOtaLog(getAddress() + ": " + "Remote device does not support OTAU update.");
                    break;
                case OtauClient.MESSAGE_INTERRUPTED:
                    onOtaLog(getAddress() + ": " + "InterruptedException!!!");
                    break;
            }

        }

        @Override
        public void onOtaProgressUpdate(int progress, int size, int sentNumber) {
            double percent = ((double) progress) / 100;
            onTransmitProgress(size, (long) (size * percent));

            otaTimeCount = 0;
        }

        @Override
        public void onOtaStateUpdate(State.OtaState state) {
            switch (state) {
                case STATE_OTA_INIT_READ_BT_ADDRESS:
                    onOtaLog(getAddress() + ": " + "Read Bluetooth address...");
                    break;
                case STATE_OTA_INIT_READ_XTAL_TRIM:
                    onOtaLog(getAddress() + ": " + "Read crystal trim value...");
                    break;
                case STATE_OTA_SET_MODE:
                    onOtaLog(getAddress() + ": " + "Enter OTAU boot loader mode");
                    break;
                case STATE_OTA_INIT_READ_IDENTITY_ROOT:
                    onOtaLog(getAddress() + ": " + "Read identity root...");
                    break;
                case STATE_OTA_INIT_READ_ENCRYPTION_ROOT:
                    onOtaLog(getAddress() + ": " + "Read encryption root...");
                    // this is for V4 bootloader
                    onOtaLog(getAddress() + ": " + "OTAU is enabled");
                    break;
                case STATE_OTA_SET_CURRENT_APP:
                    onOtaLog(getAddress() + ": " + "Set onchip application success, get ready to download image");
                    break;
                case STATE_OTA_SET_TRANSFER_CTRL:
                    onOtaLog(getAddress() + ": " + "Downloading...");
                    startDownloading();
                    break;
                case STATE_OTA_SET_TRANSFER_COMPLETE:
                    //刷新服务
                    onOtaLog(getAddress() + ": " + "Download completed");
                    OtauManager.getInstance(getAddress()).disconnect();
                    refreshing = true;
                    OtauManager.getInstance(getAddress()).refresh();
                    onOtaLog(getAddress() + ": " + "Completing update...");
                    onOtaSuccess();
                    break;
                case STATE_OTA_PAUSE_DATA_TRANSFER:
                    onOtaLog(getAddress() + ": " + "Download Paused!");
                    break;
                case STATE_OTA_ABORT_DATA_TRANSFER:
                    onOtaLog(getAddress() + ": " + "Download cancelled!");
                    onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
                    break;
                case STATE_OTA_INIT_READ_CHALLENGE:
                    onOtaLog(getAddress() + ": " + "Read challenge key");
                    break;
                case STATE_OTA_WRITE_RESPONSE:
                    onOtaLog(getAddress() + ": " + "Respond to the challenge");
                    break;
                case STATE_OTA_READ_CS_BLOCK:
                    onOtaLog(getAddress() + ": " + "Read CS Block data...");
                    break;
                case STATE_OTA_REFRESH_ATTRIBUTES:
                    break;
                case STATE_OTA_RECONNECT_GATT:
                    onOtaLog(getAddress() + ": " + "Waiting in few seconds to refresh the version and cached attributes");
                    break;
            }
        }

        @Override
        public void onOtaTransferControlStatus(short state) {
            switch (state) {
                case OtauManager.SET_TRANSFER_CONTROL_INIT:
                    onOtaLog(getAddress() + ": " + "SET_TRANSFER_CONTROL_INIT");
                    OtauManager otauManager = OtauManager.getInstance(getAddress());
                    if (otauManager.getBootloaderSoftwareVersionSupported()) {
                        otauManager.readDataTransferCharacteristic();
                    }
                    break;
                case OtauManager.SET_TRANSFER_CONTROL_READY:
                    // handle V6 onwards
                    onOtaLog(getAddress() + ": " + "SET_TRANSFER_CONTROL_READY");
                    otauManager = OtauManager.getInstance(getAddress());
                    if (otauManager.getBootloaderSoftwareVersionSupported()) {
                        onOtaLog(getAddress() + ": " + "registerBootloaderTransferControlNotifiocation");
                        otauManager.registerBootloaderTransferControlNotifiocation();
                        onOtaLog(getAddress() + ": " + "readBootloadVersion");
                        otauManager.readBootloadVersion();
                        onOtaLog(getAddress() + ": " + "readBootloaderSoftwareVersion");
                        otauManager.readBootloaderSoftwareVersion();

                        // This process is from V5 onwards
                        if (otauManager.getReadCskeyFromBootloader()) {
                            onOtaLog(getAddress() + ": " + "readCSKeysFromBootloaderService REQUEST_READ_CSKEYS_BT_FROM_BL");
                            otauManager.readCSKeysFromBootloaderService(OtauManager.REQUEST_READ_CSKEYS_BT_FROM_BL, OtauManager.USER_KEY_BT_ADDRESS);
                            otauManager.readDataTransferCharacteristic();
                            onOtaLog(getAddress() + ": " + "readCSKeysFromBootloaderService REQUEST_READ_CSKEYS_XLST_FROM_BL");
                            otauManager.readCSKeysFromBootloaderService(OtauManager.REQUEST_READ_CSKEYS_XLST_FROM_BL, OtauManager.USER_KEY_XTAL_TRIM);
                            otauManager.readDataTransferCharacteristic();
                            onOtaLog(getAddress() + ": " + "readCSKeysFromBootloaderService REQUEST_READ_CSKEYS_IDROOT_FROM_BL");
                            otauManager.readCSKeysFromBootloaderService(OtauManager.REQUEST_READ_CSKEYS_IDROOT_FROM_BL, OtauManager.USER_KEY_IDENTITY_ROOT);
                            otauManager.readDataTransferCharacteristic();
                            onOtaLog(getAddress() + ": " + "readCSKeysFromBootloaderService REQUEST_READ_CSKEYS_ENCROOT_FROM_BL");
                            otauManager.readCSKeysFromBootloaderService(OtauManager.REQUEST_READ_CSKEYS_ENCROOT_FROM_BL, OtauManager.USER_KEY_ENCRYPTION_ROOT);
                            otauManager.readDataTransferCharacteristic();
                        }

                        onOtauStarted();
                    }
                    break;
                case OtauManager.REQUEST_SET_TRANSFER_CONTROL_INPROGRESS:
                    onOtaLog(getAddress() + ": " + "Bootloader transfer control status is in progress");
                    break;
                case OtauManager.SET_TRANSFER_PAUSED:
                    onOtaLog(getAddress() + ": " + "Bootloader transfer control status is paused");
                    break;
                case OtauManager.SET_TRANSFER_COMPLETED:
                    onOtaLog(getAddress() + ": " + "Bootloader transfer control status is completed");
                    break;
                case OtauManager.SET_TRANSFER_FAILED:
                    onOtaLog(getAddress() + ": " + "Bootloader transfer control status is failed");
                    break;
                case OtauManager.SET_TRANSFER_ABORT:
                    onOtaLog(getAddress() + ": " + "Bootloader transfer control status is aborted");
                    break;
                default:
                    onOtaLog(getAddress() + ": " + "Bootloader transfer control status is unknown");
                    break;
            }
        }

        @Override
        public void onCsBlockData(byte[] data) {
            if (data == null || data.length == 0 || data.length == 1) {
                onOtaLog(getAddress() + ": " + "data =" + bytesToHex(data));
                return;
            }

            State.ReadCsBlockState state = State.getReadCsBlockState(getAddress());
            if (state == null || (state == State.ReadCsBlockState.READ_CS_BLOCK_BUILD_ID && mCSVersion < (data[1] << 8 | data[0]) && mCSVersion != 0)) {
                onOtaLog(getAddress() + ": " + "state = " + state + ", mCSVersion = " + mCSVersion);
                return;
            }

            switch (state) {
                case READ_CS_BLOCK_BUILD_ID:
                    mCSVersion = data[1] << 8 | data[0];
                    onOtaLog(getAddress() + ": " + "READ_CS_BLOCK_BUILD_ID: data = " + bytesToHex(data));
                    new LoadCskeysFromXmlFileTask().execute(mCSVersion);
                    return;
                case READ_CS_BLOCK_BT_ADDRESS:
                    String address = "";
                    for (int i = data.length - 1; i >= 0; i--) {
                        address += String.format("%02X:", data[i]);
                    }
                    mBtAddress = address.substring(0, address.length() - 1);
                    onOtaLog(getAddress() + ": " + "READ_CS_BLOCK_BT_ADDRESS: data = " + bytesToHex(data));
                    setMergedCskeyValue(State.ReadCsBlockState.READ_CS_BLOCK_BT_ADDRESS.valueOf(), data);
                    break;
                case READ_CS_BLOCK_XLST:
                    mCrystalTrim = data[1] << 8 | data[0];
                    onOtaLog(getAddress() + ": " + "READ_CS_BLOCK_XLST: data = " + bytesToHex(data));
                    setMergedCskeyValue(State.ReadCsBlockState.READ_CS_BLOCK_XLST.valueOf(), data);
                    break;
                case READ_CS_BLOCK_IDENTITY_ROOT:
                    String idRoot = "";
                    for (byte bb : data) {
                        idRoot += String.format("%02x", bb);
                    }
                    mIdentityRoot = idRoot;
                    onOtaLog(getAddress() + ": " + "READ_CS_BLOCK_IDENTITY_ROOT: data = " + bytesToHex(data));
                    setMergedCskeyValue(State.ReadCsBlockState.READ_CS_BLOCK_IDENTITY_ROOT.valueOf(), data);
                    break;
                case READ_CS_BLOCK_ENCRYPTION_ROOT:
                    String encRoot = "";
                    for (byte bb : data) {
                        encRoot += String.format("%02x", bb);
                    }
                    mEncryptionRoot = encRoot;
                    onOtaLog(getAddress() + ": " + "READ_CS_BLOCK_ENCRYPTION_ROOT: data = " + bytesToHex(data));
                    setMergedCskeyValue(State.ReadCsBlockState.READ_CS_BLOCK_ENCRYPTION_ROOT.valueOf(), data);
                    break;
                default:
                    onOtaError(OtauManager.ERROR_MESSAGE_ID_OTHERS);
                    onOtaLog(getAddress() + ": " + "invalid_csblock_id");
                    break;
            }

            handler.post(new Runnable() {
                @Override
                public void run() {
                    OtauManager.getInstance(getAddress()).setNextReadCsBlockState();
                    OtauManager.getInstance(getAddress()).readNextCsBlock();
                }
            });
        }

        @Override
        public void onOtauEnabled(boolean enabled) {
            OtauManager.getInstance(getAddress()).enableOTAMode();
            onOtaLog(getAddress() + ": " + "OTAU is enabled");
        }

        @Override
        public void onOtauStarted() {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    setApplicaitonId();
                    OtauManager.getInstance(getAddress()).setTransferControlInProgress();
                }
            });
        }

        @Override
        public void onOtherLog(String log) {
            onOtaLog(getAddress() + ": " + log);
        }
    };
}
