package hk.advanpro.android.sdk.device.protocol;


import android.util.Log;

import org.apache.commons.lang3.ArrayUtils;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Calendar;

import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleClearAllCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleClearDataCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectDataEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectStartOrStopCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleCollectStatusCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleDisableAccCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleHistoryGaitCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleHistoryStepCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRawGaitCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleReadIdCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealAccEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealGaitEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleRealStepEventResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleStepSectionsCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleStopSendCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleSyncTimeCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BLEInsoleWriteIdCommandResult;
import hk.advanpro.android.sdk.device.result.ble.insole.BleInsoleFirmwareUpgradeProtectCommandResult;

/**
 * Created by yejianfei on 2017/6/8.
 */

public class BLEInsoleProtocol {

    public static byte[] buildHeartbeatCommand() {
        Calendar calendar = Calendar.getInstance();

        return new byte[]{
                (byte) 0xbb,
                (byte) 0xbb,
                (byte) (calendar.get(Calendar.YEAR) - 2000),
                (byte) (calendar.get(Calendar.MONTH) + 1),
                (byte) calendar.get(Calendar.DATE),
                (byte) calendar.get(Calendar.HOUR_OF_DAY),
                (byte) calendar.get(Calendar.MINUTE),
                (byte) calendar.get(Calendar.SECOND)
        };
    }

    public static byte[] buildSyncTimeCommand(int year, int month, int day, int hour, int minute, int second) {
        return new byte[]{
                (byte) 0xaa,
                (byte) 0xbb,
                (byte) (year > 2000 ? year - 2000 : year),
                (byte) month,
                (byte) day,
                (byte) hour,
                (byte) minute,
                (byte) second
        };
    }

    public static BLEInsoleSyncTimeCommandResult buildSyncTimeResult(byte[] data) {
        BLEInsoleSyncTimeCommandResult result = new BLEInsoleSyncTimeCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        result.setLength(buffer.get(2));
        buffer.position(3);

        while (buffer.remaining() > 3) {
            BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog log =
                    new BLEInsoleSyncTimeCommandResult.BLEInsoleStorageLog();

            log.setYear((short) ((short) buffer.get() + 2000));
            log.setMonth((short) buffer.get());
            log.setDay((short) buffer.get());

            if (log.getYear() > 0)//过滤无效的日期
                result.getLogs().add(log);
        }

        return result;
    }

    public static byte[] buildWriteIdCommand(long id) {
        ByteBuffer buffer = ByteBuffer.allocate(10).order(ByteOrder.LITTLE_ENDIAN);
        buffer.put((byte) 0xbb);
        buffer.put((byte) 0xaa);

        buffer.putLong(id);

        return buffer.array();
    }

    public static BLEInsoleWriteIdCommandResult buildWriteIdResult(byte[] data) {
        BLEInsoleWriteIdCommandResult result = new BLEInsoleWriteIdCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        result.setSuccess(buffer.get(3) == (byte) 0x0);

        return result;
    }

    public static byte[] buildReadIdCommand() {
        return new byte[]{
                (byte) 0xbb,
                (byte) 0xaa
        };
    }

    public static BLEInsoleReadIdCommandResult buildReadIdResult(byte[] data) {
        BLEInsoleReadIdCommandResult result = new BLEInsoleReadIdCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        result.setId(buffer.getLong(2));
        return result;
    }

    public static byte[] buildStopSendCommand(int year, int month, int day, int last) {
        return new byte[]{
                (byte) 0xaa,
                (byte) 0xcc,
                (byte) (year > 2000 ? year - 2000 : year),
                (byte) month,
                (byte) day,
                (byte) last
        };
    }

    public static BLEInsoleStopSendCommandResult buildStopSendResult(byte[] data) {
        BLEInsoleStopSendCommandResult result = new BLEInsoleStopSendCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        result.setYear((short) (buffer.get(2) + 2000));
        result.setMonth(buffer.get(3));
        result.setDay(buffer.get(4));
        result.setLast(buffer.get(5));

        return result;
    }

    public static byte[] buildClearDataCommand(int year, int month, int day) {
        return new byte[]{
                (byte) 0xaa,
                (byte) 0xdd,
                (byte) (year > 2000 ? year - 2000 : year),
                (byte) month,
                (byte) day
        };
    }

    public static BLEInsoleClearDataCommandResult buildClearDataResult(byte[] data) {
        BLEInsoleClearDataCommandResult result = new BLEInsoleClearDataCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        result.setSuccess(buffer.get(2) == 0x0);
        result.setCode(buffer.get(3));

        return result;
    }

    public static byte[] buildClearAllCommand() {
        return new byte[]{
                (byte) 0xaa,
                (byte) 0xff,
                (byte) 0xff,
                (byte) 0xff,
                (byte) 0xff
        };
    }

    public static BLEInsoleClearAllCommandResult buildClearAllResult(byte[] data) {
        BLEInsoleClearAllCommandResult result = new BLEInsoleClearAllCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        result.setSuccess(buffer.get(2) == 0x01);
        result.setCode(buffer.get(3));

        return result;
    }

    public static byte[] buildEnableAccCommand(int time) {
        ByteBuffer buffer = ByteBuffer.allocate(4).order(ByteOrder.LITTLE_ENDIAN);
        buffer.put((byte) 0xaa);
        buffer.put((byte) 0xfd);
        buffer.putShort((short) time);

        return buffer.array();
    }

    public static byte[] buildDisableAccCommand() {
        return new byte[]{
                (byte) 0xaa,
                (byte) 0xed
        };
    }

    public static byte[] buildCollectStartCommand(int rate) {
        return new byte[]{
                (byte) 0xcc,
                (byte) 0x01,
                (byte) rate
        };
    }


    public static byte[] buildCollectStopCommand() {
        return new byte[]{
                (byte) 0xcc,
                (byte) 0x00,
                (byte) 0x00
        };
    }

    public static byte[] buildCollectStatusCommand() {
        return new byte[]{
                (byte) 0xcc,
                (byte) 0xcc
        };
    }

    public static BLEInsoleDisableAccCommandResult buildDisableAccResult(byte[] data) {
        BLEInsoleDisableAccCommandResult result = new BLEInsoleDisableAccCommandResult();

        return result;
    }

    public static BLEInsoleRealGaitEventResult buildGaitEventResult(byte[] data) {
        BLEInsoleRealGaitEventResult result = new BLEInsoleRealGaitEventResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        byte sensors = buffer.get(2);
        result.setTouchA(((sensors >> 7) & 0x1) == 1);
        result.setTouchB(((sensors >> 6) & 0x1) == 1);
        result.setTouchC(((sensors >> 5) & 0x1) == 1);
        result.setTouchD(((sensors >> 4) & 0x1) == 1);
        result.setImpact((short) (sensors & 0x07));
        result.setForefoot(buffer.getShort(3));
        result.setSole(buffer.getShort(5));
        result.setHeel(buffer.getShort(7));
        result.setVarus(buffer.getShort(9));
        result.setEctropion(buffer.getShort(11));
        result.setYear((short) (buffer.get(13) + 2000));
        result.setMonth(buffer.get(14));
        result.setDay(buffer.get(15));
        ALog.d("BLEInsoleDevice", String.format("实时步态  [前掌：%d ,全掌：%d,后跟：%d,内翻：%d,外翻：%d,步态:%d%d%d%d,冲击力等级：%d]",
                result.getForefoot(), result.getSole(), result.getHeel(), result.getVarus(),
                result.getEctropion(), result.getTouchA() ? 1 : 0, result.getTouchB() ? 1 : 0, result.getTouchC() ? 1 : 0, result.getTouchD() ? 1 : 0,
                result.getImpact()
        ));
        return result;
    }

    public static BLEInsoleCollectDataEventResult buildCollectEventResult(byte[] data) {
        BLEInsoleCollectDataEventResult result = new BLEInsoleCollectDataEventResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        for (int i = 0; i < 4 && buffer.remaining() >= 5; i++) {
            //步态占1个byte
            int gait = buffer.get() & 0xF;

            //三轴占4个byte
            int acc = buffer.getInt();
            double x = (acc & 0B1000000000) >> 9 == 1 ? acc | 0xFFFFFC00 : (acc & 0B111111111);
            acc = acc >> 11;
            double y = (acc & 0B1000000000) >> 9 == 1 ? acc | 0xFFFFFC00 : (acc & 0B111111111);
            acc = acc >> 11;
            double z = (acc & 0B1000000000) >> 9 == 1 ? acc | 0xFFFFFC00 : (acc & 0B111111111);

            BLEInsoleCollectDataEventResult.CollectData data1 = new BLEInsoleCollectDataEventResult.CollectData();
            data1.setGait(gait);
            data1.setAccX(x * 31.25);
            data1.setAccY(y * 31.25);
            data1.setAccZ(z * 31.25);
            result.getDatas().add(data1);
        }
        return result;
    }

    public static BLEInsoleRealStepEventResult buildStepEventResult(byte[] data) {
        BLEInsoleRealStepEventResult result = new BLEInsoleRealStepEventResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        byte[] bytes = new byte[]{0x0, 0x00, 0x00, 0x00};

        result.setStatus((short) buffer.get(1));
        result.setYear((short) (buffer.get(2) + 2000));
        result.setMonth((short) buffer.get(3));
        result.setDay((short) buffer.get(4));

        result.setGait((data[5] >> 4) & 0B1111);
        result.setIsRun((data[5] >> 3) & 0B1);
        result.setImpact(data[5] & 0B111);
        result.setFootType(data[6] & 0B11);
        result.setTouchdown((data[6] >> 2) & 0B11);
        result.setStepDur(((data[6] >> 4) & 0B1111) + ((data[7] << 4) & 0B111111111111));

        buffer.position(8);
        buffer.get(bytes, 0, 3);
        result.setWalkStep(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(11);
        buffer.get(bytes, 0, 3);
        result.setWalkDuration(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(14);
        buffer.get(bytes, 0, 3);
        result.setRunStep(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(17);
        buffer.get(bytes, 0, 3);
        result.setRunDuration(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());
        ALog.d("BLEInsoleDevice", String.format("实时步数  [走步数：%d ,跑步数：%d,走步时长：%d,跑步时长：%d,\n" +
                        "走跑状态：%d,步态：%d,着地方式:%d,足内旋：%d,冲击力等级：%d,时长：%d]",
                result.getWalkStep(), result.getRunStep(), result.getWalkDuration(), result.getRunDuration(),
                result.getIsRun(), result.getGait(), result.getTouchdown(), result.getFootType(), result.getImpact(), result.getStepDur()
        ));
        return result;
    }

    public static BLEInsoleRealAccEventResult buildACCEventResult(byte[] data) {
        BLEInsoleRealAccEventResult result = new BLEInsoleRealAccEventResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        buffer.position(2);

        result.setAx(buffer.getShort());
        result.setAy(buffer.getShort());
        result.setAz(buffer.getShort());

        result.setBx(buffer.getShort());
        result.setBy(buffer.getShort());
        result.setBz(buffer.getShort());

        result.setCx(buffer.getShort());
        result.setCy(buffer.getShort());
        result.setCz(buffer.getShort());


        return result;
    }

    public static byte[] buildHistoryStepCommand(int year, int month, int day) {
        ByteBuffer buffer = ByteBuffer.allocate(5);
        buffer.put((byte) 0x06);
        buffer.put((byte) 0x02);
        buffer.put((byte) (year > 2000 ? year - 2000 : year));
        buffer.put((byte) month);
        buffer.put((byte) day);

        return buffer.array();
    }

    public static BLEInsoleHistoryStepCommandResult buildHistoryStepResult(byte[] data) {
        BLEInsoleHistoryStepCommandResult result = new BLEInsoleHistoryStepCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        byte[] bytes = new byte[]{0x00, 0x00, 0x00, 0x00};

        result.setMonth((short) buffer.get(2));
        result.setDay((short) buffer.get(3));

        buffer.position(4);
        buffer.get(bytes, 0, 3);
        result.setWalkStep(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(7);
        buffer.get(bytes, 0, 3);
        result.setWalkDuration(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(10);
        buffer.get(bytes, 0, 3);
        result.setRunStep(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(13);
        buffer.get(bytes, 0, 3);
        result.setRunDuration(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        return result;
    }

    public static byte[] buildHistoryGaitCommand(int year, int month, int day) {
        ByteBuffer buffer = ByteBuffer.allocate(5);
        buffer.put((byte) 0x06);
        buffer.put((byte) 0x01);
        buffer.put((byte) (year > 2000 ? year - 2000 : year));
        buffer.put((byte) month);
        buffer.put((byte) day);

        return buffer.array();
    }

    public static BLEInsoleHistoryGaitCommandResult buildHisoryGaitResult(byte[] data) {
        BLEInsoleHistoryGaitCommandResult result = new BLEInsoleHistoryGaitCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);

        result.setMonth((short) buffer.get(2));
        result.setDay((short) buffer.get(3));
        result.setForefoot(buffer.getShort(4));
        result.setSole(buffer.getShort(6));
        result.setHeel(buffer.getShort(8));
        result.setVarus(buffer.getShort(10));
        result.setEctropion(buffer.getShort(12));

        return result;

    }

    public static byte[] buildRawGaitCommand(int year, int month, int day, short begin, short size) {

        ByteBuffer buffer = ByteBuffer.allocate(7);
        buffer.put((byte) 0x06);
        buffer.put((byte) 0x07);
        buffer.put((byte) (year > 2000 ? year - 2000 : year));
        buffer.put((byte) month);
        buffer.put((byte) day);
        buffer.put((byte) begin);
        buffer.put((byte) size);

        return buffer.array();
    }

    public static BLEInsoleRawGaitCommandResult buildRawGaitResult(byte[] data) {
        BLEInsoleRawGaitCommandResult result = new BLEInsoleRawGaitCommandResult();

        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        byte[] bytes = new byte[]{0x0, 0x00, 0x00, 0x00};

        result.setMonth((short) buffer.get(2));
        result.setDay((short) buffer.get(3));
        buffer.position(4);
        buffer.get(bytes, 0, 3);
        result.setLength(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(7).limit(data.length - 3);
        while (buffer.remaining() > 3) {

            short index = (short) buffer.get();
            int size = buffer.getShort();
            if (size <= 0)
                continue;
            byte[] info = new byte[size * 2];
            if (info.length > buffer.remaining()) {
                Log.e(BLEInsoleProtocol.class.getName(), String.format("index out of bounds ,array size %d , index %d", buffer.remaining(), info.length));
                continue;
            }
            buffer.get(info);

            for (int i = 0; i < info.length; i++) {
                BLEInsoleRawGaitCommandResult.BLEInsoleRawGaitSection section
                        = new BLEInsoleRawGaitCommandResult.BLEInsoleRawGaitSection();

                byte gait = info[i];
                i++;
                byte offset = info[i];

                section.setSection(index);
                section.setGait((short) ((gait >> 4) & 0B1111));
                section.setType((short) ((gait >> 3) & 0B1));
                section.setImpact((short) (gait & 0B111));
                section.setTouchdown((short) ((offset >> 2) & 0B11));
                section.setFootType((short) (offset & 0B11));
                section.setMinute((short) ((offset >> 4) & 0B1111));

                result.getSections().add(section);

            }

        }

        return result;
    }

    public static byte[] buildSectionStepCommand(int year, int month, int day, short begin, short size) {

        ByteBuffer buffer = ByteBuffer.allocate(7);
        buffer.put((byte) 0x06);
        buffer.put((byte) 0x04);
        buffer.put((byte) (year > 2000 ? year - 2000 : year));
        buffer.put((byte) month);
        buffer.put((byte) day);
        buffer.put((byte) begin);
        buffer.put((byte) size);

        return buffer.array();
    }

    public static BLEInsoleStepSectionsCommandResult buidSectionStepResult(byte[] data) {
        BLEInsoleStepSectionsCommandResult result = new BLEInsoleStepSectionsCommandResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        byte[] bytes = new byte[]{0x0, 0x00, 0x00, 0x00};

        result.setMonth((short) buffer.get(2));
        result.setDay((short) buffer.get(3));
        buffer.position(4);
        buffer.get(bytes, 0, 3);
        result.setLength(ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN).getInt());

        buffer.position(7).limit(data.length - 3);

        while (buffer.remaining() >= 5) {
            short s = buffer.get();
            short w = buffer.getShort();
            short r = buffer.getShort();
            //过滤分段小于等于0的分段号
            if (s <= 0) {
                continue;
            }
            BLEInsoleStepSectionsCommandResult.BLEInsoleStepSection section
                    = new BLEInsoleStepSectionsCommandResult.BLEInsoleStepSection();
            section.setSection(s);
            section.setWalkStep(w);
            section.setRunStep(r);

            result.getSections().add(section);
        }

        return result;
    }


    public static byte[] buildStitaProtectCommand() {
        ByteBuffer buffer = ByteBuffer.allocate(5).order(ByteOrder.LITTLE_ENDIAN);
        buffer.put((byte) 0xAA);
        buffer.put((byte) 0xEE);
        buffer.put((byte) 0xFF);
        buffer.put((byte) 0xFF);
        buffer.put((byte) 0xFF);
        return buffer.array();
    }

    public static BleInsoleFirmwareUpgradeProtectCommandResult buildStitaProtectResult(byte[] data) {
        BleInsoleFirmwareUpgradeProtectCommandResult result = new BleInsoleFirmwareUpgradeProtectCommandResult((short) (data[3] & 0xFF));
        return result;
    }

    public static BLEInsoleCollectStatusCommandResult buildCollectStatusResult(byte[] data) {
        return new BLEInsoleCollectStatusCommandResult(data[1], data[2]);
    }

    public static BLEInsoleCollectStartOrStopCommandResult buildCollectStartOrStopResult(byte[] data) {
        return new BLEInsoleCollectStartOrStopCommandResult(data[1] == 0x00, data[1]);
    }
}
