package hk.advanpro.android.sdk.device.protocol;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import hk.advanpro.android.sdk.device.result.ble.BLEEnableWritePermissionCommandResult;
import hk.advanpro.android.sdk.device.result.ble.BLESendPermissionCodeCommandResult;
import hk.advanpro.android.sdk.device.result.ble.BLEWriteDeviceCodeCommandResult;
import hk.advanpro.android.sdk.device.result.ble.BLEWriteDeviceIdCommandResult;

/**
 * Created by yejianfei on 2017/6/23.
 */

public class BLEProtocol {

    public static byte[] buildEnableWritePermissionCommand() {
        return new  byte[]{(byte) 0xff, (byte) 0x01, (byte) 0x01};
    }

    public static BLEEnableWritePermissionCommandResult buildEnableWritePermissioneResult(byte[] data) {
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        BLEEnableWritePermissionCommandResult result = new BLEEnableWritePermissionCommandResult();

        result.setTable((short) ((buffer.get(3) & 0xff) | (buffer.get(4) << 8)));
        result.setRow((short) (buffer.get(5) & 0xff));
        result.setA((short) (buffer.get(6) & 0xff));
        result.setB((short) (buffer.get(7) & 0xff));
        result.setC((short) (buffer.get(8) & 0xff));
        result.setD((short) (buffer.get(9) & 0xff));

        return result;
    }

    public static byte[] buildSendPermissionCommand(int code) {
        ByteBuffer buffer = ByteBuffer.allocate(7).order(ByteOrder.LITTLE_ENDIAN);

        buffer.put((byte) 0xff);
        buffer.put((byte) 0x01);
        buffer.put((byte) 0x02);
        buffer.put((byte) (code & 0xff));
        buffer.put((byte) ((code >> 8) & 0xff));
        buffer.put((byte) ((code >> 16) & 0xff));
        buffer.put((byte) ((code >> 24) & 0xff));

        return buffer.array();
    }

    public static BLESendPermissionCodeCommandResult buildSendPermissionResult(byte[] data) {
        BLESendPermissionCodeCommandResult result = new BLESendPermissionCodeCommandResult();
        result.setSuccess(data[3]);
        result.setCode(data[4]);
        return result;
    }

    public static byte[] buildWriteDeviceCodeCommand(String id) {
        ByteBuffer buffer = ByteBuffer.allocate(8);
        buffer.put((byte) 0x03);
        buffer.put((byte) 0x12);
        buffer.put(id.getBytes());

        return buffer.array();
    }

    public static BLEWriteDeviceCodeCommandResult buildWriteDeviceCodeResult(byte[] data) {
        BLEWriteDeviceCodeCommandResult result = new BLEWriteDeviceCodeCommandResult();
        result.setStatus(data[2]);
        if (data.length == 4) result.setCode(data[3]);
        return result;
    }

    public static byte[] buildWriteDeviceIdCommand(String id) {
        ByteBuffer buffer = ByteBuffer.allocate(20);
        buffer.put((byte) 0x03);
        buffer.put((byte) 0x11);
        buffer.put(id.getBytes());

        return buffer.array();
    }

    public static BLEWriteDeviceIdCommandResult buildWriteDeviceIdResult(byte[] data) {
        BLEWriteDeviceIdCommandResult result = new BLEWriteDeviceIdCommandResult();
        result.setStatus(data[2]);
        if (data.length == 4) result.setCode(data[3]);
        return result;
    }
}
