package hk.advanpro.android.sdk.device.protocol;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import hk.advanpro.android.sdk.device.result.ble.shirt.BLEShirtHeartRateEventResult;
import hk.advanpro.android.sdk.device.result.ble.shirt.BLEShirtRealSensorEventResult;

/**
 * 作者：jalin on 2017/12/25 12:50
 * 邮箱：zhouxiaolin@advanpro.com
 */

public class BLEShirtProtocol {
    /**
     * 获取Fss 实际电阻
     *
     * @param data 5bytes(1byte FSS_RANGE + 4bytes R0;
     *             第 1Byte 为保留位；第 2~5Bytes 共 4Bytes 用于表示 FSS
     *             串接的电阻值 R0， 低位在前高位在后，无符号数，单位为Ω ；
     *             用于计算 FSS 实际电阻值;
     *             计算公式：
     *             RFSS = R0 × Dn ÷ (65536-Dn)
     * @return Fss 实际电阻
     */
    public static int buildFssRange(byte[] data) {
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        if (buffer.remaining() == 5) {
            buffer.position(1);
            int rangeVal = buffer.getInt();
            return rangeVal;
        }
        return 0;
    }

    /**
     * 构建心电率数据
     *
     * @param data data
     * @return 心电率数据
     */
    public static BLEShirtHeartRateEventResult buildHeartRateEventResult(byte[] data) {
        BLEShirtHeartRateEventResult result = new BLEShirtHeartRateEventResult();
        if (data.length == 20) {
            // 心电正确连接
            result.setEnable(true);
            short[] hearts = new short[10];
            ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
            for (int i = 0; buffer.remaining() > 0; i++) {
                hearts[i] = buffer.getShort();
            }
            result.setHearts(hearts);

        } else if (data.length == 1 && data[0] == 0x04) {
            // 心电电极未正确连接
            result.setEnable(false);
            result.setHearts(new short[10]);
        }
        return result;
    }

    public static BLEShirtRealSensorEventResult buildRealSensorEventResult(int fssRange, byte[] data) {
        BLEShirtRealSensorEventResult result = new BLEShirtRealSensorEventResult();
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        double[] fss1 = new double[5];
        double[] fss2 = new double[5];
        for (int i = 0; buffer.remaining() > 0; i++) {
            double f1 = buffer.getShort();
            double f2 = buffer.getShort();

            // 计算公式： RFSS = R0 × Dn ÷ (65536-Dn)
            fss1[i] = (fssRange * f1) / (65536 - f1);
            fss2[i] = (fssRange * f2) / (65536 - f2);
        }
        result.setRange(fssRange);
        result.setFss1(fss1);
        result.setFss2(fss2);
        return result;
    }
}
