package hk.advanpro.android.sdk.device.result.ble;

import android.bluetooth.BluetoothDevice;
import android.content.Context;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.ble.BLEAdvanproPiDevice;
import hk.advanpro.android.sdk.device.ble.BLEBeltDevice;
import hk.advanpro.android.sdk.device.ble.BLEDevice;
import hk.advanpro.android.sdk.device.ble.BLEInsoleDevice;
import hk.advanpro.android.sdk.device.ble.BLEShirtDevice;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.result.DeviceScanResult;

/**
 * 低耗功蓝牙设备扫描信息。
 * Created by yejianfei on 2017/6/7.
 */

public class BLEDeviceScanResult extends DeviceScanResult {

    private static final String TAG = BLEDeviceScanResult.class.getSimpleName();

    private Context context;

    private ScanRecord record;

    private BluetoothDevice bluetoothDevice;

    private BLEDevice device;

    private int error;

    public BLEDeviceScanResult(Context context, ScanRecord record, BluetoothDevice bluetoothDevice) {
        this.context = context;
        this.record = record;
        this.bluetoothDevice = bluetoothDevice;
    }

    public BLEDeviceScanResult(int error) {
        this.error = error;
    }

    public Context getContext() {
        return context;
    }

    public ScanRecord getRecord() {
        return record;
    }

    public BluetoothDevice getBluetoothDevice() {
        return bluetoothDevice;
    }

    public int getError() {
        return error;
    }

    @Override
    public synchronized Device create() {
        if (device == null) {
            DeviceType type = record.getManufacturer().getType();

            switch (type) {
                case Belt:
                    device = new BLEBeltDevice();
                    break;
                case Insole:
                    device = new BLEInsoleDevice();
                    break;
                case SDP:
                case SDP_Insole:
                    device = new BLEAdvanproPiDevice();
                    break;
                case HeartShirt:
                case BreathShirt:
                    device = new BLEShirtDevice();
                    break;
                default:
                    throw new UnsupportedOperationException();
            }

            device.open(this);
            ALog.d(TAG, String.format("create and open new device instance of %s %s.", bluetoothDevice.getAddress(), this.getBluetoothDevice().getName()));
        }
        return device;
    }

    @Override
    public String getAddress() {
        return record.getAddress();
    }

    public static class ScanRecordManufacturer {

        private String serial;//序列号
        private String company;//公司
        private DeviceType type;//产品大类
        private DeviceMode mode;//固件模式
        private short version;//区分第几代产品
        private short major;//代内小版本
        private String address;

        private HashMap<String, String> description;

        /**
         * 序列号
         *
         * @return 序列号
         */
        public String getSerial() {
            return serial;
        }

        public void setSerial(String serial) {
            this.serial = serial;
        }

        /**
         * 公司标示
         *
         * @return 公司标示
         */
        public String getCompany() {
            return company;
        }

        public void setCompany(String company) {
            this.company = company;
        }


        /**
         * 产品大类标示
         *
         * @return 产品大类标示
         */
        public DeviceType getType() {
            return type;
        }

        public void setType(DeviceType type) {
            this.type = type;
        }

        /**
         * 固件模式
         *
         * @return 固件模式
         */
        public DeviceMode getMode() {
            return mode;
        }

        public void setMode(DeviceMode mode) {
            this.mode = mode;
        }

        /**
         * 区分第几代产品
         *
         * @return 区分第几代产品
         */
        public short getVersion() {
            return version;
        }

        public void setVersion(short version) {
            this.version = version;
        }

        /**
         * 代内小版本
         * 用于区分同类型设备属于第几代设备
         *
         * @return 代内小版本
         */
        public short getMajor() {
            return major;
        }

        public void setMajor(short major) {
            this.major = major;
        }

        /**
         * mac地址
         *
         * @return mac地址
         */
        public String getAddress() {
            return address;
        }

        public void setAddress(String address) {
            this.address = address;
        }

        /**
         * 设备其他描述
         *
         * @return 设备其他描述
         */
        public HashMap<String, String> getDescription() {
            return description;
        }

        public void setDescription(HashMap<String, String> description) {
            this.description = description;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            ScanRecordManufacturer that = (ScanRecordManufacturer) o;

            if (getCompany() != null ? !getCompany().equals(that.getSerial()) : that.getCompany() != null)
                if (getType() != that.getType()) return false;
            if (getVersion() != that.getVersion()) return false;
            if (getMajor() != that.getMajor()) return false;
            if (getMode() != that.getMode()) return false;
            if (getSerial() != null ? !getSerial().equals(that.getSerial()) : that.getSerial() != null)
                return false;

            boolean result = true;
            Iterator<Map.Entry<String, String>> thisDes = getDescription().entrySet().iterator();
            while (thisDes.hasNext()) {
                Map.Entry<String, String> entry1 = thisDes.next();
                String m1value = entry1.getValue() == null ? "" : entry1.getValue();
                String m2value = that.getDescription().get(entry1.getKey()) == null ? "" : getDescription().get(entry1.getKey());
                if (!m1value.equals(m2value)) {//若两个map中相同key对应的value不相等
                    result = false;
                    break;
                }
            }
            return result;

        }

        @Override
        public int hashCode() {
            int result = getCompany().hashCode();
            result = 31 * result + getType().getValue();
            result = 31 * result + getVersion();
            result = 31 * result + getMajor();
            result = 31 * result + getMode().hashCode();
            result = 31 * result + getSerial().hashCode();
            result = 31 * result + getDescription().hashCode();
            return result;
        }
    }

    public static class ScanRecord {
        private short uuid;
        private int rssi;
        private String localName;

        private String address;

        private ScanRecordManufacturer manufacturer;

        public short getUuid() {
            return uuid;
        }

        public void setUuid(short uuid) {
            this.uuid = uuid;
        }

        /**
         * 设备名称
         *
         * @return 设备名称
         */
        public String getLocalName() {
            return localName;
        }

        public void setLocalName(String localName) {
            this.localName = localName;
        }

        public int getRssi() {
            return rssi;
        }

        public void setRssi(int rssi) {
            this.rssi = rssi;
        }

        /**
         * MAC地址
         *
         * @return MAC地址
         */
        public String getAddress() {
            return address;
        }

        public void setAddress(String address) {
            this.address = address;
        }

        /**
         * 设备制造商
         *
         * @return 设备制造商
         */
        public ScanRecordManufacturer getManufacturer() {
            return manufacturer;
        }

        public void setManufacturer(ScanRecordManufacturer manufacturer) {
            this.manufacturer = manufacturer;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            ScanRecord record = (ScanRecord) o;

            if (getUuid() != record.getUuid()) return false;
            if (getLocalName() != null ? !getLocalName().equals(record.getLocalName()) : record.getLocalName() != null)
                return false;
            if (getAddress() != null ? !getAddress().equals(record.getAddress()) : record.getAddress() != null)
                return false;
            return getManufacturer() != null ? getManufacturer().equals(record.getManufacturer()) : record.getManufacturer() == null;

        }

        @Override
        public int hashCode() {
            int result = (int) getUuid();
            result = 31 * result + (getLocalName() != null ? getLocalName().hashCode() : 0);
            result = 31 * result + (getAddress() != null ? getAddress().hashCode() : 0);
            result = 31 * result + (getManufacturer() != null ? getManufacturer().hashCode() : 0);
            return result;
        }
    }
}
